SystemOrganization addCategory: #'DGV-Development Stuff'!
SystemOrganization addCategory: #'DGV-Model'!
SystemOrganization addCategory: #'DGV-Model-Exceptions'!
SystemOrganization addCategory: #'DGV-Monitoring'!
SystemOrganization addCategory: #'DGV-Network-Service'!
SystemOrganization addCategory: #'DGV-OS-Support'!
SystemOrganization addCategory: #'DGV-Tests'!

!UUID class methodsFor: '*dgv' stamp: 'EG 6/20/2005 16:28'!
fromSHAHash: aLargePositiveInteger 
	| str uuidString |
	str _ aLargePositiveInteger printStringHex asLowercase.
	uuidString _ (str copyFrom: 1 to: 8), '-', (str copyFrom: 9 to: 12), '-', (str copyFrom: 13 to: 16), '-', (str copyFrom: 17 to: 20), '-', (str copyFrom: 21 to: 32).
	^self fromString: uuidString! !

!UUID methodsFor: '*dgv-model' stamp: 'CdG 3/25/2005 20:38'!
asString36
	"Encode the UUID as a base 36 string using 0-9 and lowercase a-z.
	This is the shortest representation still being able to work as
	filenames etc since it does not depend on case nor characters
	that might cause problems."

	| candidate num |
	num _ 0.
	1 to: self size do: [:i | num _ num + ((256 raisedTo: i - 1) * (self at: i))].
	candidate _ num printStringBase: 36.
	^((candidate copyFrom: 1 to: 3) = '36r' 
		ifTrue: [candidate copyFrom: 4 to: candidate size]
		ifFalse: [candidate]) 
		asLowercase! !

!UUID methodsFor: '*dgv-model' stamp: 'CdG,TJ 2/3/2005 11:22'!
isUUID
	^true! !

AWGSingletonNode subclass: #DGVNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVNode class methodsFor: 'as yet unclassified' stamp: 'CdG 12/16/2005 00:11'!
default
	TheInstance ifNil: 
		[super default.
		self startServices].
	^TheInstance! !

!DGVNode class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
peerListConfiguration
	^DGVApplicationConfiguration peerListConfiguration! !

!DGVNode class methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 15:06'!
shutdown
	self stopServices.
	super shutdown! !

!DGVNode class methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:33'!
startServices
	| svc |

	"Starts basic services that every node should run"
	DGVStatisticsWebServer start.

	"Start services persisted on disk"
	DGVService start.

	"Start services when not loaded from disk"
	{DGVCollectionService. DGVQueryService. DGVCleanerService} do: [:each |
		svc _ DGVService serviceWithName: each serviceName.
		svc ifNil: [each createService]]! !

!DGVNode class methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:33'!
stopServices
	DGVStatisticsWebServer stop.

	DGVService stop! !

Error subclass: #DGVException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

DGVException subclass: #DGVDuplicateCommunityException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVDuplicateCommunityException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:15'!
errorMessage
	^'Een community met deze naam bestaat al: ' translated! !

DGVException subclass: #DGVEmptyDescriptionException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVEmptyDescriptionException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:21'!
errorMessage
	^'Omschrijving van een community mag niet leeg zijn.' translated! !

!DGVEmptyDescriptionException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:23'!
messageText
	^''! !

DGVException subclass: #DGVEmptyNameException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVEmptyNameException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:24'!
errorMessage
	^'De naam van een community mag niet leeg zijn.' translated! !

!DGVEmptyNameException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:23'!
messageText
	^''! !

!DGVException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:14'!
description
	^self errorMessage 
		, (self messageText ifNil: ['Niet gespecificeerd.' translated])! !

!DGVException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:14'!
errorMessage
	^'Algemene fout: ' translated! !

DGVException subclass: #DGVFileLockedException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVFileLockedException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:27'!
errorMessage
	^'Dit bestand kan niet worden geopend omdat het in gebruik is: ' translated! !

DGVException subclass: #DGVInvalidEmailAddressException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVInvalidEmailAddressException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:31'!
errorMessage
	^'Dit is geen geldig emailadres: ' translated! !

DGVException subclass: #DGVLinkCouldNotBeResolvedException
	instanceVariableNames: 'link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVLinkCouldNotBeResolvedException methodsFor: 'accessing' stamp: 'EG 12/14/2005 14:36'!
errorMessage
	^ 'Dit document kon niet van netwerk worden gehaald: ' translated! !

!DGVLinkCouldNotBeResolvedException methodsFor: 'testing' stamp: 'CdG 3/31/2005 11:36'!
isResumable
	^true! !

!DGVLinkCouldNotBeResolvedException methodsFor: 'accessing' stamp: 'CdG 3/30/2005 17:50'!
link
	^link! !

!DGVLinkCouldNotBeResolvedException methodsFor: 'accessing' stamp: 'CdG 3/30/2005 17:50'!
link: anObject
	link := anObject! !

!DGVLinkCouldNotBeResolvedException methodsFor: 'accessing' stamp: 'EG 12/14/2005 14:42'!
messageText
	^link asString! !

DGVException subclass: #DGVServiceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model-Exceptions'!

!DGVServiceException methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:34'!
errorMessage
	^'Deze service is tijdelijk niet op het netwerk beschikbaar: '! !

Object subclass: #DGVApplicationConfiguration
	instanceVariableNames: ''
	classVariableNames: 'Configuration'
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVApplicationConfiguration class methodsFor: 'api' stamp: ''!
changesFilename
	^ self instance changesFilename! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:34'!
communityDescription
	^self instance communityDescription! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:33'!
communitySearchClass
	^self instance communitySearchClass! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:35'!
configName
	^self instance configName! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:33'!
fileClass
	^self instance fileClass! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:33'!
fileListPresenterClass
	^self instance fileListPresenterClass! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:32'!
helpMenuClass
	^self instance helpMenuClass! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: ''!
imageFilename
	^ self instance imageFilename! !

!DGVApplicationConfiguration class methodsFor: 'accessing' stamp: 'JJ 10/18/2005 12:28'!
instance
	Configuration isNil ifTrue: [^self error: 'No configuration set'].
	^Configuration! !

!DGVApplicationConfiguration class methodsFor: 'test-support' stamp: 'JJ 10/18/2005 12:25'!
instance: anApplicationConfiguration
	Configuration := anApplicationConfiguration! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:32'!
newFileEditorDialogClass
	^self instance newFileEditorDialogClass! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:54'!
nsisScriptName
	^self instance nsisScriptName! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:33'!
peerListConfiguration
	^self instance peerListConfiguration! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:32'!
rootCommunityDescription
	^self instance rootCommunityDescription! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:32'!
rootCommunityId
	^self instance rootCommunityId! !

!DGVApplicationConfiguration class methodsFor: 'private' stamp: 'JJ 10/18/2005 12:20'!
setConfigurationFor: anApplicationConfiguration

	Configuration := anApplicationConfiguration new! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:31'!
showFileTabWhenNotMember
	^self instance showFileTabWhenNotMember! !

!DGVApplicationConfiguration class methodsFor: 'api' stamp: 'CdG 11/29/2005 11:32'!
toolbarSearchClass
	^self instance toolbarSearchClass! !

!DGVApplicationConfiguration methodsFor: 'accessing' stamp: 'JJ 10/18/2005 14:12'!
configName
	^'Kolibri Dummy Configuration'! !

!DGVApplicationConfiguration methodsFor: 'printing' stamp: 'JJ 10/18/2005 09:13'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self configName;
		nextPut: $)! !

Object subclass: #DGVBlobInfo
	instanceVariableNames: 'id version checksum dataSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVBlobInfo commentStamp: 'CdG 3/10/2005 11:39' prior: 0!
Info on a blob. Used for transporting data about a blob in preparation of a transfer!

!DGVBlobInfo class methodsFor: 'instance creation' stamp: 'CdG 3/11/2005 11:20'!
for: aDGVBlob
	^self new for: aDGVBlob! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/11/2005 10:58'!
calculateChecksumForData: aByteArray 
	^SecureHashAlgorithm new hashMessage: aByteArray! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:39'!
checksum
	^checksum! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:39'!
checksum: anObject
	checksum := anObject! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/11/2005 11:01'!
checkValidityOfData: aByteArray
	^(self dataSize = aByteArray size) 
		and: [self checksum = (self calculateChecksumForData: aByteArray)]! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:40'!
dataSize
	^dataSize! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:40'!
dataSize: anObject
	dataSize := anObject! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/11/2005 10:58'!
data: aByteArray 
	dataSize := aByteArray size.
	checksum := self calculateChecksumForData: aByteArray! !

!DGVBlobInfo methodsFor: 'initialize-release' stamp: 'EG,CdG 7/1/2005 11:11'!
for: aDGVBlob
	| bytes stableCopy |
	stableCopy _ aDGVBlob stableCopyForChecksumming.
	bytes _ stableCopy marshallAsByteArray.
	self
		id: stableCopy id;
		version: stableCopy version;
		data: bytes.
! !

!DGVBlobInfo methodsFor: 'comparing' stamp: 'CdG 3/10/2005 12:01'!
hash
	^id hash bitXor: (version hash bitXor: (dataSize hash bitXor: checksum hash))! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:39'!
id
	^id! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:39'!
id: anObject
	id := anObject! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:39'!
version
	^version! !

!DGVBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/10/2005 11:39'!
version: anObject
	version := anObject! !

!DGVBlobInfo methodsFor: 'comparing' stamp: 'CdG 3/10/2005 12:00'!
= anObject
	anObject class ~= self class ifTrue: [^false].
	^id = anObject id and: [
		version = anObject version and: [
			dataSize = anObject dataSize and: [
				checksum = anObject checksum]]]! !

Object subclass: #DGVBlobTransfer
	instanceVariableNames: 'transferName query asynchronousDownloadProcess targetId selectedBlobInfo busyList workNodes todoList doneList receivedResponses mutex workDoneSemaphore blobInfoFactory blobBlockFactory completedResult lastTimeOut fileName listeners'
	classVariableNames: 'ActiveBlocksPerNode'
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

DGVBlobTransfer class
	instanceVariableNames: 'blobInfoFactory timeout'!

!DGVBlobTransfer commentStamp: 'CdG 3/10/2005 11:45' prior: 0!
This thing transfers blobs with a swarm-style download!

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 15:31'!
activeBlocksPerNode
	^ActiveBlocksPerNode ifNil: [ActiveBlocksPerNode := self defaultActiveBlocksPerNode]! !

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 15:46'!
activeBlocksPerNode: aNumber
	ActiveBlocksPerNode := aNumber! !

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 12:49'!
blockSize
	^10240! !

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'CdG 4/1/2005 18:36'!
debug: aBlock
	"WorldState addDeferredUIMessage: aBlock fixTemps value"! !

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 17:10'!
defaultActiveBlocksPerNode
	^3! !

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 11:46'!
get: aUUID
	^self new
		get: aUUID! !

!DGVBlobTransfer class methodsFor: 'as yet unclassified' stamp: 'CdG 4/1/2005 18:38'!
maximumTimeForSingleBlockFetch
	^5 seconds! !

!DGVBlobTransfer methodsFor: 'accessing' stamp: 'JJ 12/5/2005 11:23'!
addListener: aListener
	self listeners add: aListener! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 4/1/2005 15:54'!
assignWork
	| node block | 
	[self hasFreeBlocks] whileTrue: [
			self hasFreeNodes ifFalse: [^self].
			self critical: [
				node := self selectFreeNode.
				block := self todoList anyOne.
				self assignWork: block to: node.
				self startRequestForBlock: block]		
			]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 4/1/2005 14:39'!
assignWork: block to: node 
	self workNodes at: node put: ((self workNodes at: node ifAbsent: [0]) + 1).
	self todoList remove: block.
	self busyList at: block put: TimeStamp now -> node.
! !

!DGVBlobTransfer methodsFor: 'testing' stamp: 'EG,CdG 7/7/2005 13:57'!
blobBlockFactory
	^ blobBlockFactory
		ifNil: [^ blobBlockFactory := DGVQueryGetBlobBlock]! !

!DGVBlobTransfer methodsFor: 'testing' stamp: 'EG,CdG 7/7/2005 13:59'!
blobBlockFactory: factory
	blobBlockFactory _ factory
! !

!DGVBlobTransfer methodsFor: 'testing' stamp: 'EG,CdG 7/7/2005 11:24'!
blobInfoFactory
	^ blobInfoFactory ifNil: [^blobInfoFactory _ DGVQueryGetBlobInfo]! !

!DGVBlobTransfer methodsFor: 'testing' stamp: 'EG,CdG 7/7/2005 10:58'!
blobInfoFactory: factory 
	blobInfoFactory := factory! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 3/11/2005 10:48'!
blockCount
	| answer |
	answer _ (selectedBlobInfo dataSize / self class blockSize) asInteger.
	^(selectedBlobInfo dataSize \\ self class blockSize) > 0
		ifTrue: [answer + 1]
		ifFalse: [answer]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'EG,CdG 7/6/2005 17:58'!
buildResult
	| resultstream answer |
	resultstream _ WriteStream on: ByteArray new.
	1 to: self doneList size do: [:each |
		resultstream nextPutAll: (self doneList at: each)].
	answer _ resultstream contents.
	^(selectedBlobInfo checkValidityOfData: answer)
		ifTrue: [DGVBlob fromByteArray: answer]
		ifFalse: [nil "we failed. And currently no means to repair"]
! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'CdG 3/10/2005 12:23'!
busyList 
	^busyList ifNil: [busyList _ Dictionary new].! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'CdG 7/7/2005 11:28'!
cancelAsynchronousDownload
	asynchronousDownloadProcess ifNotNil: 
			[asynchronousDownloadProcess terminate.
			asynchronousDownloadProcess := nil]! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'CdG 7/7/2005 11:02'!
completedBlocksString
	doneList ifNil: [^'0'].
	^doneList size printString! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 3/10/2005 12:27'!
createTodoList
	selectedBlobInfo ifNil: [^self].
	1 to: self blockCount do: [:i | self todoList add: i]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 11/21/2005 21:31'!
critical: aBlock
	mutex ifNil: [mutex _ Mutex new].
	^mutex critical: aBlock! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'CdG 3/10/2005 12:43'!
doneList
	^doneList ifNil: [doneList _ Dictionary new]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'EG,CdG 7/6/2005 17:05'!
downloadIncomplete
	^self hasFreeNodes and: [self hasFreeBlocks]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'EG,CdG 7/6/2005 17:07'!
downloadRun
	self updateNodeList.
	"self expireBusyList."
	(self hasFreeNodes and: [self hasFreeBlocks]) 
		ifTrue: 
			[self workDoneSemaphore initSignals.
			self assignWork]
		ifFalse: [self workDoneSemaphore waitTimeoutMSecs: 50]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 4/1/2005 15:43'!
expireBusyList
	| startTime node |
	self busyList keysAndValuesDo: [:key :value |
		startTime _ value key.
		node _ value value.
		(startTime < (TimeStamp now - self class maximumTimeForSingleBlockFetch)) ifTrue: [
			self critical: [
				self freeNode: node andBlock: key.
				self todoList add: key]]]! !

!DGVBlobTransfer methodsFor: 'accessing' stamp: 'EG,CdG 10/13/2005 15:51'!
fileName
	^fileName! !

!DGVBlobTransfer methodsFor: 'accessing' stamp: 'EG,CdG 10/13/2005 15:51'!
fileName: anObject
	fileName := anObject! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 4/1/2005 14:48'!
freeNode: targetNode andBlock: aNumber 
	self busyList removeKey: aNumber ifAbsent: nil.
	self workNodes at: targetNode put:  ((self workNodes at: targetNode) - 1)! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'CdG 7/7/2005 15:39'!
getAsynchronously: aUUID 
	asynchronousDownloadProcess := [
		completedResult _ self get: aUUID] fork.
	asynchronousDownloadProcess name: 'Blob transfer of ' , aUUID printString! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 10/26/2005 16:01'!
get: aUUID 
	| answer |
	targetId := aUUID.
	[selectedBlobInfo isNil]
		whileTrue: [
			query _ self queryBlobInfo.
			(Delay forDuration: self timeout) wait.
			query terminateSearch.
			self selectBlobInfoToUse].
	[self hasWorkTodo]
		whileTrue: [self downloadRun].
	answer := self buildResult.
	answer
		ifNotNil: [answer updateLastSavedTime.
			DGVBlob localStorage put: answer].
	DGVNodeStatistics forDefaultNode incrementTotalBlobDownloads; save.
	^ answer! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 3/10/2005 12:29'!
hasFreeBlocks
	^self todoList isEmpty not! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 4/1/2005 14:41'!
hasFreeNodes
	self workNodes values detect: [:each | each < self class activeBlocksPerNode] ifNone:[^false].
	^ true! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'EG,CdG 10/14/2005 14:28'!
hash
	^self fileName hash bitXor: self transferName hash! !

!DGVBlobTransfer methodsFor: 'testing' stamp: 'CdG 3/11/2005 10:19'!
hasWorkTodo
	^(self doneList size = self blockCount) not! !

!DGVBlobTransfer methodsFor: 'testing' stamp: 'CdG 7/7/2005 11:47'!
isComplete
	^completedResult notNil! !

!DGVBlobTransfer methodsFor: 'accessing' stamp: 'JJ 12/5/2005 11:23'!
listeners
	listeners isNil ifTrue: [listeners := OrderedCollection new].
	^listeners! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'EG,CdG 10/13/2005 15:50'!
osOpen
	self isComplete ifFalse: [^self].
	DGVOSSupport default openTempFileWithName: fileName contents: completedResult data! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'JJ 12/2/2005 11:42'!
percentageCompleteString
	doneList ifNil: [^'0'].
	^((doneList size / self blockCount) * 100) asFloat printShowingDecimalPlaces:2! !

!DGVBlobTransfer methodsFor: 'networking' stamp: 'CdG 7/11/2005 12:00'!
queryBlobInfo
	^self blobInfoFactory for: targetId
		whenFound: [:result | self receivedQueryBlobInfoResult: result]! !

!DGVBlobTransfer methodsFor: 'networking' stamp: 'EG,CdG 7/7/2005 13:56'!
queryBlockNumber: aNumber fromNode: targetNode 
	| result |
	result := self blobBlockFactory
				get: selectedBlobInfo
				block: aNumber
				atNode: targetNode
				timeOut: self class maximumTimeForSingleBlockFetch.
	^ result! !

!DGVBlobTransfer methodsFor: 'networking' stamp: 'CdG 7/11/2005 12:00'!
receivedQueryBlobInfoResult: anAssociation 
	| info nodeId |
	info := anAssociation key.
	nodeId := anAssociation value.
	((info isKindOf: DGVBlobInfo) and: [nodeId isKindOf: UUID]) 
		ifTrue: [self receivedResponses at: nodeId put: info]! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'TJ 3/10/2005 15:06'!
receivedResponses
	^ receivedResponses ifNil: [receivedResponses _ Dictionary new]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'TJ 3/11/2005 09:06'!
selectBlobInfoToUse
	| tally maxCount maxKey |
	tally _ Dictionary new.
	self receivedResponses values do: [: each |
		tally at: each put: (tally at: each ifAbsentPut: 0) + 1].
	maxCount _ 0.
	maxKey _ nil.
	tally keys do: [:left |
		tally keys do: [:right |
			(left id = right id and: [left version < right version]) ifTrue: [
				tally removeKey: left]]].
	tally keys do: [:each |
		(tally at: each) > maxCount ifTrue: [
			maxKey _ each.
			maxCount _ tally at: each]].
	self selectedBlobInfo: (tally isEmpty 
		ifTrue: [nil]
		ifFalse: [maxKey]).
	self createTodoList.
		! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'TJ 3/10/2005 15:34'!
selectedBlobInfo
	^selectedBlobInfo! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'TJ 3/10/2005 15:34'!
selectedBlobInfo: anObject
	selectedBlobInfo := anObject! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'CdG 7/7/2005 11:04'!
selectedNodesString
	workNodes ifNil: [^'0'].
	^workNodes size printString! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 4/1/2005 16:06'!
selectFreeNode
	^(self workNodes associations 
			select: [:assoc | assoc value < self class activeBlocksPerNode]) 
			atRandom key! !

!DGVBlobTransfer methodsFor: 'networking' stamp: 'JJ 12/5/2005 11:24'!
startRequestForBlock: aNumber 
	| targetNode result blockQuery validResult data process |
	targetNode := (self busyList at: aNumber) value.
	
	process _ [
		result := self queryBlockNumber: aNumber fromNode: targetNode.
		self critical: 
			[result ifNil: [self todoList add: aNumber]
				ifNotNil: 
					[blockQuery := result key.
					data := result value.
					validResult := blockQuery blobInfo = selectedBlobInfo 
								and: [blockQuery blockNumber = aNumber].
					validResult 
						ifTrue: [
							self doneList at: aNumber put: data.
							self updateListeners]
						ifFalse: [self todoList add: aNumber].
					self workDoneSemaphore signal].
			self freeNode: targetNode andBlock: aNumber]] 
			fork.
	process name: 'Get block ', aNumber printString, ' of ' , targetId printString
! !

!DGVBlobTransfer methodsFor: 'networking' stamp: 'EG,CdG 9/2/2005 16:12'!
timeout
	| last new |
	last := lastTimeOut
				ifNil: [lastTimeOut := 5 seconds].
	new := last * 2.
	new > 2 hours ifTrue: [new := 2 hours].
	^lastTimeOut := new! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'CdG 3/10/2005 12:24'!
todoList 
	^todoList ifNil: [todoList _ Set new].! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 17:20'!
totalBlocksString
	^selectedBlobInfo 
		ifNil: ['?']
		ifNotNil: [self blockCount printString]! !

!DGVBlobTransfer methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 16:48'!
totalNodesString
	^self receivedResponses size printString! !

!DGVBlobTransfer methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 16:25'!
transferName
	^transferName! !

!DGVBlobTransfer methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 16:25'!
transferName: anObject
	transferName := anObject! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'JJ 12/5/2005 11:24'!
updateListeners
	self listeners do: [:each | each updateBlobTransfer: self]! !

!DGVBlobTransfer methodsFor: 'private' stamp: 'CdG 3/10/2005 12:19'!
updateNodeList
	| validNodes |
	validNodes _ self receivedResponses associations select: [:assoc |
		assoc value = selectedBlobInfo].
	validNodes do: [:assoc |
		self workNodes at: assoc key ifAbsentPut: [0].
		self receivedResponses removeKey: assoc key]. ! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'CdG 4/1/2005 16:04'!
workDoneSemaphore
	^workDoneSemaphore ifNil: [workDoneSemaphore _ Semaphore new]! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'CdG 3/10/2005 12:23'!
workNodes 
	^workNodes ifNil: [workNodes _ Dictionary new].! !

!DGVBlobTransfer methodsFor: 'private-accessing' stamp: 'EG,CdG 10/14/2005 14:25'!
= anObject
	(anObject class = self class) ifFalse: [^false].
	^self fileName = anObject fileName and: [ self transferName = anObject transferName]! !

Object subclass: #DGVCollectionElement
	instanceVariableNames: 'collectionId elementId elementClass addition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVCollectionElement class methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:02'!
collId: anUUID elementId: anUUID2 elementClass: aClass
	^(self new)
		collectionId: anUUID;
		elementId: anUUID2;
		elementClass: aClass;
		makeAddition
	! !

!DGVCollectionElement class methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:02'!
removeFromCollId: anUUID elementId: anUUID2 elementClass: aClass
	^(self new)
		collectionId: anUUID;
		elementId: anUUID2;
		elementClass: aClass;
		makeRemoval
	! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/22/2005 10:44'!
collectionId
	^collectionId! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/22/2005 10:42'!
collectionId: aUUID
	collectionId _ aUUID! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/22/2005 15:24'!
elementClass
	^elementClass! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/22/2005 15:24'!
elementClass: class
	elementClass _ class! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/22/2005 10:44'!
elementId
	^elementId! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/22/2005 10:43'!
elementId: aUUID
	elementId _ aUUID! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'CdG 11/25/2005 13:58'!
hash
	^self collectionId hash bitXor: (self elementId hash bitXor: self isAddition hash)! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:00'!
isAddition
	^ addition ifNil: [ addition _ true ]! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 15:58'!
isRemoval
	^ self isAddition not! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:00'!
makeAddition
	addition _ true! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:00'!
makeRemoval
	addition _ false! !

!DGVCollectionElement methodsFor: 'as yet unclassified' stamp: 'CdG 11/25/2005 13:58'!
= anObject
	^anObject class = self class and: [
		anObject collectionId = self collectionId and: [
			anObject elementId = self elementId and: [
				anObject isAddition = self isAddition]]]! !

Object subclass: #DGVConfiguration
	instanceVariableNames: 'inifile filename'
	classVariableNames: 'DefaultConfiguration'
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVConfiguration class methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 12:15'!
default
	^ DefaultConfiguration ifNil: [DefaultConfiguration _ self new filename: 'dgv.ini']! !

!DGVConfiguration class methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 12:02'!
initialize
	Smalltalk addToStartUpList: self! !

!DGVConfiguration class methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 12:03'!
startUp
	DefaultConfiguration _ nil.! !

!DGVConfiguration methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 12:16'!
filename: aString
	filename _ aString.
	self readConfiguration! !

!DGVConfiguration methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/31/2005 11:50'!
getSection: section key: key default: default
	^ inifile section: section asLowercase at: key ifAbsent: default! !

!DGVConfiguration methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 13:05'!
readConfiguration
	| fs |
	inifile _ INIFile new.
	(FileDirectory default fileExists: filename) ifFalse: [^ self].
	fs _ (FileStream oldFileNamed: filename).
	[inifile _ INIFile readFrom: fs] 
		ensure: [fs close]! !

Object subclass: #DGVDownloadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVDownloadTest class methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 10:06'!
runTestParallel: aNumber

	| results |
	results := Dictionary new.
	1 to: aNumber do: [:parallel |
	  results at: parallel put: ( (1 to: 3) collect: [:i |
			|sem|
			sem := Semaphore new.
			Time millisecondsToRun: [
				1 to: parallel do: [:each |
					[DGVBlobTransfer 
						get: (UUID fromString: 'b908c81e-97cf-0545-a274-d1ed9eeb646f').
						sem signal] fork].
				1 to: parallel do: [:x | sem wait]]])].

	^results
    ! !

Object subclass: #DGVFriendlyGiant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVFriendlyGiant class methodsFor: 'as yet unclassified' stamp: 'JJ 11/9/2005 17:34'!
startUp
	DGVNode default.
	DGVPolicyConfirmUser createService.
	DGVPolicyOwnerConfirm createService.! !

Object subclass: #DGVInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

DGVInstaller class
	instanceVariableNames: 'errs everything'!

!DGVInstaller commentStamp: 'CdG 10/28/2005 14:24' prior: 0!
;This is dirty - we abuse the class comment so that we do not have
;to deal with escaping. Culprit: Cees ;)
;
!!define PRODUCT "Kolibri" ;Define your own software name here
!!define VERSION "@@VERSION@@" ;Define your own software version here

!!include "MUI.nsh"

;--------------------------------
;General

  ;Name and file
  Name "${PRODUCT}"
  OutFile "${PRODUCT}-${VERSION}-installer.exe"

  ;Folder selection page
  InstallDir "$PROGRAMFILES\${PRODUCT}"
  
  ;Remember install folder
  InstallDirRegKey HKCU "Software\${PRODUCT}" ""

;--------------------------------
;Variables

  Var MUI_TEMP
  Var STARTMENU_FOLDER

;--------------------------------
;Interface Settings

  !!define MUI_ABORTWARNING

;--------------------------------
;Pages

  !!insertmacro MUI_PAGE_LICENSE "License.rtf"
  !!insertmacro MUI_PAGE_COMPONENTS
  !!insertmacro MUI_PAGE_DIRECTORY
  
  ;Start Menu Folder Page Configuration
  !!define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKCU" 
  !!define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\${PRODUCT}" 
  !!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
  
  !!insertmacro MUI_PAGE_STARTMENU Application $STARTMENU_FOLDER
  
  !!insertmacro MUI_PAGE_INSTFILES

  !!define MUI_FINISHPAGE_LINK "Bezoek de Kolibri site voor meer informatie"
  !!define MUI_FINISHPAGE_LINK_LOCATION "http://www.digitaalverleden.nl/"

  !!define MUI_FINISHPAGE_RUN "$INSTDIR\wxSqueak.exe"
  !!define MUI_FINISHPAGE_RUN_PARAMETERS "-headless dgv.image"
  !!define MUI_FINISHPAGE_NOREBOOTSUPPORT
  
  !!insertmacro MUI_PAGE_FINISH

  !!insertmacro MUI_UNPAGE_CONFIRM
  !!insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages
 
  !!insertmacro MUI_LANGUAGE "Dutch"

;--------------------------------
;Installer Sections

Section "Kolibri Client" SecDummy

  SetOutPath "$INSTDIR"
  
  File "wxSqueak.exe"
  File "wxSqueak.ini"
  File "SqueakFFIPrims.dll"
  File "dgv.image"
  File "License.rtf"
  File "rootcommunity.uuid"

  SetOutPath "$INSTDIR\icons"
  File "icons\*.png" 
   SetOutPath "$INSTDIR"
  
  ;Store installation folder
  WriteRegStr HKCU "Software\${PRODUCT}" "" $INSTDIR
  
  ;Create uninstaller
  WriteUninstaller "$INSTDIR\Uninstall.exe"
  
  !!insertmacro MUI_STARTMENU_WRITE_BEGIN Application
    
  ;Create shortcuts
  CreateDirectory "$SMPROGRAMS\$STARTMENU_FOLDER"
  CreateShortCut "$DESKTOP\Kolibri.lnk" "$INSTDIR\wxSqueak.exe" "-headless dgv.image" "$INSTDIR\wxSqueak.exe" 10
  CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Kolibri.lnk" "$INSTDIR\wxSqueak.exe" "-headless dgv.image" "$INSTDIR\wxSqueak.exe" 10
  CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\License.lnk" "$INSTDIR\License.rtf"
  CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Uninstall.lnk" "$INSTDIR\Uninstall.exe"
  
  !!insertmacro MUI_STARTMENU_WRITE_END

SectionEnd

;Section "-Remove current application data" 
;
;	RmDir /r "$APPDATA\${PRODUCT}"
;
;SectionEnd

;--------------------------------
;Descriptions

  ;Language strings
  LangString DESC_SecDummy ${LANG_ENGLISH} "A test section."

  ;Assign language strings to sections
  !!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !!insertmacro MUI_DESCRIPTION_TEXT ${SecDummy} $(DESC_SecDummy)
  !!insertmacro MUI_FUNCTION_DESCRIPTION_END
 
;--------------------------------
;Uninstaller Section

Section "Uninstall"

  ;ADD YOUR OWN FILES HERE...

  Delete "$INSTDIR\wxSqueak.exe"
  Delete "$INSTDIR\wxSqueak.ini"
  Delete "$INSTDIR\SqueakFFIPrims.dll"
  Delete "$INSTDIR\dgv.image"
  Delete "$INSTDIR\License.rtf"  
  Delete "$INSTDIR\Uninstall.exe"
  Delete "$INSTDIR\rootcommunity.uuid"
  RMDir /r "$INSTDIR\icons"

  RMDir "$INSTDIR"
  
  !!insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP
    
  Delete "$SMPROGRAMS\$MUI_TEMP\Kolibri.lnk"
  Delete "$SMPROGRAMS\$MUI_TEMP\License.lnk"

  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"
  
  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"
 
  startMenuDeleteLoop:
    ClearErrors
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."
    
    IfErrors startMenuDeleteLoopDone
  
    StrCmp $MUI_TEMP $SMPROGRAMS startMenuDeleteLoopDone startMenuDeleteLoop
  startMenuDeleteLoopDone:

  DeleteRegKey /ifempty HKCU "Software\${PRODUCT}"

SectionEnd!

!DGVInstaller class methodsFor: 'building' stamp: 'CdG 12/14/2005 16:30'!
basicWorkspace
	"
		self basicWorkspace
	"
	Workspace new
		contents: '
DGVTestRunner open
DivaApplication launch
DGVNode default
DGVNode shutdown
Smalltalk garbageCollect
DGVNode allInstances
ProcessBrowser open';
		openLabel: 'DGV WorkBench[tm]'
	! !

!DGVInstaller class methodsFor: 'building' stamp: 'EG,JJ 10/25/2005 15:02'!
buildDailyBuildInstaller
	"self buildDailyBuildInstaller"

	self 
		buildInstallerWithVersion:  self versionIdentifier 
! !

!DGVInstaller class methodsFor: 'building' stamp: 'CdG 12/14/2005 16:30'!
buildImage
	"copy this script and paste/run in a fresh image"

"bootstrap. Start by manually loading MCConfigurations"
"note: we really want to use a version sorter or similar. Cf. MCRepositoryInspector>>sortedVersions"
| rep verName cfg |
FileStream fileIn: 'MCRepositories.st'.
rep := MCHttpRepository allInstances detect: [:each | 
	each locationWithTrailingSlash = 'http://dgv.aardworks.nl/mc/'].
#('Monticello-' 'MonticelloConfigurations-') do: [:each |
	verName := (rep allVersionNames select: [:each2 | each2 beginsWith: each]) last.
	(rep versionFromFileNamed: verName, '.mcz') load].
cfg := rep versionFromFileNamed: 'Cfg-Kolibri-Dev-CdG.2005112202.mcm'.
cfg updateFromRepositories; load.
cfg := rep versionFromFileNamed: 'Cfg-DivaWxUi-dailybuild-CdG.20051129.mcm'.
cfg updateFromRepositories; load.
DGVInstaller basicWorkspace! !

!DGVInstaller class methodsFor: 'private' stamp: 'CdG 11/29/2005 11:55'!
buildInstallerWithVersion: aString 
	| installScript fs scriptName |
	scriptName := self getNsisScriptName.
	installScript := self comment copyReplaceTokens: '@@VERSION@@'
				with: aString.
	fs := CrLfFileStream forceNewFileNamed: scriptName.
	[fs nextPutAll: installScript] ensure: [fs close].
	Win32Shell new 
		shellExecute: nil
		lpOperation: 'open'
		lpFile: 'C:\Program Files\NSIS\MakeNSISW.exe'
		lpParameters: self getNsisScriptName
		lpDirectory: nil
		nShowCmd: 1! !

!DGVInstaller class methodsFor: 'building' stamp: 'EG,JJ 10/25/2005 15:02'!
buildReleaseInstaller: aString
	"self buildReleaseInstaller"

	| version |
	version _ FillInTheBlank request: 'Version Number'.
	version ifEmpty: [^self].
	
	self 
		buildInstallerWithVersion: version ! !

!DGVInstaller class methodsFor: 'accessing' stamp: 'EG 12/14/2005 17:04'!
cleanUpRunningNode
	DGVApplication logOut.
	AWGSearch allInstances
		do: [:each | each terminateSearch].
	AWGRetryMessage cleanMessageDictionary.
	DGVNode shutdown.
	DGVCollection stopSignallingProcess.
	DGVTranslator selectDutch.
	DGVQueryGetBlobBlock resetCache.
	DGVStorage initializeRegistry.
	DGVApplicationConfiguration newFileEditorDialogClass resetLastDirectory.
	DGVBlobTransfer
		allInstancesDo: [:each | each cancelAsynchronousDownload].
	AWGNetworkStatistics instance reset.
	Smalltalk garbageCollect! !

!DGVInstaller class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
getChangesFilename
	^DGVApplicationConfiguration changesFilename! !

!DGVInstaller class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
getImageFilename
	^DGVApplicationConfiguration imageFilename! !

!DGVInstaller class methodsFor: 'accessing' stamp: 'CdG 11/29/2005 11:55'!
getNsisScriptName
	^DGVApplicationConfiguration nsisScriptName! !

!DGVInstaller class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
getRootCommunity
	^DGVApplicationConfiguration rootCommunityId! !

!DGVInstaller class methodsFor: 'qa' stamp: 'CdG 4/8/2005 00:57'!
isOurOwnStuff: aString
	"aString reprents a category or package. Answer true if it is one of ours"
	(aString = 'DGV-Development Stuff') ifTrue: [^false].
	(aString = 'Tric-Debugging Support') ifTrue: [^false].
	(aString endsWith: 'Tests') ifTrue: [^false].
	
	(aString beginsWith: 'DGV') ifTrue: [^true].
	(aString beginsWith: 'AardWorks') ifTrue: [^true].
	(aString beginsWith: 'Tric') ifTrue: [^true].	
	^false.! !

!DGVInstaller class methodsFor: 'building' stamp: 'EG,JJ 10/25/2005 15:03'!
prepareDailyBuildImage
	"
		KolApplicationConfiguration setConfigurationFor: DGVApplicationConfiguration.
		self prepareDailyBuildImage
	"
	self prepareImageWithVersion:  self versionIdentifier! !

!DGVInstaller class methodsFor: 'private' stamp: 'EG 12/10/2005 15:43'!
prepareImageWithVersion: aString 
	"Prepare image for installer building
		self prepareImage
	"

	| img |
	self qaRun.
	img := SmalltalkImage current.
	(SourceFiles at: 2) ifNotNil: 
			[img
				closeSourceFiles;
				saveChangesInFileNamed: (img 
							fullNameForChangesNamed: self getChangesFilename)].
	img saveImageInFileNamed: self getImageFilename.
	Smalltalk cleanUpUndoCommands.
	Preferences disable: #warnIfNoChangesFile.
	Preferences disable: #warnIfNoSourcesFile.
	Transcript closeAllViews.
	#(
		'codeSmell: aString'
		'fixMe: aString'
		'needsMoreWork'
		'needsMoreWork: aString'
		'qndTestSupport: aString'
	) do: [:sel | Object compile: sel asString].
	DGVApplication beDeploymentImage.
	TDSDebuggerSupport debugging: false.
	TDSDebuggerSupport haltOnErrors: false.
	self cleanUpRunningNode.
	UnhandledError compile: 'isDevelopmentEnvironmentPresent ^false'.

	"Clean out MC cache cleanly"
	MCWorkingCopyBrowser allInstances do: [:each | each close].
	Utilities breakDependents.
	Smalltalk at: #MCWorkingCopy
		ifPresent: 
			[:wc | 
			wc allInstances do: 
					[:ea | 
					ea breakDependents.
					Smalltalk at: #SystemChangeNotifier
						ifPresent: [:scn | scn uniqueInstance noMoreNotificationsFor: ea]]].
	MCRepositoryGroup default initialize.
	MCWorkingCopy instVarNamed: #registry put: nil.
	MCPackageManager instVarNamed: #registry put: nil.
	MCCacheRepository instVarNamed: #default put: nil.
	Smalltalk garbageCollect.
	"We come up with the main GUI"
	DGVApplication
		version: aString;
		buildDate: Date today.
	DisplayScreen 
		depth: DisplayScreen actualScreenDepth
		width: 720
		height: 567
		fullscreen: false.
	Smalltalk addToStartUpList: DGVApplication.
	SmalltalkImage current snapshot: true andQuit: false.
	Project enter: 'DGV-Client'! !

!DGVInstaller class methodsFor: 'building' stamp: 'EG,JJ 10/25/2005 15:04'!
prepareReleaseImage
	"
		KolApplicationConfiguration setConfigurationFor: DGVApplicationConfiguration.
		self prepareReleaseImage
	"
	| version |
	version _ FillInTheBlank request: 'Version Number'.
	version ifEmpty: [^self].
	
	self prepareImageWithVersion: version! !

!DGVInstaller class methodsFor: 'qa' stamp: 'CdG 4/8/2005 00:58'!
qaCheckReferencesToTranscript
	errs := Set new.
	everything _ SystemNavigation default allCallsOn: (Smalltalk associationAt: #Transcript).

	self topLevelPackageNames 
		do: [:each | self qaCheckReferencesToTranscriptInPackageNamed: each].

	errs := errs reject: [:each | each actualClass = DGVInstaller class].

	errs isEmpty 
		ifFalse: 
			[SystemNavigation default browseMessageList: errs name: 'References to Transcript'.
			self error: 'References to Transcript found!!']! !

!DGVInstaller class methodsFor: 'qa' stamp: 'CdG 4/8/2005 00:32'!
qaCheckReferencesToTranscriptInPackageNamed: aString
	| pkg methodsToCheck  |
	pkg _ (MCPackage named: aString).
	methodsToCheck _ pkg packageInfo methods.
	errs addAll: (methodsToCheck intersection: everything).
	
	self recurseDependenciesFrom: pkg! !

!DGVInstaller class methodsFor: 'qa' stamp: 'CdG 4/8/2005 00:32'!
qaCheckSendingOfHaltInPackageNamed: aString 
	| pkg methodsToCheck |
	pkg := MCPackage named: aString.
	methodsToCheck := pkg packageInfo methods.
	errs addAll: (methodsToCheck intersection: everything).
	self recurseDependenciesFrom: pkg! !

!DGVInstaller class methodsFor: 'qa' stamp: 'EG 10/31/2005 11:20'!
qaCheckSendingOfHaltInspectAndOthers
	| rejectedMethods |
	rejectedMethods := #(#halt #inspect #explore).
	errs := Set new.
	everything := Set new.
	rejectedMethods do: [:each| 
		everything addAll: (SystemNavigation default allCallsOn: each)
	].
	self topLevelPackageNames 
		do: [:each | self qaCheckSendingOfHaltInPackageNamed: each].
	errs := errs 
				reject: [:each | each methodSymbol = #qaCheckSendingOfHaltInspectAndOthers].
	errs := errs reject: [:each | each actualClass = TDSDebuggerSupport class].
	errs isEmpty 
		ifFalse: 
			[SystemNavigation default browseMessageList: errs name: 'Senders of ', rejectedMethods asString.
			self error: 'Senders of ', rejectedMethods asString, ' found!!']! !

!DGVInstaller class methodsFor: 'qa' stamp: 'EG 10/31/2005 11:15'!
qaRun
	"
		self qaRun
	"

	self qaCheckSendingOfHaltInspectAndOthers.
	self qaCheckReferencesToTranscript.
	(self confirm: 'Did you run at least DGVBugsLintDialog?') 
		ifFalse: 
			[DGVBugsLintDialog open.
			self halt: 'Run bugs lint first!!']! !

!DGVInstaller class methodsFor: 'qa' stamp: 'CdG 4/8/2005 00:32'!
recurseDependenciesFrom: pkg
	| selector |
	selector _ thisContext sender selector. "smartass!!"
	pkg workingCopy requiredPackages do: 
			[:each | 
			(self isOurOwnStuff: each name) 
				ifTrue: [self perform: selector with: each name]]! !

!DGVInstaller class methodsFor: 'qa' stamp: 'CdG 4/8/2005 00:20'!
topLevelPackageNames
	^#('DGVwxUI' 'DGVseaUI')! !

!DGVInstaller class methodsFor: 'building' stamp: 'CdG,TJ 2/17/2005 14:36'!
uploadDailyBuildInstaller
	"
		self uploadDailyBuildInstaller
	"
	self uploadInstallerWithVersion:  self versionIdentifier! !

!DGVInstaller class methodsFor: 'private' stamp: 'EG 6/15/2005 14:39'!
uploadFileNamed: aString 
	| rep fs |
	rep := MCHttpRepository 
				location: 'http://dgv.aardworks.nl/dist/in'
				user: 'aard'
				password: 'w0rks'.
	rep 
		writeStreamForFileNamed: aString
		replace: false
		do: 
			[:ws | 
			fs := FileStream oldFileNamed: aString.
			fs binary.
			[ws nextPutAll: fs contentsOfEntireFile] ensure: [fs close]]! !

!DGVInstaller class methodsFor: 'private' stamp: 'EG,CdG 9/2/2005 16:56'!
uploadInstallerWithVersion: aString
	"
		self uploadInstaller
	"

	| installerName |
	installerName := 'Kolibri-' , aString , '-installer.exe'.
	self uploadFileNamed: installerName.
	HTTPSocket httpGet: 'http://dgv.aardworks.nl/cgi-bin/link-daily-build.cgi'! !

!DGVInstaller class methodsFor: 'building' stamp: 'CdG,TJ 2/17/2005 14:36'!
uploadReleaseInstaller
	"
		self uploadReleaseInstaller
	"
	| version |
	version _ FillInTheBlank request: 'Version Number'.
	version ifEmpty: [^self].
	
	self uploadInstallerWithVersion: version! !

!DGVInstaller class methodsFor: 'private' stamp: 'CdG 2/3/2005 21:01'!
versionIdentifier
	"^((MCPackage named: 'DGV-Config-Client') workingCopy ancestors last name findTokens: '.') last"
 	^'build-', ((Date today printFormat: #(3 2 1 $- 1 1 2)) reject: [:c | c = $-])! !

Object subclass: #DGVLink
	instanceVariableNames: 'class id version lastResolveAttempt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVLink class methodsFor: 'as yet unclassified' stamp: 'EG 12/10/2005 21:00'!
class: aClass id: aUUID 
	^ self new pvtxxxClass: aClass;
		 pvtxxxId: aUUID;
		 pvtxxxVersion: 0! !

!DGVLink class methodsFor: 'as yet unclassified' stamp: 'CdG 3/9/2005 13:33'!
for: aDGVObject 
	^(self new)
		pvtxxxClass: aDGVObject class;
		pvtxxxId: aDGVObject id;
		pvtxxxVersion: aDGVObject version! !

!DGVLink class methodsFor: 'as yet unclassified' stamp: 'EG 12/10/2005 21:44'!
pauseBetweenResolveAttempts
	^ 15 minutes! !

!DGVLink methodsFor: 'private' stamp: 'CdG 4/13/2005 23:25'!
attemptConversionFromPreviousVersion
	"we have DGVLinks on disk that have the id in the class var, 
	 the version in the id var. Attempt to fix this."
	| object |
	DGVStorage registry values do: [:each |
			(each exists: class "which is in fact a UUID now")
				ifTrue: [
					object _ each get: class.
					version _ id.
					id _ class.
					class _ object class.
					"flush all objects to disk that include us"
					(Smalltalk pointersTo: self) do: [:o |
						(o isKindOf: DGVObject) ifTrue: [
							o save]].
					DGVCollection allInstances do: [:coll |
						(coll items asIdentitySet includes: self) ifTrue: [
							coll save]]]].! !

!DGVLink methodsFor: 'testing' stamp: 'EG,CdG 8/18/2005 11:16'!
dgvMatchTextSearch: pattern
	[^self value dgvMatchTextSearch: pattern]
		on: DGVLinkCouldNotBeResolvedException
		do: [:ex | 
			^false]! !

!DGVLink methodsFor: 'testing' stamp: 'TJ 2/17/2005 18:43'!
dirty
	^false "always"! !

!DGVLink methodsFor: 'message handling' stamp: 'TJ,CdG 3/30/2005 15:06'!
doesNotUnderstand: aMessage
	"	This would work just as fine:
			^self value perform: aMessage selector withArguments: aMessage arguments
		It's all a matter of performance, probably. Preliminary testing shows that from
		30 messages on, it becomes cheaper to use become. 
		
		However - with #becomeForward: we do not need to be concerned with writing ourselves (cleaning
		up resolved etcetera)"
	self resolveYourself.
	self isDGVLink
		ifTrue: [ self error: 'Object niet gevonden op het netwerk' translated ].
	^self perform: aMessage selector withArguments: aMessage arguments! !

!DGVLink methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/10/2005 12:11'!
equalsObjectWithClass: aClass andUuid: anUUID 
	^class = aClass and: [id = anUUID]! !

!DGVLink methodsFor: 'testing' stamp: 'TJ 3/31/2005 12:24'!
hash
	^id hash! !

!DGVLink methodsFor: 'accessing' stamp: 'EG,CdG 10/14/2005 12:05'!
htmlRepresentation
	^self value htmlRepresentation! !

!DGVLink methodsFor: 'accessing' stamp: 'CdG 3/8/2005 13:33'!
id
	^id! !

!DGVLink methodsFor: 'testing' stamp: 'CdG 2/14/2005 17:15'!
isDGVLink
	^true! !

!DGVLink methodsFor: 'accessing' stamp: 'EG,TJ 3/22/2005 15:32'!
myclass
	^class! !

!DGVLink methodsFor: 'printing' stamp: 'JJ 12/8/2005 22:17'!
printOn: aStream
	| storageName |
	super printOn: aStream.
	storageName := [self storageName] on: Error do: ['Corrupt Link'].
	aStream 
		nextPut: $(;
		nextPutAll: storageName;
		nextPut: $/;
		nextPutAll: self myclass asString;
		nextPut: $/;
		nextPutAll: self id printString;
		nextPut: $)! !

!DGVLink methodsFor: 'private' stamp: 'CdG 2/14/2005 17:10'!
pvtxxxClass: anObject
	class _ anObject! !

!DGVLink methodsFor: 'private' stamp: 'CdG 2/14/2005 17:10'!
pvtxxxId: anObject
	id _ anObject! !

!DGVLink methodsFor: 'private' stamp: 'CdG 3/9/2005 13:33'!
pvtxxxVersion: aNumber
	version _ aNumber! !

!DGVLink methodsFor: 'testing support' stamp: 'EG,CdG 9/7/2005 14:15'!
resetLastResolveAttempt
	lastResolveAttempt := nil! !

!DGVLink methodsFor: 'message handling' stamp: 'CdG,EG 12/9/2005 15:08'!
resolveYourself
	| answer |
	[(class isKindOf: UUID) ifTrue: [self attemptConversionFromPreviousVersion].
	lastResolveAttempt ifNotNil: [
			((TimeStamp now - lastResolveAttempt) < self class pauseBetweenResolveAttempts)
				ifTrue: [^DGVLinkCouldNotBeResolvedException new link: self; signal]].

	answer _ class get: id withVersion: version]
		on: Error
		do: [:ex | 
			TDSDebuggerSupport logError: ex printString.
			answer _ nil].
	answer 
		ifNil: [
			lastResolveAttempt _ TimeStamp now.
			DGVLinkCouldNotBeResolvedException new link: self; signal]
		ifNotNil: [self becomeForward: answer]
! !

!DGVLink methodsFor: 'accessing' stamp: 'CdG 3/10/2005 10:50'!
storageName
	^class storageName! !

!DGVLink methodsFor: 'private' stamp: 'CdG 3/8/2005 13:25'!
value
	"Transcript show: 'Resolving link to ', class printString, '/', id printString; cr.
	Transcript show: thisContext sender  longStack; cr."
	self resolveYourself.
	^self! !

!DGVLink methodsFor: 'accessing' stamp: 'CdG 3/9/2005 13:32'!
version
	^version! !

!DGVLink methodsFor: 'testing' stamp: 'TJ 3/31/2005 12:23'!
= anObject
	^(anObject respondsTo: #id) 
		and: [self id = anObject id]! !

DGVLink subclass: #DGVMockUnresolvableLink
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVMockUnresolvableLink methodsFor: 'as yet unclassified' stamp: 'CdG 7/4/2005 11:11'!
value
	DGVLinkCouldNotBeResolvedException signal! !

Object subclass: #DGVManualTestAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVManualTestAction class methodsFor: 'as yet unclassified' stamp: 'CdG 4/8/2005 00:40'!
do
	DGVManualTestAction new do! !

!DGVManualTestAction methodsFor: 'as yet unclassified' stamp: 'CdG 11/25/2005 15:59'!
do
	"
	TestAction do
	"

	| anewfile collel |
	anewfile := DGVFile new.
	anewfile id: UUID new.
	(anewfile metadata)
		identifier: anewfile id;
		title: 'My Testtitle ' , 100 atRandom printString;
		creator: 'ThijsTest';
		date: Date today yyyymmdd , ' ' , Time now print24;
		publisher: UUID new.
	anewfile save.
	collel := DGVCollectionElement 
				collId: (UUID fromString: 'da0c6736-4fc6-c04b-8839-5b492f3b0ee2')
				elementId: anewfile id
				elementClass: anewfile class.	"Community Testing, files"
	TDSDebuggerSupport 
		log: 'Adding new file to comm: Testing with id: ' , anewfile id asString.
	DGVCollection announcementPresenceSet addPresenceOf: collel
		origin: UUID new	"Some other node"! !

Object subclass: #DGVMetaData
	instanceVariableNames: 'dc'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVMetaData commentStamp: 'CdG 4/21/2005 14:17' prior: 0!
A generic metadata holder. 'dc', inappropriately named but the name has to be kept for serialization compatibility purposes, holds a dictionary of attribute names pointing to collections of attribute values. !

DGVMetaData subclass: #DGVDublinCore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVDublinCore class methodsFor: 'code generation' stamp: 'CdG,EjG 6/8/2005 10:21'!
attributes
	"Method only for SLint"
	self shouldNotImplement.! !

!DGVDublinCore class methodsFor: 'code generation' stamp: 'EG,CdG 10/13/2005 11:40'!
dcAttributeDescriptions
	^Dictionary newFrom: {
#title -> 'Titel'.
#creator -> 'Vervaardiger'.
#subject -> 'Trefwoord(en)'.
#description -> 'Beschrijving'.
#publisher -> 'Gepubliceerd door'.
#contributor -> 'Mede-vervaardiger'.
#date -> 'Publicatie datum'.
#type -> 'Categorie of genre'.
#format -> 'Bestandsformaat'.
#identifier -> 'Identificatie'.
#source -> 'Afkomstig uit'.
#language -> 'Taal'.
#relation -> 'Relatie'.
#coverageTemporal -> 'Datum of periode'.
#coverageSpatial -> 'Locatie of gebied'.
#rights -> 'Rechten'
}! !

!DGVDublinCore class methodsFor: 'code generation' stamp: 'EG 6/20/2005 14:15'!
dcAttributeNames
	^ OAIDCMetadata dcAttributeNames , #(#coverageSpatial #coverageTemporal )! !

!DGVDublinCore class methodsFor: 'instance creation' stamp: 'EG 6/20/2005 12:34'!
fromOAIDC: anOAIDCMetadata 
	^self new fromOAIDC: anOAIDCMetadata! !

!DGVDublinCore class methodsFor: 'code generation' stamp: 'TJ+CdG 1/18/2005 11:24'!
generate
	"
		self generate
	"

	| selector source |
	Cursor wait showWhile: [(self class allMethodsInCategory: 'code generation-templates') do: 
			[:meth | 
			self dcAttributeNames do: 
					[:attr | 
					source := self class sourceCodeAt: meth.
					source := source copyReplaceAll: 'Attribute' with: attr capitalized.
					source := source copyReplaceAll: 'attribute' with: attr.
					"snip template"
					source := source copyFrom: 'template' size + 1 to: source size.
					source := source , '
"GENERATED CODE - DO NOT EDIT!!!!!!!!"'.
					selector := self compile: source.
					self organization classify: selector under: 'accessing-generated']]]! !

!DGVDublinCore class methodsFor: 'private-lint fixes' stamp: 'CdG 11/29/2005 11:30'!
metadata
	"this one silences SmallLint warnings but shouldn't actually be called"
	TDSDebuggerSupport logError: 'DGVDublinCore class>>metadata called!!'! !

!DGVDublinCore class methodsFor: 'code generation-templates' stamp: 'TJ+CdG 1/18/2005 10:49'!
templateaddAttribute: aString 
	self attributes add: aString! !

!DGVDublinCore class methodsFor: 'code generation-templates' stamp: 'TJ+CdG 1/18/2005 11:25'!
templateattribute
	^self attributes
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]! !

!DGVDublinCore class methodsFor: 'code generation-templates' stamp: 'EG,JJ 10/25/2005 14:37'!
templateattributes
	^self metadata at: #attributes ifAbsentPut: [OrderedCollection new]! !

!DGVDublinCore class methodsFor: 'code generation-templates' stamp: 'EG,JJ 10/25/2005 14:37'!
templateattribute: aString 
	self metadata at: #attributes put: (OrderedCollection with: aString)! !

!DGVDublinCore class methodsFor: 'code generation' stamp: 'CdG 3/25/2005 23:07'!
visibleDcAttributeNames
	| names |
	names _ self dcAttributeNames asOrderedCollection.
	names removeAll: #(#identifier #relation).
	^names! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addContributor: aString 
	self contributors add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addCoverageSpatial: aString 
	self coverageSpatials add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addCoverageTemporal: aString 
	self coverageTemporals add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'TJ+CdG 1/18/2005 11:25'!
addCoverage: aString 
	self coverages add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addCreator: aString 
	self creators add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addDate: aString 
	self dates add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addDescription: aString 
	self descriptions add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addFormat: aString 
	self formats add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addIdentifier: aString 
	self identifiers add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addLanguage: aString 
	self languages add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addPublisher: aString 
	self publishers add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addRelation: aString 
	self relations add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addRights: aString 
	self rightss add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addSource: aString 
	self sources add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addSubject: aString 
	self subjects add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addTitle: aString 
	self titles add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
addType: aString 
	self types add: aString
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
contributor
	^self contributors
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
contributors
	^self metadata at: #contributors ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
contributor: aString 
	self metadata at: #contributors put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'TJ+CdG 1/18/2005 11:26'!
coverage
	^self coverages
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
coverageSpatial
	^self coverageSpatials
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
coverageSpatials
	^self metadata at: #coverageSpatials ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
coverageSpatial: aString 
	self metadata at: #coverageSpatials put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:39'!
coverages
	^self metadata at: #coverages ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
coverageTemporal
	^self coverageTemporals
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
coverageTemporals
	^self metadata at: #coverageTemporals ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
coverageTemporal: aString 
	self metadata at: #coverageTemporals put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:39'!
coverage: aString 
	self metadata at: #coverages put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
creator
	^self creators
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
creators
	^self metadata at: #creators ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
creator: aString 
	self metadata at: #creators put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
date
	^self dates
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
dates
	^self metadata at: #dates ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
date: aString 
	self metadata at: #dates put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
description
	^self descriptions
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
descriptions
	^self metadata at: #descriptions ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
description: aString 
	self metadata at: #descriptions put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
format
	^self formats
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
formats
	^self metadata at: #formats ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
format: aString 
	self metadata at: #formats put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'initialize-release' stamp: 'EG,JJ 10/25/2005 14:39'!
fromOAIDC: anOAIDCMetadata 
	| pluralName |
	OAIDCMetadata dcAttributeNames
		do: [:name | pluralName _ (name , 's') asSymbol. 
self metadata
				at: pluralName
				put: (anOAIDCMetadata perform: pluralName)]! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,CdG 10/13/2005 12:09'!
htmlRepresentation
	| os avs description values format first displayVal |
	os _ String new writeStream.
	avs _ self class dcAttributeNames collect: [:attr |
		attr -> (self perform: (attr, 's') asSymbol)].
	os nextPutAll: '<table>'.
	avs do: [:assoc |
		values _ assoc value select: [:each | each htmlRepresentation notEmpty].
		values notEmpty ifTrue: [
			description _ self class dcAttributeDescriptions at: assoc key.
			os nextPutAll: ('<tr><td valign="top">{1}:</td><td>' format: {description}).
			first _ true.
			assoc value do: [:val |
				first ifFalse: [os nextPutAll: '<br>'].
				displayVal := val htmlRepresentation.
				assoc key = 'title' 
					ifTrue: [displayVal := '<b>',displayVal,'</b>'].

				format _ displayVal isUrl ifTrue: ['<a href="{1}">{1}</a>'] ifFalse: ['{1}'].
				os nextPutAll: (format format: {displayVal}).
				first _ false].
			os nextPutAll: '</td></tr>'
		]
	].
	os nextPutAll: '</table>'.
	^os contents.
! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
identifier
	^self identifiers
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
identifiers
	^self metadata at: #identifiers ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
identifier: aString 
	self metadata at: #identifiers put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
language
	^self languages
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
languages
	^self metadata at: #languages ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
language: aString 
	self metadata at: #languages put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
publisher
	^self publishers
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
publishers
	^self metadata at: #publishers ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
publisher: aString 
	self metadata at: #publishers put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
relation
	^self relations
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
relations
	^self metadata at: #relations ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
relation: aString 
	self metadata at: #relations put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
rights
	^self rightss
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
rightss
	^self metadata at: #rightss ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
rights: aString 
	self metadata at: #rightss put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
source
	^self sources
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
sources
	^self metadata at: #sources ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
source: aString 
	self metadata at: #sources put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
subject
	^self subjects
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
subjects
	^self metadata at: #subjects ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
subject: aString 
	self metadata at: #subjects put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
title
	^self titles
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
titles
	^self metadata at: #titles ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
title: aString 
	self metadata at: #titles put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
type
	^self types
		ifEmpty: [nil]
		ifNotEmpty: [:v | v first]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
types
	^self metadata at: #types ifAbsentPut: [OrderedCollection new]
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVDublinCore methodsFor: 'accessing-generated' stamp: 'EG,JJ 10/25/2005 14:38'!
type: aString 
	self metadata at: #types put: (OrderedCollection with: aString)
"GENERATED CODE - DO NOT EDIT!!!!!!!!"! !

!DGVMetaData methodsFor: 'private' stamp: 'EG,JJ 10/25/2005 14:44'!
metadata
	^dc ifNil: [dc := IdentityDictionary new]! !

Object subclass: #DGVMockNode
	instanceVariableNames: 'presences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVMockNode methodsFor: 'mocking presence' stamp: 'EG,TJ 3/21/2005 16:24'!
addPresenceOf: aDGVLink 
	self presences add: aDGVLink! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/10/2005 11:51'!
connectedPeers
	^#()! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 20:35'!
dispatchDynamicEndPointTo: aBlock
	^0! !

!DGVMockNode methodsFor: 'mocking presence' stamp: 'CdG 11/29/2005 10:30'!
entryAt: aDGVLink
	^nil! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'CdG 5/3/2005 16:56'!
isConnected
	^false! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'EG 6/14/2005 11:35'!
makePacket: aByteArray destination: anUUID 
	! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/13/2005 11:33'!
networkTimeStamp
	^TimeStamp current! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'EG 4/11/2005 15:14'!
nodeId
	^UUID fromString: '892772e7-db48-e941-aca2-25aa31389c5a'! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/21/2005 16:24'!
presence
	! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/21/2005 16:24'!
presences
	^ presences ifNil: [presences _ OrderedCollection new]! !

!DGVMockNode methodsFor: 'mocking presence' stamp: 'TJ 5/4/2005 11:04'!
removePresenceOf: aDGVMockObject 
	! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 20:37'!
send: aByteArray toNodeId: aNodeId onEndPoint: aNumber reliable: aBoolean! !

!DGVMockNode methodsFor: 'mocking presence' stamp: 'EG,TJ 3/21/2005 16:24'!
setAt: aString ifAbsentConfigureWith: aBlockContext 
	! !

!DGVMockNode methodsFor: 'mocking presence' stamp: 'EG 4/11/2005 14:32'!
setAt: aString ifUnconfigured: aBlockContext 
	! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 20:23'!
stop
! !

!DGVMockNode methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 20:21'!
unRegisterEndPoint: aNumber! !

Object subclass: #DGVMockOAICollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

Object subclass: #DGVMockSMTPClient
	instanceVariableNames: 'sentMails'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVMockSMTPClient methodsFor: 'faking' stamp: 'CdG 2/14/2005 14:30'!
deliverMailFrom: fromAddress to: recipientList text: messageText usingServer: serverName
	sentMails add: (MailMessage from: messageText)! !

!DGVMockSMTPClient methodsFor: 'initialize-release' stamp: 'CdG 2/14/2005 14:28'!
initialize
	sentMails _ OrderedCollection new! !

!DGVMockSMTPClient methodsFor: 'accessing' stamp: 'CdG 2/14/2005 14:28'!
sentMailCount
	^sentMails size! !

!DGVMockSMTPClient methodsFor: 'accessing' stamp: 'CdG 2/14/2005 14:28'!
sentMails
	^sentMails! !

Object subclass: #DGVNewUserPolicy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVNewUserPolicy subclass: #DGVNewUserPolicyAlwaysAccept
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVNewUserPolicyAlwaysAccept class methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 13:19'!
policyName
	^ 'Open'! !

!DGVNewUserPolicyAlwaysAccept class methodsFor: 'as yet unclassified' stamp: 'CdG 2/15/2005 10:21'!
postApplicationMessage
	^'U bent nu lid van de community'! !

!DGVNewUserPolicyAlwaysAccept methodsFor: 'user acceptance' stamp: 'CdG 2/9/2005 11:46'!
acceptNewUser: aDGVUser in: aDGVCommunity
	aDGVCommunity acceptNewUser: aDGVUser.
	^true! !

DGVNewUserPolicy subclass: #DGVNewUserPolicyConfirmEmail
	instanceVariableNames: 'serviceName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVNewUserPolicyConfirmEmail class methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 13:20'!
policyName
	^'Controle e-mail adres'! !

!DGVNewUserPolicyConfirmEmail class methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 14:40'!
postApplicationMessage
	^'U krijgt nu een email waarmee u uw aanmelding kan bevestigen'! !

!DGVNewUserPolicyConfirmEmail class methodsFor: 'as yet unclassified' stamp: 'CdG 4/13/2005 20:24'!
serviceName
	^DGVPolicyConfirmUser serviceName! !

!DGVNewUserPolicyConfirmEmail methodsFor: 'user acceptance' stamp: 'EG 12/14/2005 14:33'!
acceptNewUser: aDGVUser in: aDGVCommunity 
	| peer entries sleeper mailserviceReceivedMessage replyPoint request tries |

	peer := DGVNode default.
	entries := DGVService presenceSet entries 
				select: [:each | each object = self class serviceName].
	entries ifEmpty: [^DGVServiceException new signal: self class serviceName].
	sleeper := Semaphore new.
	mailserviceReceivedMessage := false.
	replyPoint := peer dispatchDynamicEndPointTo: 
					[:data :source | 
					data size > 0 ifTrue: [mailserviceReceivedMessage := true].
					sleeper signal].
	request := (DGVPolicyMailServiceRequest new)
				communityId: aDGVCommunity id;
				userId: aDGVUser id;
				replyEndPoint: replyPoint.
	request _ request marshallAsByteArray.
	tries := 1.
	[tries <= 3 and: [mailserviceReceivedMessage not]] whileTrue: 
			[entries do: 
					[:entry | 
					mailserviceReceivedMessage 
						ifFalse: 
							[peer 
								send: request
								toNodeId: entry origin
								onEndPoint: entry endPoint
								reliable: false].
					sleeper waitTimeoutSeconds: 5.
					tries := tries + 1]].
	mailserviceReceivedMessage 
		ifFalse: [DGVServiceException new signal: 'Mail service did not accept message.' translated ].
	^nil! !

DGVNewUserPolicyConfirmEmail subclass: #DGVNewUserPolicyOwnerConfirms
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVNewUserPolicyOwnerConfirms class methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 14:59'!
policyName
	^'Eigenaar bevestigt'! !

!DGVNewUserPolicyOwnerConfirms class methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 14:41'!
postApplicationMessage
	^'De eigenaar van deze community moet uw aanmelding bevestigen. Na bevestiging ontvangt u hierover een email'! !

!DGVNewUserPolicyOwnerConfirms class methodsFor: 'as yet unclassified' stamp: 'CdG 4/14/2005 01:13'!
serviceName
	^DGVPolicyOwnerConfirm serviceName! !

!DGVNewUserPolicy class methodsFor: 'accessing' stamp: 'CdG 2/14/2005 13:26'!
allPolicyNames
	^( self allSubclasses collect: [:each | each policyName]) asArray! !

!DGVNewUserPolicy class methodsFor: 'accessing' stamp: 'CdG 2/14/2005 13:20'!
defaultPolicyName
	^DGVNewUserPolicyAlwaysAccept policyName! !

!DGVNewUserPolicy class methodsFor: 'accessing' stamp: 'CdG 2/14/2005 13:18'!
policyName
	self subclassResponsibility! !

!DGVNewUserPolicy class methodsFor: 'accessing' stamp: 'CdG 2/14/2005 14:39'!
postApplicationMessage
	self subclassResponsibility! !

!DGVNewUserPolicy class methodsFor: 'instance creation' stamp: 'CdG 2/14/2005 13:45'!
withName: aString
	^(self allSubclasses detect: [:each | each policyName = aString]) new.! !

!DGVNewUserPolicy methodsFor: 'user acceptance' stamp: 'CdG 2/9/2005 11:38'!
acceptNewUser: aDGVUser in: aDGVCommunity
	"accept a user or not. Three return values:
		true: user is accepted and was added to community
		false: user is definitely rejected
		nil: some sort of external confirmation process has been started"
	
	self subclassResponsibility! !

!DGVNewUserPolicy methodsFor: 'as yet unclassified' stamp: 'TJ 4/21/2005 11:10'!
policyName
	^ self class policyName! !

!DGVNewUserPolicy methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 14:49'!
postApplicationMessage
	^self class postApplicationMessage! !

Object subclass: #DGVNodeHealthMonitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Monitoring'!

!DGVNodeHealthMonitor methodsFor: 'as yet unclassified' stamp: 'CdG 6/22/2005 16:51'!
checkHealth
	| errors |
	errors _ OrderedCollection new.
	errors addAll: self checkProcesses.
	^errors! !

!DGVNodeHealthMonitor methodsFor: 'as yet unclassified' stamp: 'CdG 6/22/2005 16:51'!
checkProcesses
	| processList errors procs |
	errors _ OrderedCollection new.
	processList _ Process allInstances.
	self mandatoryProcessNames do: [:each | 
		procs _ processList select: [:elem |
			elem isTerminated not and: [elem name = each]].
		procs isEmpty ifTrue: [errors add: 'Process "', each, '" not found or terminated'].
		procs size > 1 ifTrue: [errors add: 'Multiple instances of process "', each, '"']].
	^errors! !

!DGVNodeHealthMonitor methodsFor: 'as yet unclassified' stamp: 'CdG 6/22/2005 16:47'!
mandatoryProcessNames
	"these processes MUST be running in a healthy DGV node"
	^#(
			'TUUDPChannel receiver'
			'TUUDPChannel sender'
			'TUUDPChannel ACK sender'
			'TUDatagramChannel retransmission'
			'TUDatagramChannel receiver'
			'TUDatagramChannel receptionCleaner'
			'TUDispatchingDatagramChannel dispatch'
			'AWGPeerList talker'
			'AWGPresence sender'
			'DGVCleanerService fetcher 1'
			'AntRouter decay'
			'AWGNode idle watcher'
)! !

Object subclass: #DGVNotifyingCollection
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVNotifyingCollection subclass: #DGVFilteringNotifyingCollection
	instanceVariableNames: 'filterBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVFilteringNotifyingCollection class methodsFor: 'instance creation' stamp: 'EG,CdG 6/27/2005 12:35'!
on: aDGVNotifyingCollection filterBlock: aBlockContext 
	^ self new collection: aDGVNotifyingCollection;
		 filterBlock: aBlockContext! !

!DGVFilteringNotifyingCollection methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/27/2005 13:51'!
collection
	^ super collection select: filterBlock! !

!DGVFilteringNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 12:37'!
collection: aDGVNotifyingCollection 
	collection := aDGVNotifyingCollection.
	collection
		when: #itemAdded
		send: #itemAdded:
		to: self! !

!DGVFilteringNotifyingCollection methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/27/2005 12:17'!
filterBlock: aBlockContext 
	filterBlock _ aBlockContext! !

!DGVFilteringNotifyingCollection methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 16:49'!
itemAdded: anObject
	[(filterBlock value: anObject) ifTrue: [self triggerEvent: #activeItemsChanged with: self]] on: Error do: [:ex | TDSDebuggerSupport logError: 'Error in adding item: ', ex printString]! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 12:08'!
activeItems
	^self collection! !

!DGVNotifyingCollection methodsFor: 'adding' stamp: 'EG,CdG 7/6/2005 16:44'!
addAll: anArray 
	anArray do: [:each | self add: each]! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 16:45'!
add: anObject 
	self collection add: anObject.
	self triggerEvent: #itemAdded with: anObject.
	self triggerEvent: #activeItemsChanged with: self .
	^anObject
! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 14:04'!
asOrderedCollection
	^ self collection copy! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 12:04'!
collection
	^collection ifNil: [collection _ OrderedCollection new]! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 12:51'!
includes: aDGVFile 
	^self collection includes: aDGVFile! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 12:50'!
isEmpty
	^self collection isEmpty! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'CdG 7/7/2005 10:45'!
remove: anObject 
	self collection remove: anObject.
	self triggerEvent: #itemRemoved with: anObject.
	self triggerEvent: #activeItemsChanged with: self .
	^anObject
! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 13:52'!
select: aBlockContext 
	^self collection select: aBlockContext! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 12:52'!
size
	^self collection size! !

!DGVNotifyingCollection methodsFor: 'accessing' stamp: 'JJ 12/2/2005 10:48'!
sortWith: aBlock
	collection := collection asSortedCollection: aBlock.
	self triggerEvent: #activeItemsChanged with: self .! !

Object subclass: #DGVOAIHarvestService
	instanceVariableNames: 'oaiClientFactory repositories scheduler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVOAIHarvestService methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/23/2005 08:47'!
harvestTask: oaiClient 
	| delay |
	[oaiClient harvest]
		ensure: [delay := oaiClient hasMoreRecords
						ifTrue: [15 minutes]
						ifFalse: [1 day].
			self scheduleHarvestTaskFor: oaiClient in: delay.
			^true]! !

!DGVOAIHarvestService methodsFor: 'configuring' stamp: 'EG,CdG 6/22/2005 14:18'!
oaiClientFactory: factory
	oaiClientFactory _ factory
	! !

!DGVOAIHarvestService methodsFor: 'configuring' stamp: 'EG,CdG 6/22/2005 14:19'!
repositories: anArray 
	repositories _ anArray! !

!DGVOAIHarvestService methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/22/2005 15:16'!
scheduleHarvestTaskFor: oaiClient in: aDuration
	self scheduler
		doOnce: [self harvestTask: oaiClient]
		at: TimeStamp now + aDuration! !

!DGVOAIHarvestService methodsFor: 'accessing' stamp: 'EG,CdG 6/22/2005 14:38'!
scheduler
	^scheduler ifNil: [scheduler _ TaskScheduler new]! !

!DGVOAIHarvestService methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/22/2005 15:15'!
start
	| oaiClient |
	oaiClient := oaiClientFactory withBaseUrl: repositories first.
	self scheduler start.
	self scheduleHarvestTaskFor: oaiClient in: 0 seconds.! !

!DGVOAIHarvestService methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/22/2005 15:43'!
stop
	self scheduler stop.
	scheduler _ nil! !

Object subclass: #DGVPolicyMailServiceRequest
	instanceVariableNames: 'userId communityId replyEndPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVPolicyMailServiceRequest class methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 15:21'!
fromByteArray: aByteArray
	"goes hand-in-hand with #asByteArray at the instance side"
	| answer |
	answer _ self new.
	answer 
		communityId: (UUID newFrom: (aByteArray copyFrom: 1 to: 16));
		userId: (UUID newFrom: (aByteArray copyFrom: 17 to: 32));
		replyEndPoint: (aByteArray unsignedShortAt: 33 bigEndian: true).
	^answer! !

!DGVPolicyMailServiceRequest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 13:52'!
asByteAr
	"goes hand-in-hand with #fromByteArray: at the class side"
	| answer |
	answer _ ByteArray new: 34.
	answer 
		replaceFrom: 1 to: 16 with: communityId startingAt: 1;
		replaceFrom: 17 to: 32 with: userId startingAt: 1;
		unsignedShortAt: 33 put: replyEndPoint bigEndian: true.
	^answer! !

!DGVPolicyMailServiceRequest methodsFor: 'accessing' stamp: 'TJ 3/1/2005 14:29'!
communityId
	^communityId! !

!DGVPolicyMailServiceRequest methodsFor: 'accessing' stamp: 'TJ 3/1/2005 14:29'!
communityId: anObject
	communityId := anObject! !

!DGVPolicyMailServiceRequest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 13:52'!
marshallAsByteArray
	"goes hand-in-hand with #fromByteArray: at the class side"
	| answer |
	answer _ ByteArray new: 34.
	answer 
		replaceFrom: 1 to: 16 with: communityId startingAt: 1;
		replaceFrom: 17 to: 32 with: userId startingAt: 1;
		unsignedShortAt: 33 put: replyEndPoint bigEndian: true.
	^answer! !

!DGVPolicyMailServiceRequest methodsFor: 'accessing' stamp: 'TJ 3/1/2005 15:16'!
replyEndPoint
	^replyEndPoint! !

!DGVPolicyMailServiceRequest methodsFor: 'accessing' stamp: 'TJ 3/1/2005 15:16'!
replyEndPoint: anObject
	replyEndPoint := anObject! !

!DGVPolicyMailServiceRequest methodsFor: 'accessing' stamp: 'TJ 3/1/2005 14:29'!
userId
	^userId! !

!DGVPolicyMailServiceRequest methodsFor: 'accessing' stamp: 'TJ 3/1/2005 14:29'!
userId: anObject
	userId := anObject! !

Object subclass: #DGVProtoObject
	instanceVariableNames: 'id dirty lastSavedTime parent creationTime'
	classVariableNames: 'ActiveSearches SearchLock'
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVProtoObject subclass: #DGVCollection
	instanceVariableNames: 'items name'
	classVariableNames: 'SignalingProcess SignalQueue'
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVCollection class
	instanceVariableNames: 'mutex'!

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 11/21/2005 21:32'!
critical: aBlock
	mutex ifNil: [mutex _ Mutex new].
	^mutex critical: aBlock! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'EG 12/9/2005 14:55'!
gatherElementsFromNetworkFor: aDGVCollection 
	| query object | 
	query := (DGVQueryStorageGet new)
				key: aDGVCollection id;
				storageName: self storageName.
	query executeMultiWhenFound: 
			[:collectionOrMarshalledObject |
			(collectionOrMarshalledObject isKindOf: ByteArray)
				ifTrue: [
					object := Object fromByteArray: collectionOrMarshalledObject.
					aDGVCollection addAll: {object}; save]
				ifFalse: [
					aDGVCollection addAll: (collectionOrMarshalledObject items reject: [:each| each isDGVLink and: [each myclass isNil]]).
					aDGVCollection save]].
	(Delay forDuration: 250 milliSeconds) wait! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 14:57'!
getFromNetwork: aUUID 
	| answer searchBlock |
	DGVNode default isConnected ifFalse: [^nil].

	"Get the collection from the network. As we apparently have a reference to this collection,
	we assume that it exists and therefore create a locally cached version that we return immediately. 
	When we get answers, we add them to the cached copy by the process of merging"
	searchBlock := 
			[answer := self newWithId: aUUID.
			self localStorage put: answer.
			self gatherElementsFromNetworkFor: answer].
	self serializeNetworkSearch: searchBlock forId: aUUID.
	^answer! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 12/14/2005 20:04'!
getFromNetwork: aUUID withVersion: aNumber
	^self getFromNetwork: aUUID! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG,EG 12/9/2005 12:13'!
get: aUUID withVersion: aNumber
	^self get: aUUID "collections don't deal with version numbers"! !

!DGVCollection class methodsFor: 'signaling' stamp: 'CdG 5/3/2005 19:24'!
initialize
	SignalQueue _ nil.
	SignalingProcess ifNotNil: [SignalingProcess terminate].
	SignalingProcess _ nil.! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 14:26'!
new
	TDSDebuggerSupport fatal: 'DGVCollection class>>new called, use #parent:named: instead!!'! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 14:57'!
newWithId: aString
	"used in networking code to create local replica's of existing collections - don't use in application code!!"
	^super new id: aString! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:38'!
parent: aDGVObject named: aString
	^self basicNew initialize
		id: (aDGVObject id asString, '.', aString);
		parent: aDGVObject;
		yourself! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 4/27/2005 11:45'!
presenceSetName
	^'nl.aardworks.dgv.collectiontransactions'! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'EG 6/15/2005 14:06'!
refresh: aDGVCollection
	| searchBlock |
	DGVNode default isConnected ifFalse: [^aDGVCollection].

	searchBlock := [self gatherElementsFromNetworkFor: aDGVCollection].
	self serializeNetworkSearch: searchBlock forId: aDGVCollection id.
	^aDGVCollection! !

!DGVCollection class methodsFor: 'signaling' stamp: 'CdG 5/3/2005 17:14'!
signalingProcess
	^SignalingProcess! !

!DGVCollection class methodsFor: 'signaling' stamp: 'CdG 5/2/2005 14:35'!
signalingRun
	| block |
	block _ self signalQueue next.
	block value! !

!DGVCollection class methodsFor: 'signaling' stamp: 'CdG 11/21/2005 21:21'!
signalQueue
	SignalQueue ifNil: [SignalQueue _ SharedBufferStream newObjectStream].
	(SignalingProcess isNil or: [SignalingProcess isTerminated]) ifTrue: [
		self startSignalingProcess].
	^SignalQueue! !

!DGVCollection class methodsFor: 'signaling' stamp: 'CdG 5/3/2005 16:52'!
startSignalingProcess
	SignalingProcess _ [
		[[self signalingRun] 
			on: Error
			do: [:ex | TDSDebuggerSupport logError: 'signaling run error: ', ex printString]] 
				repeat] 
			forkAt: Processor userBackgroundPriority! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'EG 12/11/2005 17:01'!
stopSignallingProcess
	SignalingProcess ifNotNil: [SignalingProcess terminate].
	SignalingProcess := nil
	! !

!DGVCollection class methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 15:48'!
storageName
	^'collection'! !

!DGVCollection methodsFor: 'accessing' stamp: 'EG 10/31/2005 15:10'!
activeItems
	[^ self values
		reject: [:each | each isDGVLink
				or: [each deleted]]]
		on: DGVLinkCouldNotBeResolvedException
		do: [:ex | ex resume: ex link]! !

!DGVCollection methodsFor: 'private' stamp: 'EG 12/11/2005 17:08'!
activeItemsChanged
	| otherInstances allItems |
	self triggerEvent: #activeItemsChanged with: self activeItems.
	otherInstances := self class allInstances 
				select: [:each | each ~~ self and: [each id = id]].

	"this is necessary mostly for the test environment where the 'persistent' instance is in memory as well"
	allItems := self items copy .

	^otherInstances do: 
			[:each | 
			(each items)
				removeAll;
				addAll: allItems.
			each triggerEvent: #activeItemsChanged with: each activeItems]! !

!DGVCollection methodsFor: 'adding' stamp: 'EG 12/9/2005 14:55'!
addAll: aCollection 
	aCollection do: [:each |
		(self items detect: [:item | item = each] ifNone: [nil]) ifNotNilDo: [:item |
			(each isNewerThan: item) ifTrue: [
				each save.
				self items remove: item]].
		items add: each].		
	self signalActiveItemsChanged.
! !

!DGVCollection methodsFor: 'adding' stamp: 'JJ 11/22/2005 17:16'!
addWithoutPresenceUpdate: aUUID elementClass: aClass 

	items detect: [:each | each class = aClass and: [each id = aUUID]]
		ifNone: 
			[self dirty: true.
			items add: (DGVLink class: aClass id: aUUID).
			self signalActiveItemsChanged]! !

!DGVCollection methodsFor: 'adding' stamp: 'CdG 11/25/2005 15:59'!
add: aDGVObject 
	| item answer |
	item := self collectionElement: aDGVObject.
	self class announcementPresenceSet addPresenceOf: item.
	self dirty: true.
	answer := items add: aDGVObject.
	self signalActiveItemsChanged.
	^answer! !

!DGVCollection methodsFor: 'accessing' stamp: 'CdG 9/13/2005 14:37'!
asOrderedCollection
	^self activeItems asOrderedCollection! !

!DGVCollection methodsFor: 'private' stamp: 'CdG,EG 12/9/2005 12:08'!
basicSave
	self dirty ifTrue: [
		self updateLastSavedTime.
		self dirty: false].
	self localStorage basicPut: self! !

!DGVCollection methodsFor: 'converting' stamp: 'TJ 5/4/2005 16:31'!
collectionElementForRemoval: aDGVObject
	^DGVCollectionElement 
		removeFromCollId: self id 
		elementId: aDGVObject id 
		elementClass: aDGVObject class.
! !

!DGVCollection methodsFor: 'converting' stamp: 'TJ 5/4/2005 15:49'!
collectionElement: aDGVObject
	^DGVCollectionElement 
		collId: self id 
		elementId: aDGVObject id 
		elementClass: aDGVObject class.
! !

!DGVCollection methodsFor: 'private' stamp: 'CdG 3/31/2005 23:52'!
critical: aBlock
	self fixMe: 'remove global lock on DGVCollection'.
	^self class critical: aBlock! !

!DGVCollection methodsFor: 'converting' stamp: 'EG,CdG 9/7/2005 13:56'!
doesNotUnderstand: aMessage
	^(self selectorsWeDelegateToActiveItems includes: aMessage selector)
		ifTrue: [self activeItems perform: aMessage selector withArguments: aMessage arguments]
		ifFalse: [super doesNotUnderstand: aMessage]! !

!DGVCollection methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:29'!
id
	^id! !

!DGVCollection methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:29'!
id: aString 
	id _ aString! !

!DGVCollection methodsFor: 'initialize-release' stamp: 'EG,TJ 3/22/2005 14:13'!
initialize
	super initialize.
	items _ Set new.! !

!DGVCollection methodsFor: 'private' stamp: 'EG,TJ 3/24/2005 14:43'!
items
	^items! !

!DGVCollection methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:46'!
matchesForTextSearch: pattern 
	^ self values
		select: [:each | each matchTextSearch: pattern]! !

!DGVCollection methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:18'!
name1
	^name! !

!DGVCollection methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:18'!
name: anObject
	name := anObject! !

!DGVCollection methodsFor: 'private' stamp: 'EG,TJ 3/22/2005 16:02'!
p2pAnnounceOnSave
	^false! !

!DGVCollection methodsFor: 'adding' stamp: 'EG,CdG 6/10/2005 12:16'!
removeWithoutPresenceUpdate: aUUID elementClass: aClass 
	| item |
	item _ items detect: [:each | each equalsObjectWithClass: aClass andUuid: aUUID] ifNone: [^self].
	self dirty: true.
	items remove: item.
	self signalActiveItemsChanged! !

!DGVCollection methodsFor: 'removing' stamp: 'CdG 11/25/2005 15:59'!
remove: aDGVObject 
	(items includes: aDGVObject) 
		ifTrue: 
			[self class announcementPresenceSet 
				addPresenceOf: (self collectionElementForRemoval: aDGVObject).
			self dirty: true.
			items remove: aDGVObject.
			self signalActiveItemsChanged]! !

!DGVCollection methodsFor: 'private' stamp: 'EG 12/10/2005 21:45'!
resolveValues
	| collectionChanged |
	collectionChanged := false.
	items do: [:each |
			each isDGVLink
				ifTrue: [
					[each value. 
					collectionChanged := true] on: DGVLinkCouldNotBeResolvedException do: []
				].
		].
	collectionChanged ifTrue: [self signalActiveItemsChanged].
	^ items	select: [:each | each notNil]! !

!DGVCollection methodsFor: 'private' stamp: 'CdG 5/2/2005 14:42'!
save
	self basicSave! !

!DGVCollection methodsFor: 'converting' stamp: 'CdG 10/5/2005 15:08'!
selectorsWeDelegateToActiveItems
	^#(
		#anyOne	
		#asSortedCollection: 
		#asSet
		#copyWithoutAll:
		#collect:
		#do: 
		#do:separatedBy:
		#ifEmpty:
		#ifNotEmpty:
		#includes: 
		#isEmpty 
		#notEmpty
		#reject: 
		#select: 
		#size
	)! !

!DGVCollection methodsFor: 'private' stamp: 'CdG 10/7/2005 11:20'!
signalActiveItemsChanged
	#DGVTestingCollectionStuff binding == true
		ifTrue: [
			"if a test wants this, we can do the event handling directly so the test doesn't have to wait for async stuff"
			self activeItemsChanged
		]
		ifFalse: [
			"fork, because activeItems forces a resolution of the links..."
			self class signalQueue nextPut: [self activeItemsChanged]]! !

!DGVCollection methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 14:02'!
size
	^self activeItems size! !

!DGVCollection methodsFor: 'private' stamp: 'CdG 7/4/2005 11:13'!
values
	| answer |
	answer _ items asOrderedCollection collect: [:each | 
		[each value]
			on: DGVLinkCouldNotBeResolvedException
			do: [nil]].
	^answer select: [:each | each notNil]! !

!DGVCollection methodsFor: 'accessing' stamp: 'EG,CdG 6/10/2005 11:25'!
version
	^0! !

DGVProtoObject subclass: #DGVObject
	instanceVariableNames: 'relations comments version deleted signer signature'
	classVariableNames: 'DGVEpoch'
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVObject subclass: #DGVBlob
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVBlob class methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:36'!
fromFileName: aString
	| fs data |
	fs _ FileStream readOnlyFileNamed: aString.
	fs binary.
	[data _ fs contentsOfEntireFile]
		ensure: [fs close].
	^self new 
		data: data;
		yourself! !

!DGVBlob class methodsFor: 'accessing' stamp: 'CdG 4/6/2005 18:23'!
getFromNetwork: aUUID 
	"For blobs we want to do something different..."
	DGVNode default isConnected ifFalse: [^nil].
	^DGVBlobTransfer get: aUUID! !

!DGVBlob class methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 11:29'!
storageName
	^'blob'! !

!DGVBlob class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 10:29'!
textSearch: aString
	^self textSearchNotImplementedForThisClass! !

!DGVBlob methodsFor: 'accessing' stamp: 'CdG 3/11/2005 11:20'!
blobInfo
	^DGVBlobInfo for: self! !

!DGVBlob methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 10:58'!
data
	^data! !

!DGVBlob methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 10:59'!
data: anObject
	data _ anObject asByteArray! !

!DGVBlob methodsFor: 'accessing' stamp: 'CdG,TJ 2/17/2005 22:38'!
delete
	data _ nil.
	^super delete! !

!DGVBlob methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 11:03'!
p2pAnnounceOnSave
	^false! !

!DGVBlob methodsFor: 'private' stamp: 'EG,CdG 7/1/2005 11:13'!
resetLastSavedTime
	lastSavedTime _ nil! !

!DGVBlob methodsFor: 'copying' stamp: 'EG,CdG 7/1/2005 11:14'!
stableCopyForChecksumming
	"for DGVBlobInfo, return a copy of myself with all the fields that don't matter
       for checksumming zeroed out"
	| answer |
	answer _ self copy.
	answer
		resetLastSavedTime.
	^answer! !

DGVObject subclass: #DGVComment
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVComment class methodsFor: 'instance creation' stamp: 'EG,CdG 7/4/2005 15:16'!
on: aDGVObject 
	| comment |
	comment _ self new.
	aDGVObject addComment: comment.
	^comment
! !

!DGVComment class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:16'!
storageName
	^ 'comment'! !

!DGVComment methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:21'!
matchTextSearch: pattern
	^self matchTextSearch: pattern inAttributes: #(text)! !

!DGVComment methodsFor: 'accessing' stamp: 'EG,CdG 7/4/2005 15:30'!
text
	^text! !

!DGVComment methodsFor: 'accessing' stamp: 'CdG 7/21/2005 13:38'!
text: aByteString 
	text _ aByteString.
	self markDirty! !

DGVObject subclass: #DGVCommunity
	instanceVariableNames: 'description owner users files newUserPolicy children communityName managers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVCommunity commentStamp: 'JJ 11/3/2005 11:10' prior: 0!
"To create new root community:
create a new UUID with:
	UUID new asString 
        DU: 	'e07cbde8-84ab-a947-8dba-9c2d6c6318cc' 
        DGV:	'098c6099-5d84-af49-9342-c633c3eb9c6e'
and poke this into:
	1. the local file rootcommunity.uuid
	2. DGVCommunity allRootCommunities
Then run the following code:
"

| root user |
root := DGVCommunity createRootCommunity.
user := DGVUser withUserName: 'administrator' password: 'AardW0rks'.
user fullName: 'DGV Super User'; email: 'kolibri@aardworks.nl'; save.
root acceptNewUser: user; owner: user; save!

!DGVCommunity class methodsFor: 'accessing' stamp: 'JJ 11/3/2005 11:10'!
allRootCommunities
	^ Dictionary new
		at: 'UvO'
		put: (UUID fromString: 'e07cbde8-84ab-a947-8dba-9c2d6c6318cc');
		
		at: 'DGV'
		put: (UUID fromString: '098c6099-5d84-af49-9342-c633c3eb9c6e');
		 yourself! !

!DGVCommunity class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 14:40'!
communitiesWithMember: aDGVUser 
	^self localStorage activeItems select: [:each| each users includes: aDGVUser]! !

!DGVCommunity class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 12:41'!
communitiesWithOwner: aDGVUser 
	^self localStorage activeItems select: [:each| each owner = aDGVUser]! !

!DGVCommunity class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
createRootCommunity
	"
		self createRootCommunity: 'UvO'
	"

	"should be called once, and only once, on dgv.aardworks.nl"

	| communityName |
	communityName := DGVApplicationConfiguration configName asString.
	^self createRootCommunity: communityName! !

!DGVCommunity class methodsFor: 'accessing' stamp: 'CdG 10/19/2005 12:58'!
createRootCommunity: aString
	"should be called once, and only once, on dgv.aardworks.nl"
	| policy |
	policy _ DGVNewUserPolicyConfirmEmail new.
	^(self new: aString)
		id: (self allRootCommunities at: aString ifAbsent: []);
		newUserPolicy: policy;
		save.! !

!DGVCommunity class methodsFor: 'as yet unclassified' stamp: 'CdG 10/19/2005 12:42'!
new: communityName
	^super new 
		communityName: communityName;
		newUserPolicy: DGVNewUserPolicyAlwaysAccept new! !

!DGVCommunity class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
rootCommunity
	^self get: DGVApplicationConfiguration rootCommunityId! !

!DGVCommunity class methodsFor: 'accessing' stamp: 'TJ 2/4/2005 09:55'!
rootCommunityAgreementURL
	^'http://dgv.aardworks.nl'! !

!DGVCommunity class methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 12:46'!
storageName
	^'community'! !

!DGVCommunity class methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
updateRootCommunity
	"stuff needed to bring the root community from one release to the next
		self updateRootCommunity
	"

	| policy |
	policy := DGVNewUserPolicyConfirmEmail new save.
	^(self get: DGVApplicationConfiguration rootCommunityId)
		newUserPolicy: policy;
		save! !

!DGVCommunity methodsFor: 'private' stamp: 'CdG 8/11/2005 11:33'!
acceptNewUser: aDGVUser 
	"actually semi-private - called by the new user policy upon acceptance"
	self users add: aDGVUser.
	self dirty: true; save! !

!DGVCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 16:16'!
addChild: aDGVCommunity 
	self children add: aDGVCommunity.
	self dirty: true! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:52'!
addFile: aDGVFile 
	self files add: aDGVFile.
	aDGVFile parent: self.
	self dirty: true
! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/8/2005 04:23'!
addManager: aDGVUser 
	self managers add: aDGVUser! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 4/13/2005 22:49'!
addUser: aDGVUser 
	^self newUserPolicy acceptNewUser: aDGVUser in: self		! !

!DGVCommunity methodsFor: 'api' stamp: 'CdG 6/8/2005 15:08'!
assertUniqueName: name 
	
	[self children do: 
			[:each | 
			each communityName = name ifTrue: [DGVDuplicateCommunityException signal]]] 
			on: DGVLinkCouldNotBeResolvedException
			do: [:ex | TDSDebuggerSupport logException: ex]! !

!DGVCommunity methodsFor: 'api' stamp: 'CdG 6/8/2005 15:08'!
assertValidDescription: theDescription 
	(theDescription isNil or: [theDescription isEmpty]) 
		ifTrue: [DGVEmptyDescriptionException signal]! !

!DGVCommunity methodsFor: 'api' stamp: 'CdG 6/8/2005 15:07'!
assertValidName: name 
	(name isNil or: [name isEmpty]) ifTrue: [DGVEmptyNameException signal]! !

!DGVCommunity methodsFor: 'testing' stamp: 'JJ 11/18/2005 10:49'!
canSeeFiles: aDGVUser
	| canSeeFiles |

	canSeeFiles := true.
	DGVApplicationConfiguration showFileTabWhenNotMember 
		ifFalse: [canSeeFiles := DGVApplication isLoggedIn 
			and: [aDGVUser isMemberOfCommunity: self]].

	^canSeeFiles! !

!DGVCommunity methodsFor: 'accessing' stamp: 'TJ 2/24/2005 15:06'!
children
	^self getCollectionAt: #children! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 2/10/2005 14:34'!
children: anObject
	children _ anObject! !

!DGVCommunity methodsFor: 'accessing' stamp: 'TJ,EG 2/16/2005 10:43'!
communityName
	^communityName ifNil: ['']! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 2/10/2005 15:13'!
communityName: anObject 
	communityName := anObject! !

!DGVCommunity methodsFor: 'api' stamp: 'CdG 10/19/2005 12:42'!
createSubcommunityNamed: name description: theDescription owner: aDGVUser policyname: policyname 
	| com policy |
	self assertValidName: name.
	self assertValidDescription: theDescription.
	self assertUniqueName: name.

	com := DGVCommunity new: name.
	policy := DGVNewUserPolicy withName: policyname.
	com
		description: theDescription;
		parent: self;
		owner: aDGVUser;
		acceptNewUser: aDGVUser;
		newUserPolicy: policy;
		save.
	self
		addChild: com;
		save.
	^com! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG 12/11/2005 20:17'!
delete
	self children copy do: [:each | each delete].
	self files do: [:each | each delete].
	parent ifNotNil: [parent removeChild: self].
	super delete.
! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG 11/24/2005 18:27'!
deleteFile: aDGVFile 
	self files remove: aDGVFile! !

!DGVCommunity methodsFor: 'printing' stamp: 'CdG 11/15/2005 15:39'!
describeYourself
	"answers a description for human beings, e.g. for confirmation mails"

	self isRootCommunity 
		ifTrue: [^DGVApplicationConfiguration rootCommunityDescription].
	description isNil 
		ifTrue: [^'een community met UUID ' , self id printString].
	^DGVApplicationConfiguration communityDescription format: { communityName }! !

!DGVCommunity methodsFor: 'accessing' stamp: 'TJ,EG 2/16/2005 13:29'!
description
	^description ifNil: ['']! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 2/10/2005 15:52'!
description: anObject
	description _ anObject.
	self dirty: true.! !

!DGVCommunity methodsFor: 'accessing' stamp: 'TJ 2/24/2005 15:06'!
files
	^self getCollectionAt: #files! !

!DGVCommunity methodsFor: 'private' stamp: 'CdG,TJ 2/3/2005 13:24'!
files: anObject
	files _ anObject! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:41'!
initialize
	super initialize.
	files _ DGVCollection parent: self named: 'files'.
	children _ DGVCollection parent: self named: 'children'.
	users _ DGVCollection parent: self named: 'users'.! !

!DGVCommunity methodsFor: 'testing' stamp: 'CdG 2/10/2005 16:12'!
isAuthorized: aDGVUser
	^owner = aDGVUser or: [self users includes: aDGVUser]! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'JJ 11/22/2005 16:41'!
isManager: aDGVUser 

	(self owner = aDGVUser or: [self managers includes: aDGVUser]) 
		ifTrue: [^true].

	^self parent isNil 
		ifFalse: [self parent isManager: aDGVUser]
		ifTrue: [false]! !

!DGVCommunity methodsFor: 'testing' stamp: 'CdG 11/15/2005 15:39'!
isRootCommunity
	^self id = DGVApplicationConfiguration rootCommunityId! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/8/2005 04:24'!
managers
	^self getCollectionAt: #managers! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:22'!
matchTextSearch: pattern 
	^ self matchTextSearch: pattern inAttributes: #(#communityName description)! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 11/2/2005 00:04'!
newUserPolicy
	^newUserPolicy! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 2/10/2005 10:18'!
newUserPolicy: anObject
	newUserPolicy _ anObject.
	self dirty: true! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 2/10/2005 14:32'!
owner
	^owner! !

!DGVCommunity methodsFor: 'accessing' stamp: 'CdG 2/10/2005 15:53'!
owner: anObject
	owner _ anObject.
	self dirty: true! !

!DGVCommunity methodsFor: 'printing' stamp: 'CdG 8/12/2005 11:11'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self communityName; nextPut: $)! !

!DGVCommunity methodsFor: 'private' stamp: 'JJ 11/24/2005 16:30'!
refresh

	DGVCollection gatherElementsFromNetworkFor: self files.
	DGVCollection gatherElementsFromNetworkFor: self users.
	DGVCollection gatherElementsFromNetworkFor: self children.! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG 11/24/2005 17:30'!
removeChild: aDGVCommunity 
	self children remove: aDGVCommunity! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/8/2005 04:55'!
removeManager: aDGVUser 
	self managers remove: aDGVUser! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'EG 11/1/2005 19:02'!
removeUser: aDGVUser 
	self users remove: aDGVUser.
	self removeManager: aDGVUser.! !

!DGVCommunity methodsFor: 'printing' stamp: 'CdG 7/11/2005 20:17'!
stringRepresentation
	^'{1} "{2}"' format: {'Community' translated. self communityName}! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 12:36'!
textSearchWithChildren: aString 
	| answer pattern |
	answer := OrderedCollection new.
	pattern := aString asLowercase substrings.

	(self matchTextSearch: pattern)
		ifTrue: [answer add: self].

	answer
		addAll: (self files matchesForTextSearch: pattern);	
		addAll: (self users matchesForTextSearch: pattern).

	self children values do: [:each |
		answer addAll: (each textSearchWithChildren: aString)].

	^ answer! !

!DGVCommunity methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 13:47'!
textSearchWithChildren: aString forUser: aDGVUser
	| answer pattern |

	answer := OrderedCollection new.
	pattern := aString asLowercase substrings.

	(aDGVUser isNil not and: [aDGVUser isMemberOfCommunity: self]) ifTrue: [
		(self matchTextSearch: pattern)
			ifTrue: [answer add: self].

		answer
			addAll: (self files matchesForTextSearch: pattern);	
			addAll: (self users matchesForTextSearch: pattern)].

	self children values do: [:each |
		answer addAll: (each textSearchWithChildren: aString forUser: aDGVUser)].

	^ answer! !

!DGVCommunity methodsFor: 'accessing' stamp: 'TJ 2/24/2005 15:06'!
users
	^self getCollectionAt: #users! !

!DGVCommunity methodsFor: 'private' stamp: 'CdG,TJ 2/3/2005 13:19'!
users: anObject
	users _ anObject! !

DGVObject subclass: #DGVFolder
	instanceVariableNames: 'folderName files subFolders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVFolder class methodsFor: 'as yet unclassified' stamp: 'CdG 9/30/2005 10:09'!
storageName
	^'folder'! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 10/7/2005 11:08'!
addFile: aDGVFile 
	^self files add: aDGVFile! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 9/30/2005 10:46'!
addSubFolder: aDGVFolder 
	aDGVFolder parent: self; markDirty; save.
	^self subFolders add: aDGVFolder! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 9/30/2005 09:55'!
files
	^self getCollectionAt: #files! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 9/30/2005 09:54'!
folderName
	^folderName! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 9/30/2005 10:07'!
folderName: aByteString 
	self markDirty.
	folderName := aByteString! !

!DGVFolder methodsFor: 'as yet unclassified' stamp: 'CdG 9/30/2005 11:17'!
populateSubtree: aWxTreeItem in: aDGVMainTree
	self subFolders do: [:child |
		aDGVMainTree populateSubTree: aWxTreeItem withFolder: child]! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 10/7/2005 11:07'!
removeFile: aDGVFile 
	^self files remove: aDGVFile! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 9/30/2005 10:07'!
removeSubFolder: aDGVFolder 
	^self subFolders remove: aDGVFolder! !

!DGVFolder methodsFor: 'accessing' stamp: 'CdG 9/30/2005 09:56'!
subFolders
	^self getCollectionAt: #subFolders! !

DGVObject subclass: #DGVMockObject
	instanceVariableNames: 'link collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVMockObject class methodsFor: 'accessing' stamp: 'CdG 2/14/2005 15:26'!
storageName
	^'test'! !

!DGVMockObject class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 11:33'!
textSearch: aString
	^self textSearchNotImplementedForThisClass! !

!DGVMockObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 11:27'!
collection
	^collection! !

!DGVMockObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 11:27'!
collection: anObject
	collection := anObject! !

!DGVMockObject methodsFor: 'testing' stamp: 'EG,CdG 6/13/2005 11:51'!
lastSavedTime: aTimeStamp 
	lastSavedTime _ aTimeStamp! !

!DGVMockObject methodsFor: 'accessing' stamp: 'CdG 2/14/2005 15:32'!
link
	^link! !

!DGVMockObject methodsFor: 'accessing' stamp: 'CdG 2/14/2005 15:28'!
link: anObject
	link _ anObject! !

!DGVMockObject methodsFor: 'testing' stamp: 'CdG 11/29/2005 10:28'!
p2pAnnounceOnSave
	^false! !

DGVObject subclass: #DGVNodeStatistics
	instanceVariableNames: 'nodeId totalBlobDownloads'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVNodeStatistics class methodsFor: 'accessing' stamp: 'CdG 10/26/2005 15:51'!
forDefaultNode
	| nodeId |
	nodeId := DGVNode default nodeId.
	^self localStorage activeItems 
		detect: [:each | each nodeId = nodeId]
		ifNone: [self new nodeId: nodeId; save]! !

!DGVNodeStatistics class methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:51'!
storageName
	^'nodestatistics'! !

!DGVNodeStatistics methodsFor: 'incrementing' stamp: 'CdG 10/26/2005 15:55'!
incrementTotalBlobDownloads
	totalBlobDownloads := self totalBlobDownloads + 1! !

!DGVNodeStatistics methodsFor: 'accessing' stamp: 'CdG 10/26/2005 15:55'!
nodeId
	^nodeId! !

!DGVNodeStatistics methodsFor: 'accessing' stamp: 'CdG 10/26/2005 15:52'!
nodeId: anUUID 
	nodeId := anUUID! !

!DGVNodeStatistics methodsFor: 'accessing-statistics' stamp: 'CdG 10/26/2005 15:54'!
totalBlobDownloads
	^totalBlobDownloads ifNil: [0]! !

DGVObject subclass: #DGVOAICollection
	instanceVariableNames: 'oaiClient collectionName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVOAICollection class methodsFor: 'creation' stamp: 'EG,CdG 9/29/2005 10:46'!
storageName
	^'oaicollection'! !

!DGVOAICollection class methodsFor: 'creation' stamp: 'EG 6/20/2005 14:42'!
with: aDGVOAIClient 
	^self new oaiClient: aDGVOAIClient! !

!DGVOAICollection methodsFor: 'accessing' stamp: 'EG,CdG 9/29/2005 11:42'!
baseUrl
	^oaiClient baseUrl! !

!DGVOAICollection methodsFor: 'as yet unclassified' stamp: 'EG 6/20/2005 16:32'!
calculateIdFor: aString 
	"Calculate ID based on the baseUrl and the OAI identifier to make sure
	that records keep the same ID when harvested repeatedly."
	| hash |
	hash := SecureHashAlgorithm new hashMessage: oaiClient baseUrl , aString.
	^ UUID fromSHAHash: hash! !

!DGVOAICollection methodsFor: 'accessing' stamp: 'EG 6/20/2005 14:49'!
collectionName
	^ collectionName ifNil: [collectionName _ oaiClient identify]! !

!DGVOAICollection methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:54'!
harvest
	oaiClient listRecords
		do: [:each | 
			(DGVOAIRecord from: each) 
				id: (self calculateIdFor: each header identifier);
				parent: self;
				save]! !

!DGVOAICollection methodsFor: 'accessing' stamp: 'EG,CdG 9/29/2005 11:41'!
latestSuccesfulHarvestDate
	^oaiClient latestSuccesfulHarvestDate! !

!DGVOAICollection methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/29/2005 11:43'!
localRecords
	^DGVOAIRecord localStorage select: [:each| each parent = self]! !

!DGVOAICollection methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/29/2005 11:11'!
matchTextSearch: anArray 
	^false! !

!DGVOAICollection methodsFor: 'accessing' stamp: 'EG 6/20/2005 10:55'!
oaiClient: aDGVOAIClient 
	oaiClient _ aDGVOAIClient! !

!DGVOAICollection methodsFor: 'accessing' stamp: 'EG,CdG 9/29/2005 11:26'!
setupNotebookIn: aDGVMainWindow 
	aDGVMainWindow setupNotebookForOAICollection: self! !

DGVObject subclass: #DGVObjectWithMetadata
	instanceVariableNames: 'dc'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVObjectWithMetadata subclass: #DGVFile
	instanceVariableNames: 'originalFileName blob'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVFile class methodsFor: 'accessing' stamp: 'EG,JJ 10/25/2005 14:36'!
create: originalFilename
mimeType: mimeType
user: user
	| answer |
	answer _ DGVFile new.
	answer
		originalFilename: originalFilename.
	answer metadata
		publisher: user;
		date: Date today yyyymmdd, ' ', Time now print24;
		identifier: answer id printString;
		format: mimeType.
	^ answer! !

!DGVFile class methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 11:29'!
storageName
	^'file'! !

!DGVFile methodsFor: 'initialize-release' stamp: 'CdG 7/27/2005 11:43'!
addBlobFrom: aString announce: aBoolean
	self blob: (DGVBlob fromFileName: aString).
	self save.
	blob save.
	aBoolean 
		ifTrue: [blob p2pAnnounceNewVersion].
! !

!DGVFile methodsFor: 'accessing' stamp: 'TJ 1/21/2005 10:28'!
asString
	^ self id! !

!DGVFile methodsFor: 'accessing' stamp: 'CdG 3/9/2005 12:40'!
blob
	^blob value! !

!DGVFile methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 15:47'!
blobId
	^blob id! !

!DGVFile methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:35'!
blob: anObject
	blob _ anObject.
	blob parent: self.! !

!DGVFile methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 10:58'!
contents
	^self blob data! !

!DGVFile methodsFor: 'actions' stamp: 'CdG 3/14/2005 15:32'!
extension
	^(self originalFilename subStrings: '.') last! !

!DGVFile methodsFor: 'initialize-release' stamp: 'CdG,TJ 2/3/2005 10:16'!
initialize
	super initialize.
	dc _ DGVDublinCore new.! !

!DGVFile methodsFor: 'testing' stamp: 'TJ 3/30/2005 13:28'!
isBlobLocallyAvailable
	^DGVBlob localStorage exists: blob id! !

!DGVFile methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:34'!
matchTextSearch: pattern 
	^ self matchTextSearch: pattern inAttributes: #(#originalFilename ) , DGVDublinCore dcAttributeNames! !

!DGVFile methodsFor: 'accessing' stamp: 'TJ 1/20/2005 13:57'!
originalFilename
	^originalFileName! !

!DGVFile methodsFor: 'accessing' stamp: 'CdG 2/14/2005 09:38'!
originalFilename: aString 
	originalFileName _ aString.
! !

!DGVFile methodsFor: 'actions' stamp: 'JJ 11/22/2005 16:04'!
osOpen
	| contents |
	self blob ifNil: [^self error: 'Dit bestand bevat geen gegevens.'].

	contents _ self blob data.
	(DGVOSSupport default openTempFileWithName: originalFileName contents: contents) = 42
		ifFalse: [self displayOpenErrorMessageWindow]
! !

!DGVFile methodsFor: 'as yet unclassified' stamp: 'JJ 11/22/2005 16:05'!
osOpenWithoutResolving
	| contents |
	contents _ (DGVBlob localStorage get: self blobId) data.
	(DGVOSSupport default openTempFileWithName: originalFileName contents: contents) = 42
		ifFalse: [self displayOpenErrorMessageWindow]
! !

!DGVFile methodsFor: 'actions' stamp: 'CdG 2/8/2005 14:58'!
storeContentsOnPath: aString
	| fs |
	fs _ FileStream forceNewFileNamed: aString.
	[fs nextPutAll: self blob data]
		ensure: [fs close]! !

!DGVFile methodsFor: 'printing' stamp: 'EG,JJ 10/25/2005 14:32'!
stringRepresentation
	^'Bestand' translated, ' "{1}" ({2})' 
		format: {self metadata title. self originalFilename}! !

DGVObjectWithMetadata subclass: #DGVOAIRecord
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVOAIRecord class methodsFor: 'instance creation' stamp: 'EG 6/20/2005 12:28'!
from: anOAIRecord 
	^ self new from: anOAIRecord! !

!DGVOAIRecord class methodsFor: 'instance creation' stamp: 'EG 6/20/2005 12:14'!
storageName
	^'oairecord'! !

!DGVOAIRecord methodsFor: 'accessing' stamp: 'EG 6/20/2005 12:29'!
from: anOAIRecord 
	dc _ DGVDublinCore fromOAIDC: anOAIRecord metadata! !

!DGVOAIRecord methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:23'!
matchTextSearch: pattern 
	^ self matchTextSearch: pattern inAttributes: DGVDublinCore dcAttributeNames! !

!DGVOAIRecord methodsFor: 'accessing' stamp: 'EG,JJ 10/25/2005 14:30'!
metadata
	^dc ifNil: [dc _ DGVDublinCore new]! !

!DGVOAIRecord methodsFor: 'accessing' stamp: 'EG,CdG 9/29/2005 10:33'!
navigateTreeIn: aDGVMainWindow
	aDGVMainWindow doNavigateToDGVOAIRecord: self! !

!DGVOAIRecord methodsFor: 'accessing' stamp: 'CdG 11/29/2005 11:36'!
renderHtmlReferenceForRelations
	^'<b>{1}:</b> <a href="command:navigateReference:{2}">{3}</a>'
		format: {
			'Record' translated.
			self qualifiedReference.
			self metadata title.
			}! !

!DGVOAIRecord methodsFor: 'accessing' stamp: 'EG,JJ 10/25/2005 14:34'!
stringRepresentation
	^'{1} "{2}"' format: {'Record' translated. self metadata title}! !

!DGVObjectWithMetadata methodsFor: 'accessing' stamp: 'EG,JJ 10/25/2005 14:41'!
doesNotUnderstand: aMessage
	aMessage arguments isEmpty 
		ifFalse: [^super doesNotUnderstand: aMessage].
	(self metadata class dcAttributeNames includes: aMessage selector) 
		ifFalse: [^super doesNotUnderstand: aMessage].

	^self metadata perform: aMessage selector! !

!DGVObjectWithMetadata methodsFor: 'accessing' stamp: 'EG,JJ 10/25/2005 14:27'!
metadata
	^dc! !

!DGVObjectWithMetadata methodsFor: 'testing support' stamp: 'EG,JJ 10/25/2005 14:28'!
metadata: aDGVDublinCore 
	dc := aDGVDublinCore! !

!DGVObject class methodsFor: 'accessing' stamp: 'CdG 12/10/2005 21:07'!
getFromNetwork: aUUID withVersion: aNumber
	DGVNode default isConnected ifFalse: [^nil].
	^self serializeNetworkSearch: 
			[DGVQueryStorageGet get: aUUID from: self storageName withVersion: aNumber]
		forId: aUUID! !

!DGVObject class methodsFor: 'accessing' stamp: 'CdG 12/10/2005 21:08'!
get: aUUID
	| answer |
	"1. Local storage"
	answer _ self getLocally: aUUID mayRefreshFromNetwork: true.
	answer ifNotNil: [^answer].
	"2. Network."
	answer _ self getFromNetwork: aUUID withVersion: nil.
	answer ifNotNil: [
		answer updateLastSavedTime.
		self localStorage put: answer].
	^answer! !

!DGVObject class methodsFor: 'accessing' stamp: 'CdG 12/14/2005 12:35'!
get: aUUID withVersion: aNumber
	| answer | 
	"1. Local storage"
	answer _ self getLocally: aUUID mayRefreshFromNetwork: true.
	answer ifNotNil: [
		(aNumber isNil or: [answer version >= aNumber]) ifTrue: [^answer]].
	"2. Network."
	answer _ self getFromNetwork: aUUID withVersion: aNumber.
	answer ifNotNil: [
		(aNumber isNil or: [answer version >= aNumber]) ifTrue: [
			answer updateLastSavedTime.
			self localStorage put: answer.
			^answer]].
	^nil! !

!DGVObject class methodsFor: 'accessing' stamp: 'CdG 3/9/2005 13:47'!
presenceSetName
	^'nl.aardworks.dgv.writetransactions'! !

!DGVObject class methodsFor: 'accessing' stamp: 'EG 12/10/2005 15:38'!
refresh: aDGVObject 
	| newObject |
	newObject := self getFromNetwork: aDGVObject id.
	newObject ifNil: 
			[aDGVObject updateLastSavedTime.
			aDGVObject localStorage put: aDGVObject.
			^aDGVObject].

	(newObject isNewerThan: aDGVObject) ifTrue: [
		aDGVObject become: newObject
	].

	aDGVObject updateLastSavedTime.
	aDGVObject localStorage put: aDGVObject.
	^aDGVObject! !

!DGVObject class methodsFor: 'as yet unclassified' stamp: 'CdG 11/1/2005 23:59'!
startOfDGVEpoch
	^DGVEpoch ifNil: [DGVEpoch := AWGTimeStamp fromString: '11-1-2005 18:00']! !

!DGVObject class methodsFor: 'as yet unclassified' stamp: 'CdG 11/29/2005 10:42'!
textSearchAllClasses: aString 
	| result |
	result _ Set new.
	self allSubclasses do: [:class |
		result addAll: (class textSearch: aString)].
	^result! !

!DGVObject class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 10:29'!
textSearchNotImplementedForThisClass
	"for documentation purposes, call this if text search does not make sense"
	^OrderedCollection new! !

!DGVObject class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/29/2005 11:09'!
textSearch: aString
	| pattern |
	self isDGVAbstractClass ifTrue: [^#()].
	pattern := aString asLowercase substrings.
	^ self localStorage activeItems
		select: [:each | each matchTextSearch: pattern]! !

!DGVObject methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:49'!
addComment: aDGVComment 
	self comments add: aDGVComment.
	aDGVComment parent: self.! !

!DGVObject methodsFor: 'relations' stamp: 'CdG 5/2/2005 14:06'!
addRelation: aDGVRelation 
	"called by DGVRelation on #add"

	self relations add: aDGVRelation! !

!DGVObject methodsFor: 'private' stamp: 'TJ 3/30/2005 13:37'!
basicSave
	self dirty 
		ifTrue: [
			self dirty: false.
			self increaseVersion.
			self updateLastSavedTime.
			self p2pAnnounceOnSave ifTrue: [
				self p2pAnnounceNewVersion]].
	self localStorage basicPut: self! !

!DGVObject methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 13:43'!
calculateSignature
	^signer sign: self hashForSigning! !

!DGVObject methodsFor: 'accessing' stamp: 'EG,CdG 7/4/2005 15:18'!
comments
	^self getCollectionAt: #comments! !

!DGVObject methodsFor: 'accessing' stamp: 'CdG 7/27/2005 10:51'!
delete
	deleted _ true.
	self dirty: true; save.
	self comments do: [:each | 
		each delete].
! !

!DGVObject methodsFor: 'accessing' stamp: 'EG 10/31/2005 15:13'!
deleted
	^(deleted ifNil: [false]) or: [(self creationTime)
						< (DGVObject startOfDGVEpoch)]! !

!DGVObject methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 12:21'!
fillHashStream: aWriteStream 
	"Subclasses should add their own stuff and make sure to do a super call"
	aWriteStream
		nextPutAll: id printString;
		nextPutAll: version printString;
		nextPutAll: deleted printString.! !

!DGVObject methodsFor: 'private' stamp: 'CdG 7/29/2005 12:51'!
getCollectionAt: aSymbol
	| answer |
	answer _ self instVarNamed: aSymbol.
	answer isDGVLink ifTrue: [[
			answer _ answer value]
				on: DGVLinkCouldNotBeResolvedException
				do: [answer _ nil]].
	answer ifNil: [
		"Lazy initialize is ok because we can determinalistically create collection uuids"
		self 
			instVarNamed: aSymbol 
			put: (answer _  self 
					instVarNamed: aSymbol 
					put: (DGVCollection parent: self named: aSymbol asString))].
	^answer! !

!DGVObject methodsFor: 'comparing' stamp: 'CdG,TJ 2/3/2005 12:54'!
hash
	^id hash! !

!DGVObject methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 12:22'!
hashForSigning
	| stream |
	stream _ ReadWriteStream on: String new.
	self fillHashStream: stream.
	^SecureHashAlgorithm new hashStream: stream reset.! !

!DGVObject methodsFor: 'private' stamp: 'JJ 12/8/2005 19:54'!
increaseVersion
	| minorVersion majorVersion |

	majorVersion := (version floor + 1) asString.
	minorVersion := AWGTimeStamper new now asSeconds asString.
	version := (majorVersion, '.', minorVersion) asNumber
! !

!DGVObject methodsFor: 'initialize-release' stamp: 'EG,CdG 7/4/2005 15:25'!
initialize
	super initialize.
	id _ UUID new.
	version _ 0.
	relations _ DGVCollection parent: self named: 'relations'.
	comments _ DGVCollection parent: self named: 'comments'! !

!DGVObject methodsFor: 'testing' stamp: 'CdG 12/10/2005 21:46'!
isDGVObject
	^true! !

!DGVObject methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 19:57'!
isNewerThan: aDGVObject 
	(self isKindOf: aDGVObject class) ifFalse: [^false].

	^self id = aDGVObject id and: [self version > aDGVObject version]! !

!DGVObject methodsFor: 'testing' stamp: 'CdG 7/11/2005 20:26'!
isRelatedTo: aDGVObject 
	^(self relations select: [:relation |
		relation includes: aDGVObject])
		notEmpty! !

!DGVObject methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:10'!
matchTextSearch: pattern inAttributes: attributes
	| results value |
	results := attributes
		select: [:attr | 
			value := self perform: attr.
			value dgvMatchTextSearch: pattern].
	^results notEmpty! !

!DGVObject methodsFor: 'accessing' stamp: 'TJ 2/4/2005 16:31'!
notDeleted
	^self deleted not! !

!DGVObject methodsFor: 'testing' stamp: 'CdG 3/9/2005 12:33'!
originIsLocal
	^self class localStorage exists: self id! !

!DGVObject methodsFor: 'private' stamp: 'CdG 12/10/2005 21:21'!
p2pAnnounceNewVersion
	| set link currentEntry |
	set := self class announcementPresenceSet.
	link := "DGVLink for:" self.
	currentEntry := set entryAt: link.
	currentEntry ifNil: [set addPresenceOf: link]
		ifNotNil: [set publishNewVersionOf: currentEntry withObject: link]! !

!DGVObject methodsFor: 'private' stamp: 'CdG 3/10/2005 11:02'!
p2pAnnounceOnSave
	^true! !

!DGVObject methodsFor: 'relations' stamp: 'CdG 5/2/2005 14:06'!
relations
	^self getCollectionAt: #relations! !

!DGVObject methodsFor: 'private' stamp: 'CdG 7/12/2005 12:24'!
save
	| instVar |
	1 to: self class instSize
		do: 
			[:i | 
			instVar _ self instVarAt: i.
			(instVar isKindOf: DGVCollection) 
				ifTrue: [instVar dirty ifTrue: [instVar basicSave]]].
	self basicSave! !

!DGVObject methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 13:53'!
signatureIsValid
	^signer verifySignature: signature ofMessageHash: self hashForSigning! !

!DGVObject methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 13:43'!
signBy: aDGVUser 
	signer := aDGVUser.
	signature := self calculateSignature! !

!DGVObject methodsFor: 'accessing' stamp: 'CdG,TJ 2/15/2005 12:26'!
signer
	^signer! !

!DGVObject methodsFor: 'printing' stamp: 'CdG 7/11/2005 18:47'!
stringRepresentation
	^self qualifiedReference! !

!DGVObject methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 14:18'!
version
	^version! !

!DGVObject methodsFor: 'private' stamp: 'CdG 3/31/2005 22:46'!
version: aSmallInteger 
	version _ aSmallInteger! !

!DGVObject methodsFor: 'comparing' stamp: 'CdG,EjG 6/8/2005 16:08'!
= anObject
	^(anObject respondsTo: #id) and: [self id = anObject id]! !

DGVObject subclass: #DGVRelation
	instanceVariableNames: 'objects description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVRelation class methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 10:31'!
storageName
	^'relation'! !

!DGVRelation class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 10:30'!
textSearch: aString
	^self textSearchNotImplementedForThisClass! !

!DGVRelation methodsFor: 'adding' stamp: 'CdG 5/2/2005 15:07'!
addAll: aCollection
	aCollection do: [:each | self add: each]! !

!DGVRelation methodsFor: 'adding' stamp: 'CdG 7/29/2005 13:39'!
add: aDGVObject
	objects add: aDGVObject.
	aDGVObject addRelation: self.
	self markDirty! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:02'!
description
	^description! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 7/29/2005 13:39'!
description: anObject
	description := anObject.
	self markDirty! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 7/11/2005 20:25'!
includes: aDGVObject
	^self objects includes: aDGVObject! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:58'!
initialize
	super initialize.
	objects _ DGVCollection parent: self named: 'objects'! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:03'!
objects
	^self getCollectionAt: #objects! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:59'!
parent
	self objects isEmpty ifTrue: [^nil].

	^self objects anyOne! !

!DGVRelation methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:58'!
parent: aDGVObject
	self shouldNotImplement! !

!DGVRelation methodsFor: 'as yet unclassified' stamp: 'CdG 7/29/2005 12:39'!
removeAll
	self objects copy do: [:each | self remove: each]! !

!DGVRelation methodsFor: 'removing' stamp: 'CdG 7/29/2005 14:10'!
remove: anObject
	(objects includes: anObject) ifTrue: [
		objects remove: anObject.
		anObject relations remove: self.
		self markDirty]! !

DGVObject subclass: #DGVService
	instanceVariableNames: ''
	classVariableNames: 'DGCPolicyMailServiceRequest ServiceRegistry'
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVService commentStamp: '<historical>' prior: 0!
I serve DGVPolicies so they can send mail.!

DGVService subclass: #DGVCollectionService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVCollectionService methodsFor: 'as yet unclassified' stamp: 'EG 11/23/2005 15:16'!
entryAdded: anAWGPresenceEntry
	| coll collectionElement |
	anAWGPresenceEntry origin = self localNode nodeId ifTrue: [^self].
	collectionElement _ anAWGPresenceEntry object.
	collectionElement elementClass ifNil: [^self].
	collectionElement elementId ifNil: [^self].
	coll _ DGVCollection 
		getLocally: collectionElement collectionId 
		mayRefreshFromNetwork: false.
	coll ifNil: [coll _ DGVCollection newWithId: collectionElement collectionId].

	collectionElement isAddition 
		ifTrue: [coll 
			addWithoutPresenceUpdate: collectionElement elementId
			elementClass: collectionElement elementClass;
			save]
		ifFalse: [coll
			removeWithoutPresenceUpdate: collectionElement elementId
			elementClass: collectionElement elementClass;
			save]! !

!DGVCollectionService methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:09'!
entryRemoved: anAWGPresenceEntry
	"removals should be done using CollectionElement with flag set to removal"! !

!DGVCollectionService methodsFor: 'as yet unclassified' stamp: 'CdG 11/25/2005 15:59'!
start
	DGVCollection announcementPresenceSet 
		when: #entryAdded
		send: #entryAdded:
		to: self! !

DGVService subclass: #DGVHoarderService
	instanceVariableNames: 'fetchQueue fetchProcesses hoardOnNetwork'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVHoarderService commentStamp: 'CdG 3/9/2005 13:46' prior: 0!
This service hoards data. It just loves to collect data and drop it into the local cahe. So it hooks into the presence announcements and indiscriminately fetches everything.!

DGVHoarderService subclass: #DGVCleanerService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVCleanerService class methodsFor: 'as yet unclassified' stamp: 'CdG 4/14/2005 01:45'!
concurrentProcessCount
	^1! !

!DGVCleanerService methodsFor: 'as yet unclassified' stamp: 'CdG 12/10/2005 22:47'!
fetchRun
	| entry linkOrObject currentObject storage |
	
	[entry := self fetchQueue next.
	linkOrObject := entry object.
	"if we have an older version, clean it out"
	
	currentObject := (storage := DGVStorage named: linkOrObject storageName) get: linkOrObject id.
	currentObject 
		ifNil: [
			(linkOrObject isKindOf: DGVObject) 
				ifTrue: 
					[linkOrObject
						updateLastSavedTime;
						save]]
		ifNotNil: 
			[currentObject version < linkOrObject version 
				ifTrue: 
					[(linkOrObject isKindOf: DGVObject) 
						ifTrue: 
							[linkOrObject
								updateLastSavedTime;
								save]
						ifFalse: 
							[currentObject updateLastSavedTimeAsExpired.
							storage put: currentObject]]]] 
			on: Error
			do: [:ex | TDSDebuggerSupport logException: ex]! !

!DGVHoarderService class methodsFor: 'as yet unclassified' stamp: 'CdG 4/14/2005 01:45'!
concurrentProcessCount
	^5! !

!DGVHoarderService methodsFor: 'as yet unclassified' stamp: 'CdG 12/16/2005 00:14'!
doNetworkFetchOf: link from: originatingNode 
	| resolvedObject |
	resolvedObject := nil.
	link myclass = DGVBlob 
		ifTrue: 
			[resolvedObject := DGVBlobTransfer get: link id]
		ifFalse: 
			["we have an origin of this announcement - try to resolve this with unicast first"

			resolvedObject := DGVQueryStorageGet 
						get: link id
						from: link storageName
						withVersion: link version
						atNode: originatingNode.
			resolvedObject ifNotNil: 
					[resolvedObject version < link version ifTrue: [resolvedObject := nil]].
			resolvedObject ifNil: 
					["nope. No go. Broadcast"

					[resolvedObject := link value] on: DGVLinkCouldNotBeResolvedException
						do: [resolvedObject := nil]]].
	resolvedObject ifNotNil: 
			[resolvedObject version < link version ifTrue: [resolvedObject := nil]].
	resolvedObject ifNil: 
			["we may go stubborn here and try it a little later...."

			^self fixMe: 'add retry queue in DGVHoarderService'].
	resolvedObject updateLastSavedTime.
	resolvedObject localStorage put: resolvedObject! !

!DGVHoarderService methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 10:49'!
entryAdded: anAWGPresenceEntry
	"hand off to a queue in order not to disturb presence itself. Ignore locally originated stuff"
	anAWGPresenceEntry origin ~= self localNode nodeId ifTrue: [
		self fetchQueue nextPut: anAWGPresenceEntry]! !

!DGVHoarderService methodsFor: 'as yet unclassified' stamp: 'CdG 11/21/2005 21:22'!
fetchQueue
	^fetchQueue ifNil: [fetchQueue _ SharedBufferStream newObjectStream]! !

!DGVHoarderService methodsFor: 'as yet unclassified' stamp: 'CdG 12/10/2005 22:33'!
fetchRun
	| entry link currentCopy |
	entry := self fetchQueue next.
	(entry respondsTo: #object) ifFalse: 
			[^TDSDebuggerSupport 
				logError: 'Hoarder Service received invalid data (not an PresenceEntry: ' 
						, entry printString , ').'].
	link := entry object.
	link isDGVObject 
		ifTrue: 
			[currentCopy := link class getLocally: link id mayRefreshFromNetwork: false]
		ifFalse: 
			[link isDGVLink 
				ifFalse: 
					[^TDSDebuggerSupport 
						logError: 'Hoarder Service received invalid data (not a DGVLink: ' 
								, link printString , ').'].
			currentCopy := link myclass getLocally: link id
						mayRefreshFromNetwork: false].
	currentCopy 
		ifNotNil: [currentCopy version > link version ifTrue: [^self	"up-to-date"]].
	link isDGVObject ifTrue: [^link updateLastSavedTime; save].
	self doNetworkFetchOf: link from: entry origin! !

!DGVHoarderService methodsFor: 'as yet unclassified' stamp: 'CdG 11/25/2005 15:59'!
start
	| proc |
	DGVObject announcementPresenceSet 
		when: #entryAdded
		send: #entryAdded:
		to: self.
	fetchProcesses := (1 to: self class concurrentProcessCount) collect: 
					[:i | 
					proc := [[self fetchRun] repeat] 
								forkAt: Processor systemBackgroundPriority.
					proc name: self class name , ' fetcher ' , i printString.
					proc]! !

!DGVHoarderService methodsFor: 'as yet unclassified' stamp: 'CdG 4/14/2005 01:44'!
stop
	fetchProcesses ifNotNil: [
		fetchProcesses do: [:each | each terminate].
		fetchProcesses _ nil]! !

DGVService subclass: #DGVPolicyService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

DGVPolicyService subclass: #DGVPolicyConfirmUser
	instanceVariableNames: 'randomKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 7/6/2005 13:46'!
acceptanceFailedOfUser: aDGVUserId in: aDGVCommunityId
	self fixMe: 'we should report an error back!!'.
	TDSDebuggerSupport logError: 'acceptance failed b/c uid and/or cid not known!!'.
	^nil! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 7/6/2005 13:44'!
acceptNewUser: aDGVUserId in: aDGVCommunityId 
	"mail out a confirmation to the user"
	| aDGVUser aDGVCommunity |
	aDGVUser _ DGVUser get: aDGVUserId.
	aDGVCommunity _ DGVCommunity get: aDGVCommunityId.
	(aDGVUser isNil or: [aDGVCommunity isNil])
		ifTrue: [^self acceptanceFailedOfUser: aDGVUserId in: aDGVCommunityId].
	self mailConfirmationTo: aDGVUser about: aDGVUser for: aDGVCommunity.
	^ nil"acceptance is in limbo"! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 3/25/2005 20:11'!
buildConfirmationLinkTo: aDGVUser for: aDGVCommunity 
	| link |
	self fixMe: 'too much hard-coded stuff'.
	link _ DGVConfirmationMailAcceptanceService baseUrl, '?'.
	link _ link, 'userId=', aDGVUser id asString36.
	link _ link, '&communityId=', aDGVCommunity id asString36.
	link _ link, '&sign=', (self signWith: aDGVUser id with: aDGVCommunity id).
	link _ link, '&service=', (self id asString36).
	
	^link! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 3/25/2005 20:42'!
buildHtmlConfirmationInto: aHtmlBuilder
	aHtmlBuilder
		html: 'Uw bevestiging is ontvangen en verwerkt. U heeft nu de door u gewenste toegang.'..! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:40'!
buildMessageTextAbout: aDGVUser for: aDGVCommunity 
	| strm |
	strm := WriteStream on: String new.
	strm
		nextPutAll: 'U heeft zich aangemeld voor deelname aan ';
		nextPutAll: aDGVCommunity describeYourself;
		nextPutAll: '. Hiervoor is het vereist om een geldig email adres te bezitten.';
		cr;
		cr;
		nextPutAll: 'Wij verzoeken u om ter bevestiging van uw email adres op onderstaande link te klikken:';
		cr;
		cr;
		nextPutAll: (self buildConfirmationLinkTo: aDGVUser for: aDGVCommunity);
		cr;
		cr;
		nextPutAll: 'Zodra u dat gedaan heeft, zal uw lidmaatschap van de community geactiveerd zijn';
		cr;
		cr;
		nextPutAll: '(deze mail is automatisch gegenereerd en kan niet beantwoord worden)';
		cr.
	^strm contents! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'EG,CdG 10/13/2005 14:04'!
mailConfirmationTo: recipientUser about: newUser for: aDGVCommunity 
	| message |
	message := MailMessage empty.
	message body: (MIMEDocument contentType: 'text/plain'
				content: (self buildMessageTextAbout: newUser for: aDGVCommunity)).
	message setField: 'From' toString: 'noreply@dgv.aardworks.nl'.
	message setField: 'To' toString: recipientUser email.
	message setField: 'Subject'
		toString: self subjectPartString, aDGVCommunity describeYourself.
	[self sendMail: message]
		on: Error
		do: [:err| TDSDebuggerSupport logError: 'Cannot send confirmation to', recipientUser email, 'Error: ', err printString]! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ 3/7/2005 13:00'!
randomKey
	randomKey ifNil: [
		randomKey _ UUID new.
		self save].
	^randomKey! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ 3/7/2005 13:20'!
receivedConfirmationFor: userId community: communityId signature: aString 
	"Called by the web service if a confirmation was received"
	|  community user |
	aString	= (self signWith: userId with: communityId) ifFalse: [^ false].
	community := DGVCommunity get: communityId.
	community	ifNil: [^ false].
	user := DGVUser get: userId.
	user ifNil: [^ false].
	community acceptNewUser: user.
	^ true! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 4/13/2005 20:45'!
requestReceived: data from: sourceUUID	
	| request |
	[request _ DGVPolicyMailServiceRequest fromByteArray: data.
	self acceptNewUser: request userId in: request communityId.
	"bounce back the request by way of confirmation"
	self localNode
		send: data
		toNodeId: sourceUUID
		onEndPoint: request replyEndPoint
		reliable: true]
		fork.! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 3/7/2005 16:25'!
sendMail: message 
	self smtpClient
		deliverMailFrom: message from
		to: (Array with: message to)
		text: message text
		usingServer: MailSender smtpServer! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:43'!
signWith: userId with: communityId
	| sig |
	sig _ (SecureHashAlgorithm new
		hashMessage: userId printString, communityId printString, self randomKey) printStringRadix: 36.
	^(sig copyFrom: 4 to: sig size) asLowercase
		! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 3/7/2005 16:25'!
smtpClient
	^#SMTPClient binding ifNil: [SMTPClient]! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 4/14/2005 01:01'!
start
	DGVConfirmationMailAcceptanceService allInstances size = 0 ifTrue: [
		DGVConfirmationMailAcceptanceService start].
	^super start! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 18:07'!
stop
	DGVConfirmationMailAcceptanceService stop.
	super stop! !

!DGVPolicyConfirmUser methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:44'!
subjectPartString
	^'Uw aanmelding voor '! !

DGVPolicyConfirmUser subclass: #DGVPolicyOwnerConfirm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'CdG 7/6/2005 13:44'!
acceptNewUser: aDGVUserId in: aDGVCommunityId 
	"mail out a confirmation to the owner"
	| aDGVUser aDGVCommunity |
	aDGVUser _ DGVUser get: aDGVUserId.
	aDGVCommunity _ DGVCommunity get: aDGVCommunityId.
	(aDGVUser isNil or: [aDGVCommunity isNil])
		ifTrue: [^self acceptanceFailedOfUser: aDGVUserId in: aDGVCommunityId].
	self mailConfirmationTo: aDGVCommunity owner about: aDGVUser for: aDGVCommunity.

	^ nil"acceptance is in limbo"! !

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:54'!
buildAcceptanceTextAbout: aDGVUser for: aDGVCommunity 
	| strm |
	strm := WriteStream on: String new.
	strm
		nextPutAll: 'U heeft zich aangemeld voor deelname aan ';
		nextPutAll: aDGVCommunity describeYourself;
		nextPutAll: '. Deze aanmelding is door de eigenaar van de community geaccepteerd.';
		cr;
		cr;
		nextPutAll: '(deze mail is automatisch gegenereerd en kan niet beantwoord worden)';
		cr.
	^strm contents! !

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:54'!
buildHtmlConfirmationInto: aHtmlBuilder
	aHtmlBuilder
		html: 'Uw bevestiging is ontvangen en verwerkt. De gebruiker heeft nu toegang.'.! !

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:55'!
buildMessageTextAbout: aDGVUser for: aDGVCommunity 
	| strm |
	strm := WriteStream on: String new.
	strm
		nextPutAll: 'De gebruiker: ';
		nextPutAll: aDGVUser fullName, ' (', aDGVUser email, ')';
		nextPutAll: ' heeft zich aangemeld voor deelname aan ';
		nextPutAll: aDGVCommunity describeYourself;
		nextPutAll: '. Hiervoor is uw toestemming vereist.';
		cr;
		cr;
		nextPutAll: 'Indien u deze gebruiker wilt toelaten, verzoeken wij u op onderstaande link te klikken:';
		cr;
		cr;
		nextPutAll: (self buildConfirmationLinkTo: aDGVUser for: aDGVCommunity);
		cr;
		cr;
		nextPutAll: 'Zodra u dat gedaan heeft, zal het lidmaatschap van deze gebruiker geactiveerd zijn.';
		cr;
		cr;
		nextPutAll: '(deze mail is automatisch gegenereerd en kan niet beantwoord worden)';
		cr.
	^strm contents! !

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'TJ 3/7/2005 13:20'!
notify: userId aboutAcceptationIn: communityId
	| message newUser community |
	newUser _ DGVUser get: userId.
	community _ DGVCommunity get: communityId.
	message := MailMessage empty.
	message body: (MIMEDocument contentType: 'text/plain'
				content: (self buildAcceptanceTextAbout: newUser for: community)).
	message setField: 'From' toString: 'noreply@dgv.aardworks.nl'.
	message setField: 'To' toString: newUser email.
	message setField: 'Subject'
		toString: 'Uw aanmelding voor ' , community describeYourself, ' is geaccepteerd.'.
	self sendMail: message! !

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:56'!
receivedConfirmationFor: userId community: communityId signature: aString 
	| result |
	result _ super receivedConfirmationFor: userId community: communityId signature: aString.
	result ifTrue: [self notify: userId aboutAcceptationIn: communityId].
	^result! !

!DGVPolicyOwnerConfirm methodsFor: 'as yet unclassified' stamp: 'TJ 3/1/2005 16:56'!
subjectPartString
	^'Nieuwe aanmelding voor '! !

DGVService subclass: #DGVQueryService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQueryService class methodsFor: 'as yet unclassified' stamp: 'CdG 4/13/2005 12:28'!
wellKnownEndPoint
	^ 16r80! !

!DGVQueryService methodsFor: 'private' stamp: 'CdG 11/1/2005 15:43'!
receivedGetSearch: data from: sourceUUID 
	| search query |
	search := AWGSearch fromByteArray: data.
	(search wasSeenBy: self localNode nodeId) ifTrue: [^self].
	
	[[query := DGVQuery fromByteArray: search query.
	query executeIn: search]
		on: Error
		do: 	[:ex | 
			TDSDebuggerSupport logError: 'Error during query: ' , ex printString].
	search routeThrough: self localNode endPoint: self class wellKnownEndPoint]
		fork! !

!DGVQueryService methodsFor: 'initialize-release' stamp: 'EG,TJ 3/22/2005 16:41'!
start
	DGVNode default
		dispatchEndPoint: self class wellKnownEndPoint 
		to: [:data :sourceUUID |
			self receivedGetSearch: data from: sourceUUID	
		].
! !

!DGVQueryService methodsFor: 'initialize-release' stamp: 'EG,TJ 3/22/2005 16:41'!
stop
	DGVNode default
		unRegisterEndPoint: self class wellKnownEndPoint ! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 15:08'!
canAddServiceInstance
	self isSingletonService ifTrue: [
		(self checkExistenceOfServiceWithName: self serviceName) ifTrue: [
			^false]].
	^true! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 14:50'!
checkExistenceOfServiceWithName: aString 
	^(self serviceWithName: aString) notNil! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 15:09'!
createService
	^self canAddServiceInstance 
		ifTrue: [self serviceRegistry add: (self new save; start; yourself)]
		ifFalse: [self serviceWithName: self serviceName]! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'TJ 3/7/2005 12:58'!
get: aUUID
	^ self allSubInstances detect: [:each | each id = aUUID] ifNone: nil
	! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 14:37'!
isSingletonService
	"By default, it doesn't make sense to have more than one instance of a service running"
	^true! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 4/14/2005 01:12'!
presenceSet
	| presence presenceSet |
	presence _ self localNode presence.
	presenceSet _ presence setAt: self presenceSetName.
	presenceSet configured ifFalse: [
		presenceSet
			timeToLive: 2 minutes;
			beforeExpiryTest: [:object | self checkExistenceOfServiceWithName: object];
			configured: true].
	^presenceSet! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/25/2005 20:44'!
presenceSetName
	^ 'nl.aardworks.dgv.services'! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 4/13/2005 20:24'!
serviceName
	^ self name! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 10:13'!
serviceRegistry
	^ServiceRegistry ifNil: [ServiceRegistry _ OrderedCollection new]! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/22/2005 15:10'!
serviceWithName: aString 
	^self serviceRegistry detect: [:each | each class serviceName = aString]
		ifNone: nil! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/25/2005 20:43'!
start
	self presenceSet. "for side effect of configuration"
	self localStorage items do: [:each |
		each class canAddServiceInstance ifTrue: [
			self serviceRegistry add: (each start; yourself)]]! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 10:15'!
stop
	self serviceRegistry copy do: [:each | each stop. self serviceRegistry remove: each].
! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'TJ 3/7/2005 12:49'!
storageName
	^ 'service'
	! !

!DGVService class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 10:30'!
textSearch: aString
	^self textSearchNotImplementedForThisClass! !

!DGVService methodsFor: 'accessing' stamp: 'EG,TJ 3/22/2005 16:42'!
localNode
	^DGVNode default! !

!DGVService methodsFor: 'accessing' stamp: 'CdG 4/14/2005 01:09'!
p2pAnnounceOnSave
	^false! !

!DGVService methodsFor: 'as yet unclassified' stamp: 'CdG 4/8/2005 01:04'!
requestReceived: data from: sourceUUID 
	"override me if you implement the standard service request handling mechanism"! !

!DGVService methodsFor: 'as yet unclassified' stamp: 'CdG,EG 3/25/2005 20:28'!
start
	| localNode endPoint |
	"this is a start method for a generic service that lives on an end-point. Override at will if you
	 have other kinds of services"
	localNode _ DGVNode default.
	endPoint _ localNode dispatchDynamicEndPointTo: [:data :sourceUUID |
			self requestReceived: data from: sourceUUID	
		].
	self class presenceSet
		addPresenceOf: self class serviceName 
		endPoint: endPoint! !

!DGVService methodsFor: 'as yet unclassified' stamp: 'CdG 3/10/2005 10:17'!
stop
	"this is a stop method for a generic service that lives on an end-point. Override at 
	will if you have other kinds of services"
! !

DGVObject subclass: #DGVUser
	instanceVariableNames: 'publicKey encryptedPrivateKey userName privateKey name email friends secretQuestion emergencyKey documentsFolder loginCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVUser class methodsFor: 'api' stamp: 'JJ 11/14/2005 13:34'!
byEmail: aString
	^self
		findAndTestUserWithName: nil
		email: aString
		testBlock: [:each | 
			each email = aString ifTrue: [^each]]! !

!DGVUser class methodsFor: 'api' stamp: 'CdG 9/16/2005 12:06'!
byName: username
	^self
		findAndTestUserWithName: username
		email: nil
		testBlock: [:each | 
			each userName = username ifTrue: [^each]]! !

!DGVUser class methodsFor: 'private' stamp: 'CdG,TJ 2/3/2005 12:11'!
defaultLocalUserFileName
	^DGVOSSupport default dataDirectory fullNameFor: 'defaultlocaluser'! !

!DGVUser class methodsFor: 'api' stamp: 'EG 11/23/2005 16:01'!
findAndTestUserWithName: userName email: email testBlock: testBlock 
	| test user |
	test := [:each | each
				ifNotNil: [each deleted
						ifFalse: [testBlock value: each]]].
	self localStorage items do: test.
	"nothing found - search the net"
	user := self userByNameEmailQueryClass name: userName email: email.
	test value: user.
	^ nil! !

!DGVUser class methodsFor: 'private' stamp: 'TJ 1/25/2005 16:44'!
generateKeys
	| dsa |
	dsa _ DigitalSignatureAlgorithm new.
	dsa initRandomNonInteractively.
	^ dsa generateKeySet.
	! !

!DGVUser class methodsFor: 'initialize-release' stamp: 'CdG 10/31/2005 17:10'!
getDefaultLocalUser
	| rs |
	rs := ReferenceStream oldFileNamed: self defaultLocalUserFileName.
	rs ifNil: [^nil].
	[^self get: rs next] ensure: [rs close]! !

!DGVUser class methodsFor: 'api' stamp: 'TJ,CdG 3/7/2005 15:28'!
isDuplicateUser: userName email: email 
	"nothing found - search the net"

	| testBlock |
	testBlock := 
			[:each | 
			each userName = userName 
				ifTrue: [^'een gebruiker met deze naam bestaat al'].
			each email = email 
				ifTrue: [^'een gebruiker met dit email adres bestaat al']].
	^self 
		findAndTestUserWithName: userName
		email: email
		testBlock: testBlock! !

!DGVUser class methodsFor: 'accessing' stamp: 'EG,CdG 10/13/2005 14:41'!
isValidEmail: anObject 
	anObject ifNil: [^false].
	^(self validEmailRegex matchesIn: anObject) notEmpty! !

!DGVUser class methodsFor: 'api' stamp: 'CdG 10/26/2005 15:28'!
login: username password: password
	| loginBlock |
	loginBlock _ [:each |
		each userName = username ifTrue: [
			(each validatePassword: password) ifNotNil: [
				each incrementLoginCount; save.
				^each]]].
	^self
		findAndTestUserWithName: username
		email: nil
		testBlock: loginBlock! !

!DGVUser class methodsFor: 'initialize-release' stamp: 'TJ 1/25/2005 16:34'!
new
	^ self error: 'Use #withName:password:.'! !

!DGVUser class methodsFor: 'accessing' stamp: 'EG,CdG 9/9/2005 11:10'!
secretQuestionNumber: anInteger
	^self secretQuestions at: anInteger ifAbsent: ['onbekende vraag']! !

!DGVUser class methodsFor: 'accessing' stamp: 'EG,CdG 9/9/2005 11:09'!
secretQuestions
	^{
		'Uw geboorteplaats'.
		'Naam van uw moeder'.
		'Merk van eerste auto'.
		'Naam van favoriete leraar vroeger op school'.
	}! !

!DGVUser class methodsFor: 'accessing' stamp: 'CdG,TJ 2/3/2005 11:45'!
storageName
	^'user'! !

!DGVUser class methodsFor: '*dgv-tests' stamp: 'TJ 4/21/2005 13:59'!
testUser
		^self testUserWithName: 'myname' password: 'secret'! !

!DGVUser class methodsFor: '*dgv-tests' stamp: 'CdG 2/1/2005 12:59'!
testUserWithName: aString password: password 
	| keys |
	keys := DigitalSignatureAlgorithm testKeySet.
	^self 
		withUserName: aString
		password: password
		keys: keys! !

!DGVUser class methodsFor: 'api' stamp: 'CdG 11/15/2005 12:40'!
userByNameEmailQueryClass
	^DGVQueryUserByNameEmail! !

!DGVUser class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 10/13/2005 14:41'!
validEmailRegex
	^'^([a-zA-Z0-9_\-.]+)@([a-zA-Z0-9_\-.]+)\.([a-zA-Z]+)$' asRegex! !

!DGVUser class methodsFor: 'initialize-release' stamp: 'CdG 2/1/2005 12:59'!
withUserName: aString password: password 
	| keys |
	keys := self generateKeys.
	^self 
		withUserName: aString
		password: password
		keys: keys! !

!DGVUser class methodsFor: 'private' stamp: 'CdG 2/1/2005 12:59'!
withUserName: aString password: password keys: keys 
	^(super new)
		userName: aString
			password: password
			keys: keys;
		validatePassword: password! !

!DGVUser methodsFor: 'accessing' stamp: 'CdG 3/15/2005 13:20'!
addFriend: aDGVUser
	^self friends add: aDGVUser! !

!DGVUser methodsFor: 'accessing' stamp: 'TJ 1/27/2005 09:56'!
asString
^ self className, ' ', self userName, ' ', self id.! !

!DGVUser methodsFor: 'objects from disk' stamp: 'CdG 2/14/2005 16:49'!
basicSave
	| tmp |
	tmp _ privateKey.
	privateKey _ nil.
	[super basicSave]
		ensure: [privateKey _ tmp]! !

!DGVUser methodsFor: 'as yet unclassified' stamp: 'CdG,TJ 2/3/2005 12:11'!
beDefaultLocalUser
	| fullname rs |
	fullname := self class defaultLocalUserFileName.
	rs := ReferenceStream forceNewFileNamed: fullname.
	[rs nextPut: self id] ensure: [rs close]! !

!DGVUser methodsFor: 'security' stamp: 'EG,CdG 10/6/2005 14:06'!
changePassword: password
	encryptedPrivateKey := self encryptPrivateKey: privateKey withPassword: password! !

!DGVUser methodsFor: 'private' stamp: 'EG,CdG 9/9/2005 11:03'!
cleanEmergencyAnswer: aString
	^aString asLowercase select: [:each | each isLowercase or: [each isDigit]]! !

!DGVUser methodsFor: 'private' stamp: 'EG,CdG 9/9/2005 10:14'!
decryptPrivateKey: aByteArray withPassword: aString
	| pwHash clearText |
	(aString isNil
			or: [aString isEmpty])
		ifTrue: [^ nil].
	pwHash := self hashPassword: aString.
	clearText := CipherSaber new decipher: aByteArray withKey: pwHash.
	(clearText beginsWith: '#(')
		ifFalse: [^ nil].
	^Array readFrom: clearText! !

!DGVUser methodsFor: 'testing' stamp: 'EG,CdG 8/18/2005 11:40'!
dgvMatchTextSearch: pattern
	"if we're used in a text search, match on the user name"
	^self fullName dgvMatchTextSearch: pattern! !

!DGVUser methodsFor: 'accessing' stamp: 'CdG 9/30/2005 11:03'!
documentsFolder
	^documentsFolder ifNil: [
		documentsFolder := DGVFolder new
			folderName: 'Mijn Documenten' translated;
			parent: self.
		documentsFolder save.
		self markDirty.
		documentsFolder]
		! !

!DGVUser methodsFor: 'printing' stamp: 'CdG 8/12/2005 14:07'!
editableString
	^self userName! !

!DGVUser methodsFor: 'accessing' stamp: 'CdG 2/9/2005 12:31'!
email
	^email! !

!DGVUser methodsFor: 'accessing' stamp: 'EG 12/14/2005 14:29'!
email: anObject 
	(self class isValidEmail: anObject)
		ifFalse: [^DGVInvalidEmailAddressException signal: email asString].

	email := anObject! !

!DGVUser methodsFor: 'accessing' stamp: 'EG,CdG 9/9/2005 10:43'!
encryptPrivateKey: key withPassword: aString
	| passphraseHash |
	passphraseHash := self hashPassword: aString.
	^CipherSaber new encipher: key storeString withKey: passphraseHash! !

!DGVUser methodsFor: 'accessing' stamp: 'CdG 3/15/2005 12:19'!
friends
	^self getCollectionAt: #friends! !

!DGVUser methodsFor: 'accessing' stamp: 'TJ 2/1/2005 15:57'!
fullName
	^ name! !

!DGVUser methodsFor: 'accessing' stamp: 'TJ 2/1/2005 15:56'!
fullName: anObject
	name _ anObject! !

!DGVUser methodsFor: 'private' stamp: 'CdG 2/1/2005 13:00'!
hashPassword: password 
	| pwHash passphraseHash |
	pwHash := (SecureHashAlgorithm new hashMessage: password) hex.
	passphraseHash := SecureHashAlgorithm new 
				hashMessage: id , pwHash.
	^passphraseHash hex! !

!DGVUser methodsFor: 'printing' stamp: 'EG,CdG 8/18/2005 11:34'!
htmlRepresentation
	^'<a href="mailto:{1}">{2}</a>' format: {self email. self fullName}! !

!DGVUser methodsFor: 'private' stamp: 'CdG 10/26/2005 15:27'!
incrementLoginCount
	loginCount := self loginCount + 1.
	self markDirty.! !

!DGVUser methodsFor: 'initialize-release' stamp: 'CdG 5/2/2005 14:41'!
initialize
	super initialize.
	friends _ DGVCollection parent: self named: 'friends'.! !

!DGVUser methodsFor: 'testing' stamp: 'EG 12/15/2005 17:24'!
isManagerOfCommunity: aDGVCommunity
	aDGVCommunity ifNil: [^false].
	^[aDGVCommunity managers includes: self] on: DGVException do: [false]! !

!DGVUser methodsFor: 'testing' stamp: 'EG,CdG 7/1/2005 10:34'!
isMemberOfCommunity: aDGVCommunity
	^aDGVCommunity users includes: self! !

!DGVUser methodsFor: 'testing' stamp: 'TJ 1/28/2005 16:23'!
isValid
	^privateKey notNil! !

!DGVUser methodsFor: 'accessing' stamp: 'CdG 10/26/2005 15:25'!
loginCount
	^loginCount ifNil: [loginCount := 0]! !

!DGVUser methodsFor: 'security' stamp: 'EG,CdG 9/9/2005 10:52'!
logout
	self privateKey: nil! !

!DGVUser methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:27'!
matchTextSearch: pattern 
	^ self matchTextSearch: pattern inAttributes: #(#email #userName #fullName )! !

!DGVUser methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 14:43'!
memberCommunities
	^DGVCommunity communitiesWithMember: self! !

!DGVUser methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 14:27'!
ownCommunities
	^DGVCommunity communitiesWithOwner: self! !

!DGVUser methodsFor: 'printing' stamp: 'CdG 9/16/2005 12:04'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self stringRepresentation;
		nextPut: $)! !

!DGVUser methodsFor: 'private' stamp: 'CdG,TJ 2/3/2005 11:50'!
privateKey: key	
	privateKey _ key! !

!DGVUser methodsFor: 'security' stamp: 'EG,CdG 9/9/2005 11:02'!
recoverKeyWithSecretQuestionAnswer: aByteString newPassword: aString 
	| cleanedPassword |
	cleanedPassword := self cleanEmergencyAnswer: aByteString.
	privateKey := self decryptPrivateKey: emergencyKey withPassword: cleanedPassword.
	encryptedPrivateKey := self encryptPrivateKey: privateKey withPassword: aString
	! !

!DGVUser methodsFor: 'accessing' stamp: 'JJ 11/18/2005 11:03'!
resetDocumentsFolder
	documentsFolder := nil! !

!DGVUser methodsFor: 'security' stamp: 'CdG 9/16/2005 11:48'!
secretQuestion
	^secretQuestion! !

!DGVUser methodsFor: 'security' stamp: 'CdG 9/16/2005 11:40'!
secretQuestionString
	^self class secretQuestionNumber: (secretQuestion ifNil: [0])! !

!DGVUser methodsFor: 'security' stamp: 'CdG 9/13/2005 15:41'!
secretQuestion: aSmallInteger answer: aByteString 
	| cleanedPassword |
	secretQuestion := aSmallInteger.
	cleanedPassword := self cleanEmergencyAnswer: aByteString.
	emergencyKey := self encryptPrivateKey: privateKey withPassword: cleanedPassword! !

!DGVUser methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 12:25'!
sign: aLargePositiveInteger 
	privateKey ifNil: [^nil].
	^DigitalSignatureAlgorithm new initRandomNonInteractively
		computeSignatureForMessageHash: aLargePositiveInteger 
		privateKey: privateKey! !

!DGVUser methodsFor: 'printing' stamp: 'CdG 7/11/2005 20:15'!
stringRepresentation
	^'{1} "{2}" ({3})' format: {'Gebruiker' translated. self userName. self fullName}! !

!DGVUser methodsFor: 'accessing' stamp: 'TJ 1/26/2005 08:59'!
userName
	^userName! !

!DGVUser methodsFor: 'accessing' stamp: 'EG,CdG 10/6/2005 14:03'!
userName: anObject
	userName := anObject! !

!DGVUser methodsFor: 'initialize-release' stamp: 'EG,CdG 9/9/2005 10:42'!
userName: aUserName password: password keys: keys 
	userName := aUserName.
	publicKey := keys second.
	encryptedPrivateKey := self encryptPrivateKey: keys first withPassword: password! !

!DGVUser methodsFor: 'private' stamp: 'EG,CdG 9/9/2005 10:35'!
validatePassword: password 
	privateKey := self decryptPrivateKey: encryptedPrivateKey withPassword: password.
	^privateKey ifNil: [nil] ifNotNil: [self]! !

!DGVUser methodsFor: 'security' stamp: 'CdG,TJ 2/15/2005 13:54'!
verifySignature: theSignature ofMessageHash: hash
	^DigitalSignatureAlgorithm new
		verifySignature: theSignature ofMessageHash: hash publicKey: publicKey! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:46'!
activeSearches
	^ActiveSearches ifNil: [ActiveSearches _ Set new]! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 11/25/2005 15:59'!
announcementPresenceSet
	^self localNode presence setAt: self presenceSetName
		ifUnconfigured: [:set | set timeToLive: self presenceAnnouncementTimeToLive]! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'EG 12/14/2005 17:21'!
getFromNetwork: aUUID
	^self getFromNetwork: aUUID withVersion: nil! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 12/16/2005 00:15'!
getFromNetwork: aUUID withVersion: aNumber
	self subclassResponsibility! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:44'!
getLocally: aUUID mayRefreshFromNetwork: aBoolean
	| answer |
	"1. try local"
	answer := self localStorage get: aUUID.
	answer ifNotNil: [
			^aBoolean 
				ifTrue: [self refreshIfNecessary: answer]
				ifFalse: [answer]].
	^nil
! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 12/10/2005 21:08'!
get: aUUID
	| answer |
	"1. Local storage"
	TDSDebuggerSupport log: 'get ', self name, '/', aUUID printString.
	answer _ self getLocally: aUUID mayRefreshFromNetwork: true.
	answer ifNotNil: [^answer].
	"2. Network."
	answer _ self getFromNetwork: aUUID withVersion: nil.
	answer ifNotNil: [
		answer updateLastSavedTime.
		self localStorage put: answer].
	^answer! !

!DGVProtoObject class methodsFor: 'testing' stamp: 'EG,CdG 7/6/2005 11:40'!
isDGVAbstractClass
	^(self class selectors includes: #storageName) not! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:31'!
localNode
	^DGVNode default! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:40'!
localStorage
	^DGVStorage named: self storageName! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 6/15/2005 12:20'!
presenceAnnouncementTimeToLive
	"how long a write transaction presence announcement should live. Long enough to
	propagate, but not much longer"

	^(self refreshTime * 1.05) + 1 minute

"Discussion:

By basing our time to live on the time between refreshes we are most likely to keep the network in a (roughly) consistent state. When an update is made, any given node that misses the presence announcement, when coming online:
- Receives the presence announcement because one of its neighbours still has it around;
- Refreshes the affected object because the local copy has expired.

We put in some overlap as a safety factor, because there's not a lot of harm in a double update."! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG,EjG 6/8/2005 10:25'!
presenceSetName
	self subclassResponsibility.! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:45'!
put: aDGVObject
	^aDGVObject class localStorage put: aDGVObject! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'EG,CdG 6/13/2005 11:24'!
refreshIfNecessary: aDGVObject 
	aDGVObject needsRefresh 
		ifFalse: [^aDGVObject].
	^self refresh: aDGVObject! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 6/15/2005 11:31'!
refreshRetryTime
	^10 minutes! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 6/30/2005 14:29'!
refreshTime
	^ 4 hours! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG,EjG 6/8/2005 10:37'!
refresh: anDGVObject
	self subclassResponsibility.! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 11/21/2005 21:32'!
searchLock
	^SearchLock ifNil: [SearchLock _ Mutex new]! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'EG 12/10/2005 17:44'!
serializeNetworkSearch: action forId: aUUID 
	| goAhead seenOtherSearch answer |
	goAhead := false.
	seenOtherSearch := false.
	"acquire lock"
	[goAhead]
		whileFalse: [self searchLock
				critical: [(self activeSearches includes: aUUID)
						ifTrue: [seenOtherSearch := true]
						ifFalse: [self activeSearches add: aUUID.
							goAhead := true]].
	goAhead
				ifFalse: [(Delay forDuration: 250 milliSeconds) wait]].
	["did we wait for the lock? if yes, see whether there was a result"
	seenOtherSearch
		ifTrue: [answer := self getLocally: aUUID mayRefreshFromNetwork: false.
			answer
				ifNotNil: [^ answer]].
	^ action value]
		ensure: [self searchLock
				critical: [self activeSearches remove: aUUID]]! !

!DGVProtoObject class methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:45'!
storageName
	self subclassResponsibility! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'EG,CdG 7/1/2005 11:15'!
basicLastSavedTime
	^lastSavedTime! !

!DGVProtoObject methodsFor: 'private' stamp: 'CdG 10/26/2005 15:21'!
creationTime
	^creationTime ifNil: [AWGTimeStamp epoch]! !

!DGVProtoObject methodsFor: 'private' stamp: 'EG 10/31/2005 14:53'!
creationTime: aAWGTimeStamp
	creationTime := aAWGTimeStamp! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:32'!
dirty
	^dirty! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:32'!
dirty: aBoolean
	dirty _ aBoolean! !

!DGVProtoObject methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/10/2005 12:13'!
equalsObjectWithClass: aClass andUuid: anUUID 
	^self class = aClass and: [id = anUUID]! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 8/11/2005 12:22'!
getPathToRootCommunity
	parent ifNil: [^#()].
	^parent getPathToRootCommunity, (Array with: parent)! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:32'!
id
	^id! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:32'!
id: anObject
	id _ anObject! !

!DGVProtoObject methodsFor: 'initialize-release' stamp: 'CdG 10/26/2005 15:13'!
initialize
	creationTime := DGVNode default networkTimeStamp.
	dirty _ true.
! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:53'!
lastSavedTime
	^lastSavedTime ifNil: [self updateLastSavedTime]! !

!DGVProtoObject methodsFor: 'testing' stamp: 'JJ 12/8/2005 16:51'!
lastSavedTime: anAWGTimeStamp 
	lastSavedTime := anAWGTimeStamp! !

!DGVProtoObject methodsFor: 'security' stamp: 'CdG 5/2/2005 14:40'!
localStorage
	^self class localStorage! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 5/2/2005 14:32'!
markDirty
	self dirty: true! !

!DGVProtoObject methodsFor: 'as yet unclassified' stamp: 'EG,CdG 10/6/2005 16:08'!
matchTextSearch: anArray 
	^false! !

!DGVProtoObject methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/13/2005 11:46'!
needsRefresh
	^self lastSavedTime < (DGVNode default networkTimeStamp - self class refreshTime)! !

!DGVProtoObject methodsFor: 'as yet unclassified' stamp: 'EG 12/9/2005 14:31'!
objectForDataStream: refStream
	^refStream rootObject 
		ifNil: [
			refStream rootObject: self.
			self]
		ifNotNil: [
				refStream rootObject == self 
					ifTrue: [self]
					ifFalse: [DGVLink for: self]]! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:04'!
parent
	^parent! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 7/27/2005 11:05'!
parent: anObject
	parent := anObject.
	self dirty: true! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 7/11/2005 15:12'!
qualifiedReference
	^self storageName , '/' , self id printString! !

!DGVProtoObject methodsFor: 'as yet unclassified' stamp: 'CdG 6/28/2005 14:47'!
srpSavePreMap: aContext
	| root |
	(aContext userData at: #root ifAbsent: nil)
		ifNil: [
			aContext userData at: #root put: self.
			^self].
	root _ aContext userData at: #root.
	root == self ifTrue: [^self].
	((root isKindOf: Array) and: [root includes: self])
		ifTrue: [^self].
	^DGVLink for: self! !

!DGVProtoObject methodsFor: 'accessing' stamp: 'CdG 7/12/2005 14:03'!
storageName
	^self class storageName! !

!DGVProtoObject methodsFor: 'private' stamp: 'EG,CdG 6/13/2005 11:30'!
updateLastSavedTime
	^lastSavedTime := DGVNode default networkTimeStamp! !

!DGVProtoObject methodsFor: 'private' stamp: 'CdG 5/2/2005 14:53'!
updateLastSavedTimeAsExpired
	^lastSavedTime := AWGTimeStamp epoch! !

!DGVProtoObject methodsFor: 'private' stamp: 'CdG 5/2/2005 14:53'!
updateLastSavedTimeForRetry
	^lastSavedTime := lastSavedTime + self class refreshRetryTime! !

Object subclass: #DGVQuery
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQuery commentStamp: 'TJ,CdG 3/7/2005 15:13' prior: 0!
Presently, in cooperation with DGVQueryService, a very dumb query: it returns a single result, and this must be a DGVObject subclass instance. !

DGVQuery subclass: #DGVQueryCommunity
	instanceVariableNames: 'namePattern descriptionPattern managerId memberId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:42'!
descriptionPattern
	^descriptionPattern! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:43'!
descriptionPattern: anObject
	descriptionPattern _ self makePattern: anObject! !

!DGVQueryCommunity methodsFor: 'querying' stamp: 'EG,CdG 6/24/2005 13:57'!
executeIn: anAWGSearch
	| answer |
	answer _ DGVCommunity localStorage items
		asSet collect: [:each | self match: each].
	answer ifEmpty: [^self].
	
	anAWGSearch 
		returnResult: answer asArray marshallAsByteArray
		resultCount: answer size
		onPeer: self localNode! !

!DGVQueryCommunity methodsFor: 'private' stamp: 'TJ,CdG 3/7/2005 15:44'!
makePattern: aString
	"ensure that patterns start and end with wildcard chars"
	| answer |
	answer _ aString.
	answer ifEmpty: [^''].
	(answer first = $*) ifFalse: [answer _ '*', answer].
	(answer last = $*) ifFalse: [answer _ answer, '*'].
	^answer! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:42'!
managerId
	^managerId! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:42'!
managerId: anObject
	managerId := anObject! !

!DGVQueryCommunity methodsFor: 'testing' stamp: 'EG,TJ 3/24/2005 13:48'!
match: aDGVCommunity 
	| match |
	match _ namePattern isEmpty or: [
		namePattern match: aDGVCommunity communityName].
	match _ match and: [descriptionPattern isEmpty or: [
		descriptionPattern match: aDGVCommunity description]].
	match _ match and: [memberId isNil or: [
		(aDGVCommunity users collect: [:each | each id]) includes: memberId]].
	match _ match and: [managerId isNil or: [
		aDGVCommunity owner notNil and: [aDGVCommunity owner id = managerId]]].
	^match! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:42'!
memberId
	^memberId! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:42'!
memberId: anObject
	memberId := anObject! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:41'!
namePattern
	^namePattern! !

!DGVQueryCommunity methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:43'!
namePattern: aString 
	namePattern _ self makePattern: aString! !

DGVQuery subclass: #DGVQueryGetBlobBlock
	instanceVariableNames: 'blobInfo blockNumber'
	classVariableNames: 'Cache'
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQueryGetBlobBlock class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 16:57'!
cache
	^Cache ifNil: 
			[Cache := LRUCache size: 25 factory: [:id | self createCacheEntryFor: id]]! !

!DGVQueryGetBlobBlock class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 16:57'!
createCacheEntryFor: id 
	| blob |
	blob := (DGVStorage named: DGVBlob storageName) get: id.
	^blob ifNil: [nil]
		ifNotNil: 
			[blob := blob stableCopyForChecksumming.
			blob -> { (blob marshallAsByteArray). (blob blobInfo) }]! !

!DGVQueryGetBlobBlock class methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 11:47'!
get: aBlobInfo block: aNumber atNode: aUUID timeOut: aDuration
	| query |
	query _ self new
		blobInfo: aBlobInfo;
		blockNumber: aNumber.
	^query executeUnicastTo: aUUID timeOut: aDuration! !

!DGVQueryGetBlobBlock class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 16:42'!
initialize
	self resetCache! !

!DGVQueryGetBlobBlock class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/1/2005 11:00'!
resetCache
	Cache _ nil! !

!DGVQueryGetBlobBlock methodsFor: 'accessing' stamp: 'CdG 3/10/2005 12:36'!
blobInfo
	^blobInfo! !

!DGVQueryGetBlobBlock methodsFor: 'accessing' stamp: 'CdG 3/10/2005 12:36'!
blobInfo: anObject
	blobInfo := anObject! !

!DGVQueryGetBlobBlock methodsFor: 'accessing' stamp: 'CdG 3/10/2005 12:36'!
blockNumber
	^blockNumber! !

!DGVQueryGetBlobBlock methodsFor: 'accessing' stamp: 'CdG 3/10/2005 12:36'!
blockNumber: anObject
	blockNumber := anObject! !

!DGVQueryGetBlobBlock methodsFor: 'accessing' stamp: 'CdG 7/6/2005 17:46'!
executeIn: anAWGSearch
	| base answer top data bytes hit |
	hit _ self class cache at: blobInfo id.
	hit ifNil: [^self].
	bytes _ hit value first.
	base _ ((blockNumber - 1) * DGVBlobTransfer blockSize) + 1.
	top _ (base + DGVBlobTransfer blockSize - 1) min: bytes size.
	data _ bytes copyFrom: base to: top.
	answer _ (self -> data) marshallAsByteArray.
	anAWGSearch 
		returnResult: answer onNode: self localNode;
		dontPassOn! !

DGVQuery subclass: #DGVQueryGetBlobInfo
	instanceVariableNames: 'blobId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQueryGetBlobInfo class methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 11:43'!
for: key whenFound: aBlock
	| query |
	query _ self new
		blobId: key.
	^query executeMultiWhenFound: aBlock! !

!DGVQueryGetBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/11/2005 10:11'!
blobId
	^blobId! !

!DGVQueryGetBlobInfo methodsFor: 'accessing' stamp: 'CdG 3/11/2005 10:11'!
blobId: anObject
	blobId := anObject! !

!DGVQueryGetBlobInfo methodsFor: 'as yet unclassified' stamp: 'CdG 7/6/2005 17:45'!
executeIn: anAWGSearch
	| answer hit | 
	"blob _ (DGVStorage named: DGVBlob storageName) get: blobId."
	hit _ DGVQueryGetBlobBlock cache at: blobId.

	hit ifNotNil: [
		answer _ (hit value last -> self localNode nodeId) marshallAsByteArray.
		anAWGSearch returnResult: answer onNode: self localNode].
! !

!DGVQueryGetBlobInfo methodsFor: 'accessing' stamp: 'CdG 6/15/2005 10:27'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: blobId printString;
		nextPut: $).! !

DGVQuery subclass: #DGVQueryStorageGet
	instanceVariableNames: 'storageName key version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

DGVQueryStorageGet class
	instanceVariableNames: 'testInstance'!

DGVQueryStorageGet subclass: #DGVMockQueryStorageGet
	instanceVariableNames: 'results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVMockQueryStorageGet methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 13:41'!
executeMultiWhenFound: aBlock
	results do: [:each | 
		| coll |
		coll _ DGVCollection newWithId: '123'.
		coll addAll: (each collect: [:i | DGVMockObject new id: i]).
		aBlock value: coll].! !

!DGVMockQueryStorageGet methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 13:28'!
results: anArray
	results _ anArray! !

!DGVQueryStorageGet class methodsFor: 'as yet unclassified' stamp: 'CdG 12/10/2005 21:07'!
get: key from: storageName withVersion: version
	| query |
	query _ self new
		key: key;
		storageName: storageName;
		version: version.
	^query execute! !

!DGVQueryStorageGet class methodsFor: 'as yet unclassified' stamp: 'CdG 12/10/2005 21:09'!
get: key from: storageName withVersion: version atNode: aUUID
	| query |
	query _ self new
		key: key;
		storageName: storageName;
		version: version.
	^query executeUnicastTo: aUUID! !

!DGVQueryStorageGet class methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 12:23'!
new
	^testInstance 
		ifNil: [self basicNew initialize] 
		ifNotNil: [testInstance]! !

!DGVQueryStorageGet class methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 12:22'!
testInstance: anObject 
	testInstance := anObject! !

!DGVQueryStorageGet methodsFor: 'querying' stamp: 'EG 12/9/2005 17:52'!
executeCollectionQueryIn: anAWGSearch 
	| answer collection objects links resolvedLinks |

	collection := (DGVStorage named: storageName) get: key.
	collection ifNil: [^self].

	objects := collection items reject: [:each| each isDGVLink].
	links := collection items select: [:each | each isDGVLink].
	resolvedLinks := Set new.
	links do: [:each | 
		[|resolvedObject |
		resolvedObject := each myclass getLocally: each id mayRefreshFromNetwork: false.
		(resolvedObject notNil and: [resolvedObject version >= each version]) ifTrue: [resolvedLinks add: resolvedObject]] on: Error do: []].
	objects addAll: resolvedLinks.

	answer := objects reject: [:each | each isNil].
	answer := answer collect: [:each | each marshallAsByteArray].
	answer ifNotEmpty: [
		anAWGSearch
				returnResult: answer marshallAsByteArray onNode: self localNode;
				dontPassOn]! !

!DGVQueryStorageGet methodsFor: 'querying' stamp: 'EG 12/14/2005 23:17'!
executeIn: anAWGSearch 
	storageName = DGVCollection storageName 
		ifTrue: [self executeCollectionQueryIn: anAWGSearch]
		ifFalse: [self executeRegularQueryIn: anAWGSearch]! !

!DGVQueryStorageGet methodsFor: 'querying' stamp: 'EG 12/14/2005 23:09'!
executeRegularQueryIn: anAWGSearch 
	| answer |
	answer := (DGVStorage named: storageName) get: key.
	(version notNil and: [answer version < version]) ifTrue: [answer := nil].
	answer ifNotNil: 
			[anAWGSearch
				returnResult: answer marshallAsByteArray onNode: self localNode;
				dontPassOn]! !

!DGVQueryStorageGet methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:08'!
key
	^key! !

!DGVQueryStorageGet methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:08'!
key: anObject
	key := anObject! !

!DGVQueryStorageGet methodsFor: 'printing' stamp: 'CdG 3/8/2005 10:46'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: storageName;
		nextPut: $/;
		nextPutAll: key printString;
		nextPut: $).! !

!DGVQueryStorageGet methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:08'!
storageName
	^storageName! !

!DGVQueryStorageGet methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:08'!
storageName: anObject
	storageName := anObject! !

!DGVQueryStorageGet methodsFor: 'accessing' stamp: 'CdG 12/10/2005 21:29'!
version
	^version! !

!DGVQueryStorageGet methodsFor: 'accessing' stamp: 'CdG 12/10/2005 21:29'!
version: anObject
	version := anObject! !

DGVQuery subclass: #DGVQueryTextSearch
	instanceVariableNames: 'queryString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

DGVQueryTextSearch subclass: #DGVQueryTextSearchInCommunity
	instanceVariableNames: 'community'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQueryTextSearchInCommunity methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 13:54'!
basicExecute
	^community textSearchWithChildren: queryString! !

!DGVQueryTextSearchInCommunity methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 12:08'!
community
	^community! !

!DGVQueryTextSearchInCommunity methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 12:08'!
community: anObject
	community := anObject! !

!DGVQueryTextSearch methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 12:11'!
basicExecute
	^DGVObject textSearchAllClasses: queryString! !

!DGVQueryTextSearch methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 14:44'!
executeIn: anAWGSearch 
	| answer |

	answer := self basicExecute.
	answer ifEmpty: [^self].
	anAWGSearch 
		returnResult: answer asArray marshallAsByteArray
		resultCount: answer size
		onPeer: self localNode! !

!DGVQueryTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/24/2005 14:50'!
queryString
	^queryString! !

!DGVQueryTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/24/2005 14:50'!
queryString: aByteString 
	queryString := aByteString! !

DGVQuery subclass: #DGVQueryUserByNameEmail
	instanceVariableNames: 'userName email'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVQueryUserByNameEmail class methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 11:35'!
name: username email: email
	| query |
	query _ self new
		userName: username;
		email: email.
	^query execute! !

!DGVQueryUserByNameEmail methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:09'!
email
	^email! !

!DGVQueryUserByNameEmail methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:09'!
email: anObject
	email := anObject! !

!DGVQueryUserByNameEmail methodsFor: 'querying' stamp: 'EG,CdG 6/24/2005 13:57'!
executeIn: anAWGSearch
	| testBlock |
	testBlock _ [:each |
		((self userName isNil or: [each userName = self userName]) and: 
		[self email isNil or: [each email = self email]])
			ifTrue: [
				^anAWGSearch 
					returnResult: each marshallAsByteArray
					onNode: self localNode]].
	DGVUser localStorage items do: testBlock.
		
		! !

!DGVQueryUserByNameEmail methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:09'!
userName
	^userName! !

!DGVQueryUserByNameEmail methodsFor: 'accessing' stamp: 'TJ,CdG 3/7/2005 15:09'!
userName: anObject
	userName := anObject! !

!DGVQuery class methodsFor: 'as yet unclassified' stamp: 'JJ 11/24/2005 10:58'!
fromByteArray: aByteArray
	^SrpConfiguration default loadObjectFromBinary: aByteArray! !

!DGVQuery methodsFor: 'client' stamp: 'EG 12/10/2005 20:04'!
execute
	"execute a query. Query returns a single DGVObject"
	| search foundObject sleeper newObject results |
	sleeper := Semaphore new.
	search := AWGSearch new.
	results := OrderedCollection new.
	foundObject := nil.
	search query: self marshallAsByteArray;
		onReceivedDo: [:result :source |
			newObject := DGVObject fromByteArray: result.
			results add: newObject.
			(foundObject isNil or: [newObject isNewerThan: foundObject])
				ifTrue: [
					foundObject := newObject.
					sleeper signal]].

	search 
		startFloodSearchThrough: DGVNode default 
		endPoint: DGVQueryService wellKnownEndPoint.

	(Delay forMilliseconds: 1000) wait.
	"we wait 250ms so we can gather multiple versions, if available"
	sleeper waitTimeoutSeconds: 2.
	^ foundObject! !

!DGVQuery methodsFor: 'server' stamp: 'CdG 3/9/2005 14:16'!
executeIn: anAWGSearch
	self subclassResponsibility! !

!DGVQuery methodsFor: 'client' stamp: 'EG 12/10/2005 20:37'!
executeMultiWhenFound: monadicBlock 
	"execute a query. Query returns potentially multiple DGVObjects. For
	each object found, executes monadicBlock"
	| answer |
	^ AWGSearch new query: self marshallAsByteArray;
		
		onReceivedDo: [:result :source | 
			answer := Object fromByteArray: result.
			((answer respondsTo: #do:)
					and: [(answer isKindOf: DGVCollection) not])
				ifTrue: [answer
						do: [:each | (self isDeleted: each)
								ifFalse: [monadicBlock value: each]]]
				ifFalse: [(self isDeleted: answer)
						ifFalse: [monadicBlock value: answer]]];
		 startFloodSearchThrough: DGVNode default endPoint: DGVQueryService wellKnownEndPoint! !

!DGVQuery methodsFor: 'client' stamp: 'CdG,EjG 6/8/2005 11:47'!
executeUnicastTo: aUUID
	"execute a query. Query returns a single DGVObject"
	^self executeUnicastTo: aUUID timeOut: 2 seconds! !

!DGVQuery methodsFor: 'client' stamp: 'CdG 11/1/2005 16:37'!
executeUnicastTo: aUUID timeOut: aDuration
	"execute a query. Query returns a single DGVObject"

	| search foundObject sleeper tmp |
	sleeper := Semaphore new.
	search := AWGSearch new.
	search
		query: self marshallAsByteArray;
		onReceivedDo: 
				[:result :source | 
				tmp := Object fromByteArray: result. 
				(self isDeleted: tmp) ifFalse: [foundObject := tmp].
				sleeper signal].
	search
		startUnicastSearchThrough: DGVNode default
		to: aUUID
		endPoint: DGVQueryService wellKnownEndPoint.
	sleeper waitTimeoutSeconds: aDuration asSeconds.
	search terminateSearch.
	^foundObject! !

!DGVQuery methodsFor: 'testing' stamp: 'CdG 11/1/2005 16:39'!
isDeleted: anObject
	"protect against objects we consider deleted. You never know what garbage we get fed 
	 from the network"
	(anObject respondsTo: #deleted) ifFalse: [^false].
	^anObject deleted
	! !

!DGVQuery methodsFor: 'querying' stamp: 'EG,TJ 3/22/2005 16:41'!
localNode
	^DGVNode default! !

Object subclass: #DGVSearch
	instanceVariableNames: 'query newItemFoundBlock searchResults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 11:19'!
description
	self subclassResponsibility! !

!DGVSearch methodsFor: 'private' stamp: 'EG,CdG 6/24/2005 14:45'!
doLocalSearch
	self subclassResponsibility! !

!DGVSearch methodsFor: 'private' stamp: 'JJ 11/18/2005 13:36'!
doRemoteSearch
	query executeMultiWhenFound: [:object |
		self signalMatch: object]! !

!DGVSearch methodsFor: 'private' stamp: 'EG,CdG 6/27/2005 12:49'!
initialize
	super initialize.
	searchResults _ DGVNotifyingCollection new! !

!DGVSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/24/2005 14:56'!
newItemFoundBlock: aBlockContext 
	newItemFoundBlock _ aBlockContext! !

!DGVSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/24/2005 14:56'!
searchResults
	^searchResults! !

!DGVSearch methodsFor: 'searching' stamp: 'EG,CdG 6/24/2005 14:46'!
searchRun 
	self doLocalSearch.
	self doRemoteSearch.! !

!DGVSearch methodsFor: 'private' stamp: 'EG,CdG 6/27/2005 14:11'!
signalMatch: anObject 
	(searchResults includes: anObject)
		ifTrue: [^ self].
	newItemFoundBlock
		ifNotNil: [newItemFoundBlock value: anObject].
	searchResults add: anObject! !

!DGVSearch methodsFor: 'searching' stamp: 'EG,CdG 6/24/2005 14:46'!
startSearch
	| proc |
	proc _ [self searchRun] forkAt: Processor userBackgroundPriority.
	proc name: self class name, ' search'.! !

DGVSearch subclass: #DGVTextSearch
	instanceVariableNames: 'filteredResults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

DGVTextSearch subclass: #DGVTextSearchInCommunity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVTextSearchInCommunity methodsFor: 'accessing' stamp: 'EG,CdG 7/6/2005 12:19'!
community: aDGVCommunity 
	query community: aDGVCommunity! !

!DGVTextSearchInCommunity methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 12:14'!
description
	^ query queryString, ' in ' translated, query community communityName! !

!DGVTextSearchInCommunity methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 13:53'!
doLocalSearch
	(query community textSearchWithChildren: query queryString) 
		do: [:each | self signalMatch: each]! !

!DGVTextSearchInCommunity methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 12:06'!
initialize
	super initialize.
	query := DGVQueryTextSearchInCommunity new! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 13:58'!
communities
	^ self filterResultsOnClass: DGVCommunity! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 11:19'!
description
	^ query queryString! !

!DGVTextSearch methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 14:59'!
doLocalSearch
	(DGVObject textSearchAllClasses: query queryString)
		do: [:each | self signalMatch: each]! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'CdG 11/15/2005 15:39'!
files
	^self filterResultsOnClass: DGVApplicationConfiguration fileClass! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 13:46'!
filteredResults
	^filteredResults ifNil: [filteredResults _ IdentityDictionary new]! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 13:46'!
filterResultsOnClass: aClass 
	^self filteredResults
		at: aClass
		ifAbsentPut: [DGVFilteringNotifyingCollection
				on: searchResults
				filterBlock: [:each | each isKindOf: aClass]]! !

!DGVTextSearch methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 14:53'!
initialize
	super initialize.
	query _ DGVQueryTextSearch new! !

!DGVTextSearch methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 14:53'!
queryString: aString
	query queryString: aString! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 11:48'!
records
	^self filterResultsOnClass: DGVOAIRecord! !

!DGVTextSearch methodsFor: 'accessing' stamp: 'EG,CdG 6/27/2005 11:48'!
users
	^self filterResultsOnClass: DGVUser! !

Object subclass: #DGVStorage
	instanceVariableNames: 'name mutex cache'
	classVariableNames: 'Registry TestData'
	poolDictionaries: ''
	category: 'DGV-Model'!

!DGVStorage class methodsFor: 'creation' stamp: 'CdG 7/11/2005 15:11'!
getFromQualifiedReference: aString
	| parts storageName uuid |
	parts _ aString subStrings: '/'.
	storageName _ parts first.
	uuid _ UUID fromString: parts last.
	^(self named: storageName) get: uuid.! !

!DGVStorage class methodsFor: 'creation' stamp: 'EG,CdG 10/13/2005 15:44'!
initializeRegistry
	^Registry := Dictionary new! !

!DGVStorage class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:50'!
makeReadyForTesting
	TestData := Dictionary new! !

!DGVStorage class methodsFor: 'creation' stamp: 'EG,CdG 8/18/2005 11:51'!
named: aString
	TestData ifNotNil: [^self testDataNamed: aString].
	^self registry at: (self -> aString) ifAbsentPut: [self new openOrCreateOn: aString]! !

!DGVStorage class methodsFor: 'creation' stamp: 'EG,CdG 10/13/2005 15:44'!
registry
	^Registry ifNil: [self initializeRegistry]! !

!DGVStorage class methodsFor: 'creation' stamp: 'EG,CdG 8/18/2005 11:51'!
testDataNamed: aString
	^TestData at: aString ifAbsentPut: [DGVTestStorage new]! !

!DGVStorage class methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 12:09'!
testingDone
	TestData := nil! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG 10/31/2005 15:10'!
activeItems
	^ self items
		reject: [:each | each isNil
				or: [each deleted]
				]! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG 12/14/2005 14:25'!
at: aUUID put: aByteArray
	| rs fullName tries | 
	fullName _ self storageDirectory 
				fullNameFor: aUUID asString.
	tries := 3.
	[tries > 0] whileTrue: [
		self critical: [
			[rs _ FileStream forceNewFileNamed: fullName]
				on: Error do: [
					Smalltalk garbageCollect.
					tries := tries - 1.
					tries = 0 ifTrue: [DGVFileLockedException signal: fullName].
					rs := nil].
			rs ifNotNil: [
				[rs 
					binary;
					nextPutAll: aByteArray.
				tries := 0] 
					ensure: [rs close]]]]
	! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG,CdG 10/13/2005 15:37'!
basicPut: aDGVObject
	self cache at: aDGVObject id put: aDGVObject.
	self storageDirectory assureExistence.
	self at: aDGVObject id put: aDGVObject marshallAsByteArray! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 15:36'!
cache
	^cache ifNil: [cache _ Dictionary new]! !

!DGVStorage methodsFor: 'copying' stamp: 'CdG,TJ 2/3/2005 11:24'!
cloneNamed: aString 
	^self class named: aString! !

!DGVStorage methodsFor: 'private' stamp: 'CdG 11/21/2005 21:32'!
critical: aBlockContext 
	mutex ifNil: [mutex _ Mutex new].
	mutex critical: aBlockContext! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 10:02'!
do: aBlockContext 
	^self activeItems do: aBlockContext! !

!DGVStorage methodsFor: 'testing' stamp: 'CdG,TJ 2/3/2005 10:42'!
exists: aUUID 
	^self storageDirectory fileExists: aUUID asString! !

!DGVStorage methodsFor: 'accessing' stamp: 'CdG 3/31/2005 22:54'!
getData: aUUID 
	| fullName rs | 
	fullName _ self storageDirectory 
				fullNameFor: aUUID asString.

	self critical: [[rs := FileStream oldFileNamed: fullName]
		on: FileDoesNotExistException
		do: [^nil].
	rs ifNil: [^nil].
	[[^rs 
		binary;
		contentsOfEntireFile] 
			ensure: [rs close]]
		ifError: [^nil]]! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 15:28'!
get: aUUID 
	^self cache at: aUUID ifAbsentPut: [
		[DGVObject fromByteArray: (self getData: aUUID)]
			on: Error
			do: [nil]]! !

!DGVStorage methodsFor: 'accessing' stamp: 'CdG 2/10/2005 12:23'!
items
	^self keys collect: [:each | self get: each]! !

!DGVStorage methodsFor: 'accessing' stamp: 'CdG 4/13/2005 18:58'!
keys
	^[self storageDirectory fileNames collect: [:each | UUID fromString: each]]
		on: Error
		do: [^#()]! !

!DGVStorage methodsFor: 'initialize-release' stamp: 'CdG,TJ 2/3/2005 10:06'!
openOrCreateOn: aString
	name _ aString.
	self storageDirectory assureExistence
	
	! !

!DGVStorage methodsFor: 'accessing' stamp: 'TJ 3/30/2005 13:36'!
put: aDGVObject 
	self basicPut: aDGVObject.
! !

!DGVStorage methodsFor: 'accessing' stamp: 'EG,CdG 9/7/2005 15:29'!
remove: aUUID
	self cache removeKey: aUUID ifAbsent: nil.
	self storageDirectory deleteFileNamed: aUUID asString ifAbsent: [nil].
! !

!DGVStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/29/2005 11:44'!
select: aBlockContext 
	self activeItems select: aBlockContext! !

!DGVStorage methodsFor: 'private' stamp: 'CdG,TJ 2/3/2005 10:06'!
storageDirectory
	^ DGVOSSupport default dataDirectory directoryNamed: name! !

DGVStorage subclass: #DGVTestStorage
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVTestStorage class methodsFor: 'accessing' stamp: 'EG,CdG 8/24/2005 15:18'!
storageName
	self shouldNotImplement! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 15:28'!
at: aUUID put: aDGVObject
	^self data at: aUUID put: aDGVObject copy! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:56'!
basicPut: aDGVObject
	^self at: aDGVObject id put: aDGVObject ! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:52'!
data
	^data ifNil: [data _ Dictionary new]! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/24/2005 14:08'!
exists: aUUID
	^self keys includes: aUUID! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:54'!
getData: aUUID
	self shouldNotImplement! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:54'!
get: aUUID
	^self data at: aUUID ifAbsent: nil! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:58'!
keys
	^self data keys asOrderedCollection! !

!DGVTestStorage methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:55'!
remove: aUUID
	self data removeKey: aUUID ifAbsent: nil! !

Object subclass: #DGVTranslator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 22:21'!
allTranslateableStrings
	^TranslatedReceiverFinder new dgvSenders asSet! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 22:21'!
allTranslatedStrings
	^(NaturalLanguageTranslator isoLanguage: 'en') translations keys asSet! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 22:42'!
cleanupTranslations
	"
	self cleanupTranslations
	"
	| xlats ws |
	xlats := DGVApplication englishTranslations.
	self unusedEnglishTranslations do: [:each |
		xlats removeKey: each ifAbsent: []].
	ws := String new writeStream.
	xlats associationsDo: [:ass |
		ws 
			nextPutAll: ass key printString;
			nextPutAll: ' -> ';
			nextPutAll: ass value printString;
			nextPut: $.;
			cr].
	DGVApplication class
		compile: 'newEnglishTranslations 
	^Dictionary newFrom: {
', ws contents, '}'
		classified: 'babel'! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 22:08'!
collectUntranslatedStringsInAMethodCalledFoo
	"Yup, that's what it does :-)

	self collectUntranslatedStringsInAMethodCalledFoo
	"

	| ws |
	ws := String new writeStream.
	self untranslatedStrings do: [:e | ws nextPutAll: e printString; nextPutAll: ' -> ''''.'; cr].
	DGVApplication class compile: 'foo {', ws contents, '}'.! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 21:16'!
feedEnglishTranslations
	"
		self feedEnglishTranslations
	"
	| packages |
	packages _ PackageInfo allPackages select: [ :package | 
		package packageName beginsWith: 'DGV'].
	packages do: [:package | package classes 
		do: [ :each |	(each respondsTo: #englishTranslations) ifTrue:
				[(NaturalLanguageTranslator isoLanguage: 'en') mergeTranslations:
				 	each englishTranslations]
			]]
			! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 6/10/2005 13:13'!
selectDutch
	self selectLanguage: 'nl'
! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 6/10/2005 13:13'!
selectEnglish
	self selectLanguage: 'en'
! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 6/10/2005 13:13'!
selectLanguage: aString
	Locale switchTo: (Locale isoLanguage: aString)! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 22:21'!
untranslatedStrings
	"
	Find strings that are in use but not translated.

		self untranslatedStrings
	"

	self feedEnglishTranslations.
	^self allTranslateableStrings  copyWithoutAll: self allTranslatedStrings ! !

!DGVTranslator class methodsFor: 'as yet unclassified' stamp: 'CdG 10/3/2005 22:21'!
unusedEnglishTranslations
	"
	Find strings that are translated but not in use.

		self unusedEnglishTranslations
	"

	self feedEnglishTranslations.
	^self allTranslatedStrings  copyWithoutAll: self allTranslateableStrings ! !

!Object class methodsFor: '*dgv-model' stamp: 'TJ,CdG 3/7/2005 16:00'!
fromByteArray: aByteArray
	^SrpConfiguration default loadObjectFromBinary: aByteArray! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 4/8/2005 00:38'!
codeSmell: aString
	"sender has smelly code - sniff out the refactorings!!"
	| msg |
	TDSDebuggerSupport log: [
		msg _ 'Code smell in ', thisContext sender printString.
		aString ifNotNil: [msg _ msg , ' (', aString, ')'].
		msg]
! !

!Object methodsFor: '*dgv-model' stamp: 'EG,CdG 8/18/2005 11:09'!
dgvMatchTextSearch: pattern
	^self asLowercase substrings includesAllOf: pattern! !

!Object methodsFor: '*dgv-network-client' stamp: 'CdG 2/6/2005 02:23'!
dgvReceivedAsAnswerIn: aDGVNetworkServiceClient
! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 8/12/2005 14:05'!
editableString
	^self asString! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 4/8/2005 00:50'!
fixMe: aString
	"sender of this method needs a bugfix, reason in a String"
	| msg |
	TDSDebuggerSupport log: [
		msg _ 'Fix Me in ', thisContext sender printString.
		aString ifNotNil: [msg _ msg ,' (', aString, ')'].
		msg]! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 8/12/2005 14:19'!
htmlRepresentation
	^self asString! !

!Object methodsFor: '*dgv-model' stamp: 'CdG 8/12/2005 14:22'!
isUrl
	^false! !

!Object methodsFor: '*dgv-model' stamp: 'CdG,TJ 2/3/2005 11:23'!
isUUID
	^false! !

!Object methodsFor: '*dgv-model' stamp: 'EG,CdG 6/24/2005 13:49'!
marshallAsByteArray
	^SrpConfiguration default saveObjectAsBinary: self! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 1/22/2005 15:22'!
needsMoreWork
	"sender of this method needs more work"
	self needsMoreWork: nil! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 4/8/2005 00:48'!
needsMoreWork: aString
	"sender of this method needs more work, reason in a String"
	| msg |
	TDSDebuggerSupport log: [
		msg _ 'More work needed in ', thisContext sender printString.
		aString ifNotNil: [msg _ msg , ' (', aString, ')'].
		msg]! !

!Object methodsFor: '*dgv-network-server' stamp: 'CdG 2/6/2005 01:37'!
passedByValue
	^true! !

!Object methodsFor: '*dgv-base extensions' stamp: 'CdG 4/8/2005 00:46'!
qndTestSupport: aString
	"sender of this method has some Quick-and-Dirty testing support,
	 that should probably be refactored - details in aString"
	| msg |
	TDSDebuggerSupport log: [
		msg _ 'Unclean testing support in ', thisContext sender printString.
		aString ifNotNil: [msg _ msg , ' (', aString, ')'].
		msg]! !

!DataStream class methodsFor: '*dgv-base extensions' stamp: 'CdG,TJ 2/3/2005 12:08'!
forceNewFileNamed: aString
	| strm |
	strm _  self on: (FileStream forceNewFileNamed: aString).		"will be binary"
	strm byteStream setFileTypeToObject.
		"Type and Creator not to be text, so can attach correctly to an email msg"
	^ strm! !

TestCase subclass: #DGVImageGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

!DGVImageGeneratorTest methodsFor: 'as yet unclassified' stamp: 'CdG 11/1/2005 18:13'!
livetestFindPackageName
	| rep targetVersion fileNames fileName fullPath depName depPath |
	rep := MCCacheRepository default.
	targetVersion := (((rep allVersionNames
				select: [:each | each beginsWith: 'DGVwxUI'])
				collect: [:each | (each copyAfterLast: $.)
						-> each])
				asSortedCollection: [:a :b | a key asNumber < b key asNumber]) last value.
	fileNames := rep allFileNamesForVersionNamed: targetVersion.
	self assert: fileNames notEmpty.
	fileName := fileNames first.
	self assert: fileName = 'DGVwxUI-CdG.238.mcz'.
	fullPath := rep directory fullPathFor: fileName.
	self assert: fullPath = 'D:\Kolibri\Squeak04\package-cache\DGVwxUI-CdG.238.mcz'.
	(rep versionFromFileNamed: fileName)
		withAllDependenciesDo: [:each | 
			depName := (rep allFileNamesForVersionNamed: each info name) first.
			depPath := rep directory fullPathFor: depName]
		ifUnresolved: [TDSDebuggerSupport fatal: 'dependency not found!!']! !

!SrpState methodsFor: '*dgv-overrides' stamp: 'EG 11/23/2005 21:31'!
srpPostLoadAction
	"Answer a command (directed message or block) that
	 will answer an object to take the place of the
	 receiver when sent #value. The command will be
	 sent #value after the root object has been completely
	 loaded. Answer nil if no further processing is required."

	^[ Error signal: 'Unmarshalling error' ].

! !

LintDialog subclass: #DGVLintDialog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

DGVLintDialog subclass: #DGVBugsLintDialog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

!DGVBugsLintDialog commentStamp: '<historical>' prior: 0!
Open a SmallLint dialog configured to run a bugs-only lint over our stuff.

DGVBugsLintDialog open run!

!DGVBugsLintDialog methodsFor: 'as yet unclassified' stamp: 'CdG 4/8/2005 00:01'!
initialize
	super initialize.
	testCategories listSelectionAt: 2 put: true "bugs".
	"testCategories listSelectionAt: 3 put: true" "possible bugs".
	tests selectAll.
	self selectOurClasses! !

DGVLintDialog subclass: #DGVFullLintDialog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

!DGVFullLintDialog commentStamp: '<historical>' prior: 0!
Open a SmallLint dialog setup to run a full lint over our stuff!

!DGVFullLintDialog methodsFor: 'as yet unclassified' stamp: 'CdG 4/7/2005 23:57'!
initialize
	super initialize.
	testCategories selectAll.
	tests selectAll.
	self selectOurClasses! !

!DGVLintDialog methodsFor: 'as yet unclassified' stamp: 'CdG 6/7/2005 16:35'!
selectOurClasses
	| cat cg |
	self fixMe: 'Dirty code - could/should be rewritten with Environments'.
	selector _ CategoryAndClassSelector new.
	cg _ [selector instVarNamed: #classCategories].
	1 to: cg value list size
		do: 
			[:i | 
			cat := cg value list at: i.
			((DGVInstaller isOurOwnStuff: cat) and: [(cat endsWith: 'Tests') not]) 
				ifTrue: [cg value listSelectionAt: i put: true]].
	(selector instVarNamed: #classes) selectAll.
	self environment: selector selectedClassEnvironment.
! !

ExternalLibrary subclass: #DGVOSSupport
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-OS-Support'!

DGVOSSupport class
	instanceVariableNames: 'soleInstance'!

!DGVOSSupport class methodsFor: 'private' stamp: 'EG 4/11/2005 13:31'!
activeOSSupportClass
	^self allSubclasses
		detect: [:class | class isActiveOSSupportClass]
		ifNone: [self error: 'no platform support for this operating system!!']
! !

!DGVOSSupport class methodsFor: 'instance creation' stamp: 'CdG 1/26/2005 22:21'!
default
	^soleInstance ifNil: [
			Smalltalk addToStartUpList: self.
			soleInstance _ self activeOSSupportClass new]! !

!DGVOSSupport class methodsFor: 'private' stamp: 'CdG 1/26/2005 21:48'!
isActiveOSSupportClass
	^false! !

!DGVOSSupport class methodsFor: 'instance creation' stamp: 'TJ,CdG 2/18/2005 13:44'!
makeReadyForTesting
	^soleInstance _ DGVTestSupport new! !

!DGVOSSupport class methodsFor: 'private' stamp: 'TJ 2/16/2005 20:34'!
startUp: something
	"The system is coming up. If it is on a new platform, clear out the existing handles."
	soleInstance _ nil "We might have changed operating systems"! !

!DGVOSSupport class methodsFor: 'instance creation' stamp: 'TJ,CdG 2/18/2005 13:45'!
testingDone
	^soleInstance ifNotNil: [
			soleInstance clean.
			soleInstance _ nil]! !

!DGVOSSupport methodsFor: 'files and directories' stamp: 'TJ,CdG 3/7/2005 14:04'!
cacheDirectory
	"sensible default, may be overridden"
	| base sub |
	base _ self homeDirectory.
	sub _ '.dgv-cache'.
	(base directoryExists: sub) ifFalse: [base createDirectory: sub].
	^base directoryNamed: sub! !

!DGVOSSupport methodsFor: 'as yet unclassified' stamp: 'JJ 11/24/2005 15:19'!
critical: aBlockContext 
	mutex ifNil: [mutex _ Mutex new].
	mutex critical: aBlockContext! !

!DGVOSSupport methodsFor: 'files and directories' stamp: 'CdG 11/1/2005 11:24'!
dataDirectory
	"sensible default, may be overridden"
	| base sub |
	base _ self homeDirectory.
	sub _ '.kolibri'.
	(base directoryExists: sub) ifFalse: [base createDirectory: sub].
	^base directoryNamed: sub! !

!DGVOSSupport methodsFor: 'files and directories' stamp: 'CdG 1/28/2005 14:53'!
documentsDirectory
	"sensible default, override if you want"
	^self homeDirectory! !

!DGVOSSupport methodsFor: 'files and directories' stamp: 'CdG 1/26/2005 21:54'!
homeDirectory
	self subclassResponsibility! !

!DGVOSSupport methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:25'!
openTempFileWithName: originalFileName contents: contents
	| fs tries |
	
	tries := 3.
	[tries > 0] whileTrue: [
		self critical: [
			[fs _ self tempDirectory forceNewFileNamed: originalFileName]
				on: Error
				do: [
					Smalltalk garbageCollect.
					tries := tries - 1.
					tries = 0 ifTrue: [DGVFileLockedException signal: originalFileName asString].
					fs := nil].
			fs ifNotNil: [
				[fs 
					binary; 
					nextPutAll: contents.
				tries := 0]
					ensure: [fs close]]]].

	^self open: (self tempDirectory fullNameFor: originalFileName)
! !

!DGVOSSupport methodsFor: 'as yet unclassified' stamp: 'TJ 2/4/2005 09:43'!
open: filename
	"Open this file with system default handler"
	self subclassResponsibility! !

!DGVOSSupport methodsFor: 'files and directories' stamp: 'CdG 1/26/2005 21:54'!
tempDirectory
	self subclassResponsibility! !

DGVOSSupport subclass: #DGVTestSupport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-OS-Support'!

!DGVTestSupport methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 13:42'!
clean
	self homeDirectory recursiveDelete! !

!DGVTestSupport methodsFor: 'as yet unclassified' stamp: 'TJ 4/12/2005 12:34'!
dataDirectory
	^ self homeDirectory! !

!DGVTestSupport methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 13:40'!
homeDirectory
	^(FileDirectory default directoryNamed: 'DGVTest') assureExistence.! !

!DGVTestSupport methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 10:28'!
open: aFileName
	self shouldNotImplement "yet. This is for UI's"! !

!DGVTestSupport methodsFor: 'as yet unclassified' stamp: 'TJ 4/12/2005 12:35'!
tempDirectory
	^ self homeDirectory! !

DGVOSSupport subclass: #DGVUnixSupport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-OS-Support'!

!DGVUnixSupport class methodsFor: 'as yet unclassified' stamp: 'EG 4/11/2005 13:24'!
isActiveOSSupportClass
	"Can't think of anything better at the moment".
	"or use SmalltalkImage current platformName = 'linux'"
	^UnixFileDirectory isActiveDirectoryClass! !

!DGVUnixSupport methodsFor: 'files and directories' stamp: 'CdG,EjG 6/8/2005 10:35'!
homeDirectory
	"indirectly get UnixProcess to avoid Lint warnings under Windows"
	^FileDirectory on: ((Smalltalk at: #UnixProcess) env at: 'HOME')! !

!DGVUnixSupport methodsFor: 'files and directories' stamp: 'CdG 4/8/2005 01:05'!
open: aFileName
	self shouldNotImplement "yet. This is for UI's"! !

!DGVUnixSupport methodsFor: 'files and directories' stamp: 'CdG 2/1/2005 11:45'!
tempDirectory
	^FileDirectory on: '/tmp'! !

DGVOSSupport subclass: #DGVWin32Support
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-OS-Support'!

!DGVWin32Support class methodsFor: 'private' stamp: 'EG 4/11/2005 13:23'!
isActiveOSSupportClass
	| img |
	img _ SmalltalkImage current.
	^img platformName = 'Win32' and: [img osVersion = 'NT']! !

!DGVWin32Support methodsFor: 'private' stamp: 'CdG 1/26/2005 23:49'!
apiGetCurrentProcess
	<apicall: long 'GetCurrentProcess' () module: 'kernel'>
	^self externalCallFailed! !

!DGVWin32Support methodsFor: 'private' stamp: 'CdG 1/26/2005 23:32'!
apiGetTempPathBufferSize: aNumber inTo: aString
	<apicall: long 'GetTempPathA' (long char *) module: 'kernel'>
	^self externalCallFailed

! !

!DGVWin32Support methodsFor: 'private' stamp: 'CdG 1/27/2005 00:22'!
apiGetUserProfileDirectoryToken: token into: aString size: aNumberBuffer
	<apicall: long 'GetUserProfileDirectoryA' (long char* long*) module: 'userenv'>
	^self externalCallFailed
! !

!DGVWin32Support methodsFor: 'private' stamp: 'CdG 1/27/2005 00:03'!
apiOpenProcessTokenProcessHandle: procHandle desiredAccess: access tokenHandle: tokHandle
	<apicall: long 'OpenProcessToken' (long long byte*) module: 'advapi'>
	^self externalCallFailed! !

!DGVWin32Support methodsFor: 'files and directories' stamp: 'TJ,CdG 3/7/2005 14:07'!
cacheDirectory
	| dir sub |
	dir _ self homeDirectory directoryNamed: 'Application Data'.
	sub _ dir directoryNamed: 'DGV Cache'.
	^sub assureExistence! !

!DGVWin32Support methodsFor: 'files and directories' stamp: 'CdG 11/1/2005 11:25'!
dataDirectory
	| dir sub |
	dir _ self homeDirectory directoryNamed: 'Application Data'.
	sub _ dir directoryNamed: 'Kolibri'.
	^sub assureExistence! !

!DGVWin32Support methodsFor: 'files and directories' stamp: 'CdG 1/28/2005 14:55'!
documentsDirectory
	| dir |
	dir _ self homeDirectory.
	^(dir directoryExists: 'My Documents') 
		ifTrue: [dir directoryNamed: 'My Documents']
		ifFalse: [dir]! !

!DGVWin32Support methodsFor: 'files and directories' stamp: 'CdG 2/1/2005 11:06'!
homeDirectory
	| tok bufSize buf len lenBuf lenPtr |
	tok _ (ByteArray new: 4).
	self 
		apiOpenProcessTokenProcessHandle: self apiGetCurrentProcess 
		desiredAccess: self kTokenQuery 
		tokenHandle: tok.
	tok _ tok unsignedLongAt: 1.
	bufSize _ 200.
	buf _ ByteArray new: bufSize.
	lenBuf _ ByteArray new: 4.
	lenBuf longAt: 1 put: bufSize bigEndian: false.
	lenPtr _ ExternalData fromHandle: lenBuf type: ExternalType long asPointerType.
	"Win32 API says returns BOOL and puts len in lenPtr, but that's apparently not true"
	len _ self apiGetUserProfileDirectoryToken: tok into: buf size: lenPtr.
	^FileDirectory on: (buf copyFrom: 1 to: len - 1) asString
! !

!DGVWin32Support methodsFor: 'constants' stamp: 'CdG 1/26/2005 23:48'!
kTokenQuery
	^8! !

!DGVWin32Support methodsFor: 'files and directories' stamp: 'JJ 11/22/2005 15:35'!
open: filename
	^Win32Shell new shellOpen: filename! !

!DGVWin32Support methodsFor: 'files and directories' stamp: 'CdG 1/27/2005 00:48'!
tempDirectory
	| bufSize buf retSize |
	bufSize _ 200.
	buf _ ByteArray new: bufSize.
	retSize _ self apiGetTempPathBufferSize: bufSize inTo: buf.
	^FileDirectory on: (buf copyFrom: 1 to: retSize) asString
	! !

DGVWin32Support subclass: #DGVWin98Support
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-OS-Support'!

!DGVWin98Support class methodsFor: 'as yet unclassified' stamp: 'EG 4/11/2005 13:31'!
isActiveOSSupportClass
	| img |
	img _ SmalltalkImage current.
	^img platformName = 'Win32' and: [img osVersion = '95']! !

!DGVWin98Support methodsFor: 'as yet unclassified' stamp: 'EG 4/11/2005 14:08'!
homeDirectory
	^FileDirectory on: 'C:\Local Settings'! !

!UndefinedObject methodsFor: '*dgv-model' stamp: 'EG,CdG 8/18/2005 11:13'!
dgvMatchTextSearch: pattern
	^false! !

!UndefinedObject methodsFor: '*dgv-base extensions' stamp: 'CdG 8/12/2005 14:08'!
editableString
	^''! !

!UndefinedObject methodsFor: '*dgv-base extensions' stamp: 'CdG 8/12/2005 14:19'!
htmlRepresentation
	^''! !

HVTransientView subclass: #DGVConfirmationMailAcceptanceService
	instanceVariableNames: ''
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVConfirmationMailAcceptanceService class methodsFor: 'accessing' stamp: 'CdG 2/9/2005 13:28'!
baseUrl
	^'http://', self defaultHostname, ':', self defaultPort printString, '/confirm'! !

!DGVConfirmationMailAcceptanceService class methodsFor: 'accessing' stamp: 'CdG 3/25/2005 20:09'!
defaultHostname
	^ DGVConfiguration default getSection: 'confirmationmailacceptanceservice' key: 'hostname' default: (NetNameResolver localHostName)! !

!DGVConfirmationMailAcceptanceService class methodsFor: 'accessing' stamp: 'CdG 6/14/2005 11:38'!
defaultPort
	^ ( DGVConfiguration default getSection: 'ConfirmationMailAcceptanceService' key: 'port' default: '8001') asInteger! !

!DGVConfirmationMailAcceptanceService class methodsFor: 'startup' stamp: 'CdG 10/28/2005 16:39'!
start
	Instance ifNil: [
		Instance _ self startOn: self defaultPort]! !

!DGVConfirmationMailAcceptanceService class methodsFor: 'startup' stamp: 'CdG 10/28/2005 16:40'!
stop
	[super stop. Instance := nil]
		on: Error
		do: [:ex | TDSDebuggerSupport logException: ex]! !

!DGVConfirmationMailAcceptanceService methodsFor: 'urls' stamp: 'CdG 3/25/2005 20:42'!
confirm
	| sign b userId communityId service serviceid |
	userId _ UUID fromString36: (req getFields at: 'userId' 
		ifAbsent: [^self invalidUrlError]).
	communityId _ UUID fromString36: (req getFields at: 'communityId'
		 ifAbsent: [^self invalidUrlError]).
	serviceid _ UUID fromString36: (req getFields at: 'service' 
		ifAbsent: [^self invalidUrlError]).
	sign _ req getFields at: 'sign' 
		ifAbsent: [^self invalidUrlError].
	(service _ DGVPolicyService get: serviceid)
		ifNil: [^self invalidUrlError].
	(service receivedConfirmationFor: userId community: communityId signature: sign)
		ifFalse: [^self invalidUrlError].

	b _ self builder.	
	b start; h1: 'Dankuwel'.
	service buildHtmlConfirmationInto: b.
	^b! !

!DGVConfirmationMailAcceptanceService methodsFor: 'private' stamp: 'CdG 2/9/2005 17:31'!
invalidUrlError

	| b |
	b _ self builder.	
	b start; h1: 'Foute URL';
		html: 'Deze URL is niet bekend op het systeem.'.
	^b! !

HVTransientView subclass: #DGVStatisticsWebServer
	instanceVariableNames: ''
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'DGV-Network-Service'!

!DGVStatisticsWebServer class methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:33'!
defaultPort
	^ ( DGVConfiguration default getSection: 'StatisticsWebServer' key: 'port' default: '51920') asInteger! !

!DGVStatisticsWebServer class methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:33'!
start
	Instance ifNotNil: [[self stop] 
		on: Error 
		do: [:ex | TDSDebuggerSupport logException: ex]].
	Instance _ self startOn: self defaultPort! !

!DGVStatisticsWebServer class methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:33'!
stop
	[super stop]
		on: Error
		do: [:ex | TDSDebuggerSupport logException: ex]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:38'!
averageTimeBetweenUserLogins
	^(DGVUser localStorage activeItems collect: [:each |
		(DGVNode default networkTimeStamp - each creationTime) /
		(each loginCount max: 1)]) average
! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG,EG 12/9/2005 15:15'!
default
	| b |
	b _ self builder.
	b start; h1: 'Statistics for node ', DGVNode default nodeId printString; br.
	b table: [b tableBody: [
		self statisticsSelectors do: [:each |
			b tableRow: [
				b tableData: (self titleForSelector: each).
				b tableData: ([self perform: each] on: Error do: ['Error'])
			]
		]]
	].
	^b! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:49'!
numberOfBlobs
	^DGVBlob localStorage items size! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:49'!
numberOfComments
	^DGVComment localStorage items size! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:24'!
numberOfCommentsPerFile
	^self numberOfFiles = 0
		ifFalse: [(self numberOfComments / self numberOfFiles) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:19'!
numberOfCommentsPerUser
	^self numberOfUsers = 0
		ifFalse: [(self numberOfComments / self numberOfUsers) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:52'!
numberOfCommunities
	^DGVCommunity localStorage items size! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:04'!
numberOfCommunityManagers
	| answer |
	answer := Set new.	
	^(((DGVCommunity localStorage activeItems reject: [:each | each isRootCommunity])
		inject: answer into: [:set :each | set , each managers , {each owner}]))
		size
	! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:59'!
numberOfDownloads
	^DGVNodeStatistics localStorage activeItems inject: 0
		into: [:sum :each | sum + each totalBlobDownloads]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:20'!
numberOfDownloadsPerFile
	^self numberOfFiles = 0
		ifFalse: [(self numberOfDownloads / self numberOfFiles) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:17'!
numberOfDownloadsPerUser
	^(self numberOfUsers = 0) 
		ifFalse: [(self numberOfDownloads / self numberOfUsers) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:50'!
numberOfFiles
	^DGVFile localStorage items size! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:21'!
numberOfFilesPerUser
	^self numberOfUsers = 0 
		ifFalse: [(self numberOfFiles / self numberOfUsers) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:31'!
numberOfLocallyPublishedFiles
	^(self numberOfFiles - self numberOfBlobs) max: 0! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:33'!
numberOfLoginsPerUser
	^(DGVUser localStorage activeItems collect: [:each |
		each loginCount]) average asFloat

! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:21'!
numberOfPublicationsPerCommunity
	^self numberOfCommunities = 0
		ifFalse: [(self numberOfFiles / self numberOfCommunities) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:50'!
numberOfRelations
	^DGVRelation localStorage items size! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:22'!
numberOfRelationsPerFile
	^self numberOfFiles = 0
		ifFalse: [(self numberOfRelations / self numberOfFiles) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'JJ 12/7/2005 09:22'!
numberOfRelationsPerUser
	^self numberOfUsers = 0 
		ifFalse: [(self numberOfRelations / self numberOfUsers) asFloat]
		ifTrue: [0]! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:50'!
numberOfUsers
	^DGVUser localStorage items size! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:02'!
numberOfUsersWithAtLeastOneMembership
	| answer |
	answer := Set new.	
	^(((DGVCommunity localStorage activeItems reject: [:each | each isRootCommunity])
		inject: answer into: [:set :each | set , each users]))
		size
	! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 16:00'!
statisticsSelectors
	^#(#numberOfUsers 
		#numberOfFiles #numberOfFilesPerUser
		#numberOfComments #numberOfCommentsPerUser
		#numberOfRelations #numberOfRelationsPerUser
		#numberOfCommentsPerFile #numberOfRelationsPerFile
		#numberOfBlobs
		#numberOfLocallyPublishedFiles
		#numberOfCommunities
		#numberOfUsersWithAtLeastOneMembership
		#numberOfCommunityManagers
		#numberOfPublicationsPerCommunity
		#numberOfLoginsPerUser #averageTimeBetweenUserLogins
		#numberOfDownloads #numberOfDownloadsPerUser #numberOfDownloadsPerFile)! !

!DGVStatisticsWebServer methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 14:42'!
titleForSelector: aSymbol
	^aSymbol asString! !

!ProtoObject methodsFor: '*dgv-base extensions' stamp: 'TJ,CdG 2/8/2005 12:19'!
ifNil: nilBlock ifNotNil: ifNotNilBlock
	"Evaluate the block, unless I'm == nil (q.v.)"

	^ ifNotNilBlock valueWithPossibleArgs: (Array with: self)! !

!ProtoObject methodsFor: '*dgv-base extensions' stamp: 'TJ,CdG 2/8/2005 12:19'!
ifNotNil: ifNotNilBlock
	"Evaluate the block, unless I'm == nil (q.v.)"

	^ ifNotNilBlock valueWithPossibleArgs: (Array with: self)! !

!ProtoObject methodsFor: '*dgv-base extensions' stamp: 'TJ,CdG 2/8/2005 12:19'!
ifNotNil: ifNotNilBlock ifNil: nilBlock 
	"If I got here, I am not nil, so evaluate the block ifNotNilBlock"

	^ ifNotNilBlock valueWithPossibleArgs: (Array with: self)! !

!ProtoObject methodsFor: '*dgv-model' stamp: 'CdG 2/14/2005 17:15'!
isDGVLink
	^false! !

!ProtoObject methodsFor: '*dgv-model' stamp: 'CdG 12/10/2005 21:45'!
isDGVObject
	^false! !

!Win32Shell methodsFor: '*dgv-overrides' stamp: 'JJ 11/22/2005 15:35'!
shellOpen: aFileString
	"Opens the file specified by aFileString. The file can be an executable file, a document file, 
	 or a folder."

	"We have noticed that Word returns err=1 in some circumstances. As this error is not
      documented by Microsoft, we ignore it - the document is shown and opened in these
      cases, so hopefully no harm done..."

	| err |
	err := self shellExecute: nil 
		lpOperation: 'open'
		lpFile: aFileString
		lpParameters: nil
		lpDirectory: nil
		nShowCmd: 1.

	(err > 1 and: [err <= 32]) ifTrue: [
		TDSDebuggerSupport logError: 'system error'].
	^err! !

!TaskScheduler methodsFor: '*dgv' stamp: 'EG,CdG 6/22/2005 14:42'!
isRunning
	^running! !

!String methodsFor: '*dgv-model' stamp: 'EG,CdG 6/28/2005 11:27'!
isUrl
	^(Url schemeNameForString: self) notNil! !

!Collection methodsFor: '*dgv-base extensions' stamp: 'CdG 5/2/2005 15:32'!
removeAll
	self copy do: [:each | self remove: each]! !

!UnhandledError methodsFor: '*dgv-base overrides' stamp: 'EG,CdG 6/30/2005 10:15'!
runtimeDefaultAction
	"Dump the stack trace to a log file, then exit the program (image)."

	| file fileName text |
	fileName _ ('error', Utilities dateTimeSuffix, FileDirectory dot, 'log') asFileName.
	fileName _ FileDirectory default fullNameFor: fileName.
	file := FileStream forceNewFileNamed: fileName.
	file ascii.
	Smalltalk timeStamp: file.
	'Kolibri version: ' printOn: file.
	DGVApplication version printOn: file.
	thisContext sender stack do: [:ctx | file cr. ctx printOn: file].
	file close.

	text _ 'Een fout is opgetreden. Het programma sluit daarom af.
	 Wilt u zo vriendelijk zijn om een foutrapport, met als attachment het bestand 
	''', fileName, ''' 
	naar kolibri@aardworks.nl te sturen?'.
	(Smalltalk includesKey: #WxMessageDialog) 
		ifTrue: [
			(WxMessageDialog
				parent: nil
				message: text
				caption: 'Fatale fout' translated) 
				showModal]
		ifFalse: [
			self inform: text].
	SmalltalkImage current snapshot: false andQuit: true! !

TestRunner subclass: #DGVTestRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Development Stuff'!

!DGVTestRunner class methodsFor: 'as yet unclassified' stamp: 'CdG 7/4/2005 11:37'!
open
	| runner |
	runner _ self new.
	runner openAsMorph.
	runner selectTestsSuchThat: [:t | (#('DGV' 'OAI' 'TU' 'AWG') select: [:prefix | t beginsWith: prefix]) notEmpty]! !

TDSTestCase subclass: #DGVTestCase
	instanceVariableNames: 'oldHaltOnErrorsFlag configuration'
	classVariableNames: 'TheOldInstance'
	poolDictionaries: ''
	category: 'DGV-Tests'!

DGVTestCase subclass: #DGVBaseExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVBaseExtensionTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 4/12/2005 12:00'!
testPrintKiMiGiStringOn
	| s |
	s _ ReadWriteStream on: ''.
	1235567 printKiMiGiStringOn: s.
	self assert: s contents = '1.1M'.

	s _ ReadWriteStream on: ''.
	12534 printKiMiGiStringOn: s.
	self assert: s contents = '12.2k'
! !

DGVTestCase subclass: #DGVBlobInfoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVBlobInfoTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/1/2005 11:18'!
testChecksumStability
	| blob bi1 bi2 |
	blob _ DGVBlob fromFileName: 'wxsqueak.ini'.
	blob updateLastSavedTime.

	bi1 _ DGVBlobInfo for: blob.

	blob updateLastSavedTimeAsExpired "so it is sure to differ...".

	bi2 _ DGVBlobInfo for: blob.

	self assert: bi1 checksum = bi2 checksum! !

DGVTestCase subclass: #DGVBlobTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVBlobTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:36'!
testFromFileName
	| fileName bytes blob id |
	fileName _ 'wxsqueak.ini'.
	bytes _ (FileStream readOnlyFileNamed: fileName) binary; contentsOfEntireFile.
	
	blob _ DGVBlob fromFileName: fileName.
	blob save.
	
	"check that da bytes have been read in correctly"
	self assert: blob data = bytes.
	
	"check that blob was saved"
	id _ blob id.
	blob _ DGVBlob getLocally: id mayRefreshFromNetwork: false.
	
	self assert: blob notNil.
	self assert: blob data = bytes.! !

!DGVBlobTest methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 10:13'!
testFromFileNameBinary
	| fileName blob data binfile |

	
	fileName _ 'blobbinarytest.tst'.
	data _ #(0 13 26 224 13 10 9) asByteArray.
	binfile _ FileStream forceNewFileNamed: fileName.
	binfile binary; nextPutAll: data; close.
	
	blob _ DGVBlob fromFileName: fileName.
	
	"check that da bytes have been read in correctly"
	self assert: blob data = data.! !

!DGVBlobTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/1/2005 11:16'!
testStableCopyForChecksumming
	| fileName blob blob2 |
	fileName _ 'wxsqueak.ini'.
	
	blob _ DGVBlob fromFileName: fileName.
	blob save.
	self deny: blob lastSavedTime isNil.

	blob2 _ blob stableCopyForChecksumming.

	self assert: blob2 basicLastSavedTime isNil.! !

DGVTestCase subclass: #DGVBlobTransferTest
	instanceVariableNames: 'nodes shuntUUID shuntBlockArg'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVBlobTransferTest methodsFor: 'mocking' stamp: 'TJ 3/10/2005 15:40'!
createBlobInfoVersion: version
	^ self createBlobInfoVersion: version	dataSize: DGVBlobTransfer blockSize + 1.
! !

!DGVBlobTransferTest methodsFor: 'mocking' stamp: 'TJ 3/10/2005 15:39'!
createBlobInfoVersion: version dataSize: size
	^(DGVBlobInfo new)
		id: (UUID fromString: '9b93121d-0255-684b-9fde-f72d7e2e0b31');
		version: version;
		dataSize: size;
		checksum: 1! !

!DGVBlobTransferTest methodsFor: 'self shunt' stamp: 'EG,CdG 7/7/2005 14:11'!
for: anUUID whenFound: aBlockContext 
	| blobInfo |
	shuntUUID _ anUUID.
	"the block must have an association (blobInfo -> nodeId) as argument"
	aBlockContext value: shuntBlockArg.
	"next time right"
	blobInfo _ DGVBlobInfo new.
	blobInfo version: 1.
	blobInfo dataSize: 10000.
	shuntBlockArg _ blobInfo -> anUUID.! !

!DGVBlobTransferTest methodsFor: 'mocking' stamp: 'TJ 3/10/2005 15:59'!
nodeIdA
	^ UUID fromString: 'f80098e6-05d2-fb44-82c0-2e937534b4f5'! !

!DGVBlobTransferTest methodsFor: 'mocking' stamp: 'TJ 3/10/2005 15:59'!
nodeIdB
	^ UUID fromString: '15db24fa-a376-b549-92fd-eebd1779a631'! !

!DGVBlobTransferTest methodsFor: 'mocking' stamp: 'TJ 3/10/2005 15:59'!
nodeIdC
	^ UUID fromString: 'ee391a89-ef93-044c-ae9a-4570badb3596'! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'TJ 3/10/2005 15:42'!
testCreateTodoList
	| blobber |
	blobber _ DGVBlobTransfer new.
	blobber selectedBlobInfo: (self createBlobInfoVersion: 3 dataSize: 102400).
	blobber createTodoList.
	self assert: 10 = blobber todoList size.
	blobber selectedBlobInfo: (self createBlobInfoVersion: 3 dataSize: 102401).
	blobber createTodoList.
	self assert: 11 = blobber todoList size

	! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'EG,CdG 10/14/2005 14:27'!
testEqualsAndHash
	| blobber1 blobber2 blobber3 blobber4 |
	blobber1 := DGVBlobTransfer new.
	blobber1 transferName: 'aap'.
	blobber1 fileName: 'noot'.
	blobber2 := DGVBlobTransfer new.
	blobber2 transferName: 'aap'.
	blobber2 fileName: 'noot'.
	self assert: blobber1 = blobber2.
	self assert: blobber1 hash = blobber2 hash.

	blobber3 := DGVBlobTransfer new.
	blobber3 transferName: 'mies'.
	blobber3 fileName: 'noot'.
	self deny: blobber1 = blobber3.

	blobber4 := DGVBlobTransfer new.
	blobber4 transferName: 'aap'.
	blobber4 fileName: 'teun'.
	self deny: blobber1 = blobber4.

	self deny: blobber4 = 42
! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 13:56'!
testFinishDownload
	| blobber version3 result blob blobAsBytes |
	blobber := DGVBlobTransfer new.
	version3 _ (self createBlobInfoVersion: 3).
	blob _ DGVBlob new.
	blobAsBytes _ blob marshallAsByteArray.
	version3 data: blobAsBytes.
	blobber selectedBlobInfo: version3.
	blobber doneList at: 1 put: (blobAsBytes copyFrom: 1 to: 10).
	blobber doneList at: 2 put: (blobAsBytes copyFrom: 11 to: blobAsBytes size).
	result _ blobber buildResult.
	self assert: result = blob! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'CdG 7/11/2005 12:00'!
testGet
	" Without really testing #get itself. It is a template method,
	 with some calls to networklayer. The results of the networklayer
	 are simulated here.
	"

	| blobber |
	blobber := DGVBlobTransfer new.
	"get: aUUID
	 fromNetwork retrieve blobInfo's
	"
	blobber receivedQueryBlobInfoResult: (self createBlobInfoVersion: 3) 
				-> self nodeIdA.
	blobber receivedQueryBlobInfoResult: (self createBlobInfoVersion: 3) 
				-> self nodeIdB.
	blobber receivedQueryBlobInfoResult: (self createBlobInfoVersion: 2) 
				-> self nodeIdC.
	"
	selectBlobInfoToUse
	"
	blobber selectBlobInfoToUse.
	self assert: blobber selectedBlobInfo = (self createBlobInfoVersion: 3)! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'TJ 3/10/2005 16:33'!
testHasFreeBlocks
	| blobber version3 |
	blobber := DGVBlobTransfer new.
	version3 _ (self createBlobInfoVersion: 3).
	blobber selectedBlobInfo: version3.
	blobber createTodoList.
	self assert: blobber hasFreeBlocks.
	
	! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'CdG 4/1/2005 14:42'!
testHasFreeNodes
	| blobber version3 |
	blobber := DGVBlobTransfer new.
	version3 _ (self createBlobInfoVersion: 3).
	blobber selectedBlobInfo: version3.
	blobber workNodes at: self nodeIdA put: 0.
	self assert: blobber hasFreeNodes.
	blobber workNodes at: self nodeIdA put: (DGVBlobTransfer activeBlocksPerNode).
	self deny: blobber hasFreeNodes.
	! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'EG,CdG 9/2/2005 16:15'!
testIncreasingBlobInfoQueryTimeOut
	| blobTransfer |
	blobTransfer := DGVBlobTransfer new.
	self assert: blobTransfer timeout = 10 seconds.
	self assert: blobTransfer timeout = 20 seconds.
	self assert: blobTransfer timeout = 40 seconds.
	self assert: blobTransfer timeout = 80 seconds.
	self assert: blobTransfer timeout = 160 seconds.
	self assert: blobTransfer timeout = 320 seconds.
	self assert: blobTransfer timeout = 640 seconds.
	self assert: blobTransfer timeout = 1280 seconds.
	self assert: blobTransfer timeout = 2560 seconds.
	self assert: blobTransfer timeout = 5120 seconds. "85 minutes, maxmize on 2h"
	self assert: blobTransfer timeout = 2 hours
! !

!DGVBlobTransferTest methodsFor: 'tests' stamp: 'CdG 7/11/2005 12:00'!
testUpdateNodeList
	| blobber version3 |
	blobber := DGVBlobTransfer new.
	version3 := self createBlobInfoVersion: 3.
	blobber receivedQueryBlobInfoResult: version3 -> self nodeIdA.
	blobber receivedQueryBlobInfoResult: (self createBlobInfoVersion: 3) 
				-> self nodeIdB.
	blobber receivedQueryBlobInfoResult: (self createBlobInfoVersion: 2) 
				-> self nodeIdC.
	blobber selectedBlobInfo: (self createBlobInfoVersion: 3).
	self assert: (blobber receivedResponses at: self nodeIdA) = version3.
	blobber updateNodeList.
	self deny: (blobber receivedResponses includesKey: self nodeIdA).
	self assert: blobber workNodes size = 2.
	self assert: (blobber workNodes includesKey: self nodeIdA).
	self assert: (blobber workNodes includesKey: self nodeIdB)! !

DGVTestCase subclass: #DGVCollectionTest
	instanceVariableNames: 'presences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG,CdG 9/7/2005 13:34'!
makeTestCollectionOfSize: collSize withDeletedItems: deletedSize andUnresolvableItems: unresolvableSize 
	| answer |
	answer := DGVCollection basicNew initialize.
	1
		to: collSize
		do: [:i | answer add: (DGVMockObject new save; yourself)].
	1
		to: deletedSize
		do: [:i | answer add: (DGVMockObject new delete; save; yourself)].
	1
		to: unresolvableSize
		do: [:i | answer
				add: (DGVLink for: DGVMockObject new)].
	^answer! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG 5/3/2005 19:31'!
setUp
	"reset signaling stuff"
	DGVCollection initialize.
	super setUp! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG 5/2/2005 15:30'!
testActiveItemsChanged
	| coll1 coll2 object |
	coll1 _ DGVCollection newWithId: 'test'.
	coll2 _ DGVCollection newWithId: 'test'.
	object _ DGVMockObject new.
	
	coll1 items add: object.
	coll1 activeItemsChanged.
	
	self assert: (coll2 items includes: object).
	
	coll2 items remove: object.
	coll2 activeItemsChanged.
	
	self deny: (coll1 items includes: object).! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG,CdG 9/7/2005 14:07'!
testAddAll
	| coll o |
	coll _ DGVCollection newWithId: '345'.
	o := DGVMockObject new.
	coll addAll: {o. DGVMockObject new. DGVMockObject new}.
	self assert: coll size = 3.
	coll addAll: {o. DGVMockObject new. DGVMockObject new}.
	self assert: coll size = 5.! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG 12/9/2005 11:29'!
testAddAndReplaceWithNewerVersion
	| coll obj1 obj2 |
	coll _ DGVCollection newWithId: '345'.
	obj1 := DGVMockObject new.

	self assert: coll size = 0.
	coll addAll: {obj1.}.
	self assert: coll size = 1.

	obj2 := DGVMockObject new id: obj1 id.
	self assert: obj1 = obj2.
	obj2 increaseVersion.

	coll addAll: {obj2.}.
	self assert: coll size = 1.
	self assert: coll anyOne version = obj2 version.! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'TJ 5/4/2005 16:29'!
testAddTrigger
	| mock coll a b  ce |
	a _ 'Nothing happened'.
	b _ [a _ 'Something did happen.'].

	mock _ DGVMockObject new.
	coll _ DGVCollection newWithId: 'test'.

	self assert: DGVNode default presences size = 0.	"DGVNode default returns a DGVMockNode"

	coll when: #activeItemsChanged send: #value to: b.
	coll add: mock.
	(Delay forMilliseconds: 200) wait.
	
	self assert: DGVNode default presences size = 1.
	ce _ DGVNode default presences first.
	self assert: (ce isKindOf: DGVCollectionElement).
	self assert: ce collectionId = coll id.
	self assert: ce elementId = mock id.
	self assert: ce elementClass = mock class.
	self assert: ce isAddition.
	
	self assert: coll size = 1.
	self assert: a = 'Something did happen.'! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG,CdG 9/7/2005 14:16'!
testAddWithoutPresenceUpdate
	| mock coll  |

	mock _ DGVMockObject new.
	coll _ DGVCollection newWithId: 'test'.
	coll dirty: false.

	coll addWithoutPresenceUpdate: mock id elementClass: mock class.

	self assert: coll size = 0.
	self assert: coll dirty.

	coll items anyOne resetLastResolveAttempt.

	mock save.

	self assert: coll size = 1.! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG,CdG 9/7/2005 13:46'!
testDo
	| coll count |
	coll := self makeTestCollectionOfSize: 0 withDeletedItems: 0 andUnresolvableItems: 0.
	count := 0.
	coll do: [:each| count := count + 1].
	self assert: count = 0.

	coll := self makeTestCollectionOfSize: 1 withDeletedItems: 0 andUnresolvableItems: 0.
	count := 0.
	coll do: [:each| count := count + 1].
	self assert: count = 1.

	coll := self makeTestCollectionOfSize: 0 withDeletedItems: 1 andUnresolvableItems: 0.
	count := 0.
	coll do: [:each| count := count + 1].
	self assert: count = 0.

	coll := self makeTestCollectionOfSize: 0 withDeletedItems: 0 andUnresolvableItems: 1.
	count := 0.
	coll do: [:each| count := count + 1].
	self assert: count = 0.
! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG,EjG 6/8/2005 13:32'!
testGatherElementsFromNetworkFor
	| collection mockInstance |
	mockInstance _ DGVMockQueryStorageGet new.
	mockInstance results: {#(1 2 3). #(4 5 6)}.
	DGVQueryStorageGet testInstance: mockInstance.
	
	[collection := DGVCollection newWithId: '42'.
	DGVCollection gatherElementsFromNetworkFor: collection]
		ensure: [DGVQueryStorageGet testInstance: nil].

	self assert: (collection size = 6)! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG,CdG 9/7/2005 13:33'!
testIsEmpty
	| coll |
	coll := self makeTestCollectionOfSize: 0 withDeletedItems: 0 andUnresolvableItems: 0.
	self assert: coll isEmpty.

	coll := self makeTestCollectionOfSize: 1 withDeletedItems: 0 andUnresolvableItems: 0.
	self deny: coll isEmpty.

	coll := self makeTestCollectionOfSize: 0 withDeletedItems: 1 andUnresolvableItems: 0.
	self assert: coll isEmpty.

	coll := self makeTestCollectionOfSize: 0 withDeletedItems: 0 andUnresolvableItems: 1.
	self assert: coll isEmpty! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG 6/15/2005 14:04'!
testRefreshWithoutNetwork
	| collection newcollection |
	self deny: DGVNode default isConnected.
	collection := DGVCollection newWithId: '89'.
	newcollection _ DGVCollection refresh: collection.
	self assert: newcollection == collection.

! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'TJ 5/4/2005 16:28'!
testRemoveTrigger
	| mock coll a b ce |

	a := 'Nothing happened'.
	b := [a := 'Something did happen.'].
	coll := DGVCollection newWithId: 'test'.
	mock := DGVMockObject new.
	coll add: mock.
	coll save.
	coll 
		when: #activeItemsChanged
		send: #value
		to: b.
	DGVNode default presences removeAll.
	self assert: DGVNode default presences size = 0.
	coll remove: mock.
	(Delay forMilliseconds: 200) wait.
	self assert: DGVNode default presences size = 1.
	ce _ DGVNode default presences first.
	self assert: ce elementId = mock id.
	self assert: ce isRemoval.
	
	self assert: coll size = 0.
	self assert: a = 'Something did happen.'
! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG 5/2/2005 15:44'!
testRemoveWithoutPresenceUpdate
	| mock coll  |

	mock _ DGVMockObject new.
	coll _ DGVCollection newWithId: 'test'.
	coll add: mock.

	self assert: coll size = 1.
	coll dirty: false.

	coll removeWithoutPresenceUpdate: mock id elementClass: mock class.

	self assert: coll size = 0.
	self assert: coll dirty.! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG 5/3/2005 16:55'!
testSaveCollection
	| collection  mockObject parent presenceelement |
	parent _ DGVMockObject new.
	collection _ (DGVCollection parent: parent named: 'test') save.
	self deny: collection p2pAnnounceOnSave.
	self deny: collection dirty.
	self assert: DGVNode default presences isEmpty.

	mockObject _ DGVMockObject new.
	collection add: mockObject.
	self assert: collection dirty.
	self assert: mockObject = collection anyOne.
	self assert: DGVNode default presences size = 1.
	presenceelement _ DGVNode default presences at: 1.
	self assert: presenceelement notNil.
	self assert: presenceelement elementId = mockObject id.
	self assert: presenceelement collectionId = collection id.
	self assert: presenceelement elementClass = mockObject class! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'EG,CdG 9/7/2005 14:10'!
testSelectCollect
	| coll x y z sel |
	coll := DGVCollection newWithId: '345'.
	x := DGVMockObject new.
	y := DGVMockObject new.
	z := DGVMockObject new.
	coll addAll: { x. y. z }.
	sel := coll select: [:each | each = y].
	self assert: sel size = 1.
	self assert: (sel includes: y).
	sel := coll collect: [:each | each id].
	self assert: sel size = 3.
	self assert: (sel includes: x id).
	self assert: (sel includes: y id).
	self assert: (sel includes: z id)! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG 5/3/2005 17:10'!
testSignalingProcess
	| a |
	a _ Set new.
	DGVCollection signalQueue nextPut: [a add: 4].
	(Delay forMilliseconds: 200) wait.
	self assert: a anyOne = 4! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG,EjG 6/8/2005 14:22'!
testSignalingProcessRunsEvenWithError
	| a old |
	a _ Set new.
	old _ TDSDebuggerSupport haltOnErrors: false.
	[DGVCollection signalQueue nextPut: [a add: 4].
	DGVCollection signalQueue nextPut: [self error: 'Expected error from unit test'].
	DGVCollection signalQueue nextPut: [a add: 6].
	(Delay forMilliseconds: 200) wait.
	self assert: a size = 2]
		ensure: [TDSDebuggerSupport haltOnErrors: old]! !

!DGVCollectionTest methodsFor: 'testing' stamp: 'CdG 7/4/2005 11:07'!
testValuesWithUnresolvableLinks
	| link coll |
	link _ DGVMockUnresolvableLink new.
	coll _ DGVCollection newWithId: 'test'.
	coll add: link.
	self should: [link value] raise: DGVLinkCouldNotBeResolvedException.
	self assert: coll values isEmpty.! !

DGVTestCase subclass: #DGVCommentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVCommentTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/4/2005 15:29'!
testCommentWithText
	| comment |
	comment _ DGVComment on: DGVMockObject new.
	comment text: 'Zoals ik al zeg, zei mijn Opa.'.
	self assert: comment text = 'Zoals ik al zeg, zei mijn Opa.'! !

!DGVCommentTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:48'!
testCreateComment
	| object comment |
	object := DGVMockObject new.
	comment := DGVComment on: object.
	self assert: object comments notEmpty.
	self assert: comment parent = object.! !

!DGVCommentTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 10:31'!
testDelete
	| c1 c2 c3 o |
	c1 _ DGVComment on: (o _ DGVMockObject new).
	c2 _ DGVComment on: c1.
	c3 _ DGVComment on: c2.

	o delete.

	self assert: c1 deleted.
	self assert: c2 deleted.
	self assert: c3 deleted.! !

!DGVCommentTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/21/2005 13:38'!
testUpdate
	| comment |
	comment _ DGVComment on: DGVMockObject new.
	comment save.
	self deny: comment dirty.
	comment text: 'Zoals ik al zeg, zei mijn Opa.'.
	self assert: comment dirty! !

DGVTestCase subclass: #DGVCommunityTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:51'!
testAddFile
	| community file |
	community _ DGVCommunity new.
	file _ DGVFile new.
	community addFile: file.
	self assert: community files notEmpty.
	self assert: file parent = community.
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/8/2005 04:22'!
testAddManager
	| com1 user |
	com1 := DGVCommunity new: 'root'.
	user := DGVUser testUser.
	com1 addManager: user.
	self assert: (com1 isManager: user)
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'CdG 8/11/2005 11:34'!
testAddUser
	| community user |
	community _ DGVCommunity new.
	user _ DGVUser testUser.
	community acceptNewUser: user.
	self assert: community users notEmpty.
	self assert: user parent isNil.
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/8/2005 15:13'!
testAssertUniqueName
	| parentCommunity user |
	parentCommunity := DGVCommunity new.
	user := DGVUser testUser.
	parentCommunity owner: user.
	parentCommunity acceptNewUser: user.
	parentCommunity 
				createSubcommunityNamed: 'TestNewCom'
				description: 'A new testcom'
				owner: user
				policyname: DGVNewUserPolicyAlwaysAccept policyName.
	
	self 
		should: [parentCommunity 
			createSubcommunityNamed: 'TestNewCom'
			description: 'A new testcom'
			owner: user
			policyname: DGVNewUserPolicyAlwaysAccept policyName]
		raise: DGVDuplicateCommunityException! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/8/2005 15:11'!
testAssertValidDescription
	| parentCommunity |
	parentCommunity := DGVCommunity new.
	self 
		should: [parentCommunity assertValidDescription: nil] 
		raise: DGVEmptyDescriptionException.
	self 
		should: [parentCommunity assertValidDescription: ''] 
		raise: DGVEmptyDescriptionException.
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/8/2005 15:11'!
testAssertValidName
	| parentCommunity |
	parentCommunity := DGVCommunity new.
	self 
		should: [parentCommunity assertValidName: nil] 
		raise: DGVEmptyNameException.
	self 
		should: [parentCommunity assertValidName: ''] 
		raise: DGVEmptyNameException.
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 10:37'!
testCanSeeFiles
	| community user |
	community _ DGVCommunity new.
	user _ DGVUser testUser.

	self assert: (community canSeeFiles: user).
	community acceptNewUser: user.
	self assert: (community canSeeFiles: user).! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'TJ,EG 2/16/2005 10:40'!
testCommunityName
	| community |
	community _ DGVCommunity new.
	self assert: '' = community communityName.! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 14:22'!
testCreateSubcommunityAndDealWithUnresolvedChildren
	| parentCommunity user |
	parentCommunity := DGVCommunity new.
	user := DGVUser testUser.
	parentCommunity owner: user.
	parentCommunity acceptNewUser: user.
	"Add unresolvable link"
	parentCommunity addChild: (DGVLink class: DGVCommunity id: '123').
	parentCommunity 
		createSubcommunityNamed: 'TestNewCom'
		description: 'A new testcom'
		owner: user
		policyname: DGVNewUserPolicyAlwaysAccept policyName.
	self assert: parentCommunity children size = 1.
	self assert: parentCommunity children items size = 2.! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'TJ 4/21/2005 12:07'!
testCreateSubcommunityNamedDescriptionOwnerPolicyname
	"Test the method #createCommunityNamed:description:parent:owner:policyname:"

	"It's currently somewhere on DGVMainWindow. I think it should be moved."

	| parentCommunity newcom user |
	parentCommunity := DGVCommunity new.
	user := DGVUser testUser.
	parentCommunity owner: user.
	parentCommunity acceptNewUser: user.
	newcom _ parentCommunity 
				createSubcommunityNamed: 'TestNewCom'
				description: 'A new testcom'
				owner: user
				policyname: DGVNewUserPolicyAlwaysAccept policyName.
	self assert: parentCommunity children size = 1.
	self assert: newcom == parentCommunity children anyOne.
	self assert: newcom communityName = 'TestNewCom'.
	self assert: newcom parent == parentCommunity.
	self assert: newcom newUserPolicy policyName 
				= DGVNewUserPolicyAlwaysAccept policyName.
	self assert: newcom description = 'A new testcom'.
	self assert: newcom owner = user.
	self assert: (newcom users includes: user)! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/30/2005 16:11'!
testDeleteCommunity
	| com1 com2 file1 file2 |
	com1 _ DGVCommunity new: 'root'.
	com2 _ DGVCommunity new: 'child'.
	file1 _ DGVFile new.
	file2 _ DGVFile new.

	com1 addChild: com2.
	com1 addFile: file1.
	com2 addFile: file2.

	com1 delete.

	self assert: com1 deleted.
	self assert: com2 deleted.
	self assert: file1 deleted.
	self assert: file2 deleted.
	
	
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 14:36'!
testFindCommunitiesByMember
	| community user myCommunities community2 user2 |
	community _ DGVCommunity new: 'DGVTestComm1'.
	user := (DGVUser testUserWithName: 'barbara' password: 'geheim') save.
	community addUser:  user.
	community save.

	community2 _ DGVCommunity new: 'DGVTestComm2'.
	user2 := (DGVUser testUserWithName: 'arabrab' password: 'geheim') save.
	community2 addUser: user2.
	community2 save.

	myCommunities := DGVCommunity communitiesWithMember: user.
	self assert: (myCommunities includes: community).
	self assert: (myCommunities size = 1)

	
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 12:45'!
testFindCommunitiesByOwner
	| community user myCommunities community2 user2 |
	community _ DGVCommunity new: 'DGVTestComm1'.
	user := (DGVUser testUserWithName: 'barbara' password: 'geheim') save.
	community owner:  user.
	community save.

	community2 _ DGVCommunity new: 'DGVTestComm2'.
	user2 := (DGVUser testUserWithName: 'arabrab' password: 'geheim') save.
	community2 owner:  user2.
	community2 save.
	
	myCommunities := DGVCommunity communitiesWithOwner: user.
	self assert: (myCommunities includes: community).
	self assert: (myCommunities size = 1)

	
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'TJ 4/12/2005 18:03'!
testLazyInitializeIsForbidden
	| community |
	community _ DGVCommunity new: 'DGVTestComm'.
	self assert: community files notNil.
	self assert: community users notNil.
	self assert: community children notNil.
	
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'JJ 11/22/2005 16:47'!
testManagerHierarchy
	| com1 user com2 |
	com1 := DGVCommunity new: 'parent'.
	com2 := DGVCommunity new: 'child'.
	user := DGVUser testUser.
	com1 owner: user.
	com1 addChild: com2.
	com2 parent: com1.
	self assert: (com1 isManager: user).
	self assert: (com2 isManager: user)
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'JJ 11/22/2005 16:48'!
testManagerHierarchy2
	| com1 user com2 |
	com1 := DGVCommunity new: 'parent'.
	com2 := DGVCommunity new: 'child'.
	user := DGVUser testUser.
	com2 owner: user.
	com1 addChild: com2.
	com2 parent: com1.
	self deny: (com1 isManager: user).
	self assert: (com2 isManager: user).

! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/8/2005 04:28'!
testOwnerIsManager
	| com1 user |
	com1 := DGVCommunity new: 'root'.
	user := DGVUser testUser.
	com1 owner: user.
	self assert: (com1 isManager: user)
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/8/2005 04:54'!
testRemoveManager
	| com1 user |
	com1 := DGVCommunity new: 'root'.
	user := DGVUser testUser.
	com1 addManager: user.
	self assert: (com1 isManager: user).

	com1 removeManager: user.
	self deny: (com1 isManager: user).

	self shouldnt: [com1 removeManager: user] raise: Error.
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG 11/1/2005 19:03'!
testRemoveUser
	| community user |
	community := DGVCommunity new.
	user := DGVUser testUser.
	community acceptNewUser: user.
	self assert: community users notEmpty.
	community removeUser: user.
	self
		deny: (user isMemberOfCommunity: community).
	self
		shouldnt: [community removeUser: user]
		raise: Error.
	self
		deny: (community isManager: user)! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG 11/1/2005 19:03'!
testRemoveUserAndManager
	| community user |
	community := DGVCommunity new.
	user := DGVUser testUser.
	community acceptNewUser: user.
	community addManager: user.
	self assert: community users notEmpty.
	community removeUser: user.
	self
		deny: (user isMemberOfCommunity: community).
	self
		shouldnt: [community removeUser: user]
		raise: Error.
	self
		deny: (community isManager: user)! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 20:12'!
testSaveCommunity
	| community |
	community _ DGVCommunity new: 'DGVTestComm'.
	community save.
	self assert: 'DGVTestComm' = community communityName.
	self assert: community version floor = 1.
	
	
! !

!DGVCommunityTest methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 3/7/2005 16:17'!
testSaveLoadCommunity
	| community user file |
	community _ DGVCommunity new: 'DGVTestComm'.
	community save.
	community addUser: (DGVUser testUserWithName: 'barbara' password:'geheim') save; save.
	community addFile: (DGVFile new) save; save.
	community save.
	community _ DGVCommunity get: community id.
	user _ community users anyOne.
	self assert: (user isKindOf: DGVUser).
	file _ community files anyOne.
	self assert: (file isKindOf: DGVFile).

	
! !

DGVTestCase subclass: #DGVConfigurationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVConfigurationTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 14:40'!
testInifile
	| config fs |
	fs _ FileStream forceNewFileNamed: 'test.ini'.
	fs nextPutAll:'
[Section]
whatever=yeah
[OtherSection]
whatever=bar
'.
	fs close.
	config _ DGVConfiguration new filename: 'test.ini'.
	self assert: 'yeah' = (config getSection: 'Section' key: 'whatever' default: 'Default')! !

!DGVConfigurationTest methodsFor: 'as yet unclassified' stamp: 'TJ,CdG 2/18/2005 12:18'!
testNoInifile
	| config |
	config _ DGVConfiguration new filename: 'blablabla.ini'.
	self assert: 'Default' = (config getSection: 'Section' key: 'whatever' default: 'Default')! !

DGVTestCase subclass: #DGVDublinCoreTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVDublinCoreTest methodsFor: 'as yet unclassified' stamp: 'TJ+CdG 1/18/2005 10:45'!
testAddTitle
	| dc |
	dc := DGVDublinCore new.
	dc addTitle: 'test' .
	self assert: dc title = 'test'.
	dc addTitle: 'testAnother' .
	self assert: dc title = 'test'.
	self assert: dc titles size = 2.
	self assert: dc titles first = 'test'.
	self assert: dc titles second = 'testAnother'! !

!DGVDublinCoreTest methodsFor: 'as yet unclassified' stamp: 'TJ+CdG 1/18/2005 10:32'!
testCreateNew
	| dc |
	dc := DGVDublinCore new.
	self assert: dc title isNil.
	self assert: dc titles isEmpty! !

!DGVDublinCoreTest methodsFor: 'as yet unclassified' stamp: 'EG 6/20/2005 13:48'!
testCreateNewFromOAI
	| dc |
	dc := DGVDublinCore fromOAIDC: OAITest new getMockRecord metadata.
	self assert: dc title = 'Static Histomorphometry and Bone Strength of the Human Spine and Iliac Crest : relation to Age and Sex'.
	self assert: dc identifier = 'http://urn.kb.se/resolve?urn=urn:nbn:dk:au:diva-1'.
	self assert: dc formats size = 3! !

!DGVDublinCoreTest methodsFor: 'as yet unclassified' stamp: 'TJ+CdG 1/18/2005 10:36'!
testSetTitle
	| dc |
	dc := DGVDublinCore new.
	dc title: 'test'.
	self assert: dc title = 'test'
! !

DGVTestCase subclass: #DGVExceptionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:48'!
testDGVLinkCouldNotBeResolvedException
	| link |
	[
	link := DGVLink class: Object id: 2344.
	DGVLinkCouldNotBeResolvedException new link: link signal.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'Dit document kon niet van netwerk worden gehaald: a DGVLink(Corrupt Link/Object/2344)']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:17'!
testDuplicateCommunityException
	[
	DGVDuplicateCommunityException signal: 'Keezies Test Circus'.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'Een community met deze naam bestaat al: Keezies Test Circus']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:21'!
testEmptyDescriptionException
	[
	DGVEmptyDescriptionException signal.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'Omschrijving van een community mag niet leeg zijn.']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:24'!
testEmptyNameException
	[
	DGVEmptyNameException signal.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'De naam van een community mag niet leeg zijn.']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 10/31/2005 18:12'!
testException
	[	DGVException signal.
	 	self fail] 
		on: DGVException
		do: [:ex | self assert: ex messageText isNil.]! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:07'!
testExceptionMessage
	[
	DGVException signal: 'aap'.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 'Algemene fout: aap']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:12'!
testExceptionMessageEmpty
	[
	DGVException signal.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 'Algemene fout: Niet gespecificeerd.']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:27'!
testFileLockedException
	[
	DGVFileLockedException signal: 'mies'.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'Dit bestand kan niet worden geopend omdat het in gebruik is: mies']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:31'!
testInvalidEmailAddressException
	[
	DGVInvalidEmailAddressException signal: 'aap@noot.mies'.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'Dit is geen geldig emailadres: aap@noot.mies']! !

!DGVExceptionTest methodsFor: 'as yet unclassified' stamp: 'EG 12/14/2005 14:32'!
testServiceException
	[
	DGVServiceException signal: 'laundry service'.
	self fail
	] 
		on: DGVException
		do: [:ex | self assert: ex description = 
			'Deze service is tijdelijk niet op het netwerk beschikbaar: laundry service']! !

DGVTestCase subclass: #DGVFileTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVFileTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:47'!
testAddBlob
	| fileName bytes file |
	fileName _ 'wxsqueak.ini'.
	file _ (DGVFile new originalFilename: fileName).
	file addBlobFrom: fileName announce: false.

	bytes _ (FileStream readOnlyFileNamed: fileName) binary; contentsOfEntireFile.

	self assert: file blob data = bytes.
	self assert: file blob parent = file
	
! !

!DGVFileTest methodsFor: 'as yet unclassified' stamp: 'EG,JJ 10/25/2005 14:32'!
testAddDC
	| newFile user |
	user _ DGVUser testUser.
	newFile _ DGVFile create: 'something.txt' mimeType: 'text/plain' user: user.
	self assert: newFile metadata publisher = user.
	self assert: newFile metadata format = 'text/plain'.

	! !

!DGVFileTest methodsFor: 'as yet unclassified' stamp: 'TJ 1/21/2005 10:27'!
testAsString
	| file |
	file _ DGVFile new.
	self assert: file id = file asString! !

!DGVFileTest methodsFor: 'as yet unclassified' stamp: 'EG,JJ 10/25/2005 14:32'!
testCreateMimeType
	| newFile user |
	user _ DGVUser testUser.
	newFile _ DGVFile create: 'something.txt' mimeType: 'text/plain' user: user.
	self assert: newFile id notNil.
	self assert: 'something.txt' = newFile originalFilename.
	self assert: newFile metadata publisher = user.
	self assert: newFile metadata format = 'text/plain'.
	self assert: newFile metadata identifier = newFile id printString.
	self assert: (newFile metadata date beginsWith: (Date today yyyymmdd)).! !

!DGVFileTest methodsFor: 'as yet unclassified' stamp: 'TJ+CdG 1/18/2005 11:55'!
testNew
	| newFile |
	newFile _ (DGVFile new originalFilename: 'something.txt').
	self assert: newFile id notNil! !

!DGVFileTest methodsFor: 'as yet unclassified' stamp: 'TJ 2/25/2005 13:16'!
testWithoutContents
	| file |
	file _ (DGVFile new originalFilename: 'something.txt').
	self should: [file osOpen] raise: Error
	! !

DGVTestCase subclass: #DGVFilteringNotifyingCollectionTest
	instanceVariableNames: 'presences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVFilteringNotifyingCollectionTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/27/2005 12:11'!
testSimple
	| coll flag filtered |
	coll _ DGVNotifyingCollection new.
	filtered _ DGVFilteringNotifyingCollection on: coll filterBlock: [:each | each isNumber].

	flag _ false.
	filtered when: #activeItemsChanged send: #value to: [flag _ true].
	
	coll add: 'foo'.
	self deny: flag.
	
	coll add: 42.
	self assert: flag.

	self assert: filtered activeItems size = 1.
	self assert: filtered activeItems first = 42! !

DGVTestCase subclass: #DGVFolderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVFolderTest methodsFor: 'as yet unclassified' stamp: 'CdG 10/7/2005 11:20'!
testAddFile
	| folder file triggered block |
	Bindings clamp: [
		#DGVTestingCollectionStuff binding: true.

		folder := DGVFolder new.
		folder save.
		self assert: folder files isEmpty.
	
		triggered := false.
		block := [triggered _ true].
		folder files when: #activeItemsChanged send: #value to: block.
	
		file := DGVFile new.
		folder addFile: file.
	
		self deny: folder dirty.
		self assert: folder files size = 1.
		self assert: folder files anyOne = file.
		self assert: triggered]! !

!DGVFolderTest methodsFor: 'as yet unclassified' stamp: 'CdG 9/30/2005 10:45'!
testAddFolder
	| folder subFolder |
	folder := DGVFolder new.
	folder save.
	self assert: folder subFolders isEmpty.

	subFolder := DGVFolder new.
	folder addSubFolder: subFolder.

	self deny: folder dirty.
	self assert: folder subFolders size = 1.
	self assert: folder subFolders anyOne = subFolder.

	self assert: folder subFolders anyOne parent = folder.! !

!DGVFolderTest methodsFor: 'as yet unclassified' stamp: 'CdG 9/30/2005 10:09'!
testCreate
	| folder |
	folder := DGVFolder new.
	folder save.
	self deny: folder dirty.
	folder folderName: 'Test'.
	self assert: folder dirty.

	self assert: folder folderName = 'Test'.

	self assert: folder files isEmpty.
	self assert: folder subFolders isEmpty.! !

!DGVFolderTest methodsFor: 'as yet unclassified' stamp: 'CdG 10/7/2005 11:18'!
testRemoveFile
	| folder file triggered block |
	Bindings clamp: [
		#DGVTestingCollectionStuff binding: true.

		folder := DGVFolder new.
		file := DGVFile new.
		folder addFile: file.
		folder save.
		self assert: folder files size = 1.
		triggered := false.
		block := [triggered _ true].
		folder files when: #activeItemsChanged send: #value to: block.
	
		folder removeFile: file.
	
		self deny: folder dirty.
		self assert: folder files isEmpty.
		self assert: triggered]! !

!DGVFolderTest methodsFor: 'as yet unclassified' stamp: 'CdG 9/30/2005 10:12'!
testRemoveFolder
	| folder subFolder |
	folder := DGVFolder new.
	subFolder := DGVFolder new.
	folder addSubFolder: subFolder.
	folder save.
	self assert: folder subFolders size = 1.

	folder removeSubFolder: subFolder.

	self deny: folder dirty.
	self assert: folder subFolders isEmpty.! !

DGVTestCase subclass: #DGVHarvestServiceTest
	instanceVariableNames: 'withBaseUrlCalled withBaseUrlCalledBaseUrl harvestCalled hasMoreRecordsCalled hasMoreRecordsReturn harvestError hasMoreRecordsError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVHarvestServiceTest methodsFor: 'self-shunt' stamp: 'EG,CdG 6/23/2005 08:35'!
harvest
	harvestCalled := true.
	harvestError ifNotNil: [OAIError signal: 'test error']! !

!DGVHarvestServiceTest methodsFor: 'self-shunt' stamp: 'EG,CdG 6/23/2005 08:49'!
hasMoreRecords
	hasMoreRecordsCalled := true.
	hasMoreRecordsError ifNotNil: [Error signal: 'test error'].
	^hasMoreRecordsReturn! !

!DGVHarvestServiceTest methodsFor: 'private' stamp: 'EG,CdG 6/23/2005 08:10'!
runFirstTaskOf: scheduler 
	self	runTask: (scheduler taskAt: 1)! !

!DGVHarvestServiceTest methodsFor: 'private' stamp: 'EG,CdG 6/23/2005 08:10'!
runTask: task 
	task task value.
	task wasRun! !

!DGVHarvestServiceTest methodsFor: 'tests' stamp: 'EG,CdG 6/23/2005 08:06'!
setUp
	super setUp.
	harvestCalled := false.
	hasMoreRecordsCalled := false.
	hasMoreRecordsReturn := true.
	withBaseUrlCalled := false.
	withBaseUrlCalledBaseUrl := ''! !

!DGVHarvestServiceTest methodsFor: 'tests' stamp: 'EG,CdG 6/23/2005 08:46'!
testCatchError
	| ensureBlockCalled |
	ensureBlockCalled := false.
	self shouldnt: [[Error signal: 'error'] ensure: [^ensureBlockCalled := true]] raise: Error.
	self assert: ensureBlockCalled! !

!DGVHarvestServiceTest methodsFor: 'tests' stamp: 'EG,CdG 6/23/2005 17:42'!
testCreateHarvestService
	| service scheduler task |
	[service := DGVOAIHarvestService new.
	service oaiClientFactory: self.
	service repositories: #('http://blah1' ).
	service start.
	scheduler := service scheduler.
	self assert: scheduler isRunning.
	scheduler stop.
	self runFirstTaskOf: scheduler.
	self assert: withBaseUrlCalled.
	self assert: withBaseUrlCalledBaseUrl = 'http://blah1'.
	self assert: harvestCalled.
	self assert: hasMoreRecordsCalled.
	self assert: scheduler taskCount = 1.
	task := scheduler taskAt: 1.
	self assert: task nextRunTime > (TimeStamp now + 14 minutes).
	self assert: task nextRunTime < (TimeStamp now + 16 minutes).
	hasMoreRecordsReturn := false.
	self runTask: task.
	task := scheduler taskAt: 1.
	self assert: task nextRunTime > (TimeStamp now + 1 day - 1 minute).
	self assert: task nextRunTime < (TimeStamp now + 1 day + 1 minutes)]
		ensure: [service stop]! !

!DGVHarvestServiceTest methodsFor: 'tests' stamp: 'EG,CdG 6/23/2005 17:48'!
testHarvestErrorResistancy
	| service scheduler |
	[service := DGVOAIHarvestService new.
	service oaiClientFactory: self.
	service repositories: #('http://blah1' ).
	harvestError := true.
	hasMoreRecordsError = true.
	service start.
	scheduler := service scheduler.
	scheduler stop.
	self assert: (scheduler taskAt: 1) nextRunTime < (TimeStamp now + 1 second).
	self shouldnt: [self runFirstTaskOf: scheduler] raise: Error.
	self assert: scheduler taskCount = 1.
	self assert: (scheduler taskAt: 1) nextRunTime > (TimeStamp now + 14 minutes)]
		ensure: [service stop]! !

!DGVHarvestServiceTest methodsFor: 'self-shunt' stamp: 'EG,CdG 6/23/2005 17:39'!
withBaseUrl: aByteString 
	withBaseUrlCalled := true.
	withBaseUrlCalledBaseUrl := aByteString! !

DGVTestCase subclass: #DGVHoarderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVHoarderTest methodsFor: 'as yet unclassified' stamp: 'CdG 12/14/2005 20:02'!
testDoNotCrashOnGarbage
	| hoarder obj pe |
	hoarder _ DGVHoarderService new.

	hoarder fetchQueue nextPut: 'some garbage'.
	self shouldnt: [hoarder fetchRun] raise: Error.

	hoarder fetchQueue nextPut: (DGVLink for: DGVMockObject new).
	self shouldnt: [hoarder fetchRun] raise: Error.

	hoarder fetchQueue nextPut: AWGPresenceEntry new.
	self shouldnt: [hoarder fetchRun] raise: Error.

	obj _ DGVMockObject new save. "object is already here"
	pe _ AWGPresenceEntry new.
	pe object: (DGVLink for: obj).
	hoarder fetchQueue nextPut: pe.
	self shouldnt: [hoarder fetchRun] raise: Error.
! !

DGVTestCase subclass: #DGVHTMLGenerationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVHTMLGenerationTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/28/2005 11:25'!
testIsUrl
	self deny: 'aap' isUrl.
	self assert: 'http://aap' isUrl.
	self assert: 'ftp://aap.noot.mies' isUrl.
	self assert: 'rtsp://audio.server' isUrl.
	self assert: 'https://some.secure.server' isUrl! !

DGVTestCase subclass: #DGVLinkTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVLinkTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/10/2005 12:09'!
testProtoObjectEqualsLinkObject
	| proto uuid class link |
	proto _ DGVMockObject new.
	uuid _ proto id.
	class _ proto class.
	link _ DGVLink for: proto.

	self assert: (link equalsObjectWithClass: class andUuid: uuid).
	self assert: (proto equalsObjectWithClass: class andUuid: uuid).! !

!DGVLinkTest methodsFor: 'as yet unclassified' stamp: 'EG 11/23/2005 20:36'!
testUnmarshalDGVLinkWithoutClassBeingPresent
	| bytearray object |
	self withMockSubclassOf: Object instanceVariables: '' methods: #() classMethods: #() do: [:cls|
		object _ cls new.
		bytearray := SrpConfiguration default saveObjectAsBinary: object.
	].
	Smalltalk garbageCollect.
	self should: [SrpConfiguration default loadObjectFromBinary: bytearray] raise: Error.	

	self withMockSubclassOf: DGVProtoObject instanceVariables: '' methods: #() classMethods: #() do: [:cls|
		object _ DGVLink class: cls id: UUID new.
		bytearray := SrpConfiguration default saveObjectAsBinary: object.
	].
	Smalltalk garbageCollect.
	self should: [SrpConfiguration default loadObjectFromBinary: bytearray] raise: Error.
! !

!DGVLinkTest methodsFor: 'as yet unclassified' stamp: 'EG 11/23/2005 20:02'!
testUnmarshalDGVLinkWithoutClassBeingPresentUsingReferenceStream
	| link stream |
	stream := ReferenceStream
				on: (RWBinaryOrTextStream
						on: (ByteArray new: 1000)).
	self
		withMockSubclassOf: DGVProtoObject
		instanceVariables: ''
		methods: #()
		classMethods: #()
		do: [:cls | 
			link := DGVLink class: cls id: UUID new.
			stream nextPut: link].
	stream reset.
	Smalltalk garbageCollect.
	self should: [stream next] raise: Error.! !

DGVTestCase subclass: #DGVNewUserPolicyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVNewUserPolicyTest methodsFor: 'as yet unclassified' stamp: 'CdG 10/19/2005 12:58'!
testAddUserAlwaysAccept
	| policy community answer user |
	policy _ DGVNewUserPolicyAlwaysAccept new.
	community _ DGVCommunity new.
	community newUserPolicy: policy.
	community save.
	user _ DGVUser testUser.
	answer _ community addUser: user.
	self assert: answer.
	self assert: (community users includes: user)! !

!DGVNewUserPolicyTest methodsFor: 'as yet unclassified' stamp: 'CdG 4/26/2005 09:41'!
testAddUserConfirmEmail
	| policy community answer user goodSig smtp message |
	Bindings clamp:	[#SMTPClient binding: (smtp _ DGVMockSMTPClient new).
		
	policy _ DGVPolicyConfirmUser new.
	policy save.
	community _ DGVCommunity new.
	community newUserPolicy: policy.
	community save.
	user _ DGVUser testUser.
	user email: 'mock@example.com'.
	user save.
	answer _ policy acceptNewUser: user id in: community id	.
	self assert: answer isNil.
	self assert: smtp sentMailCount = 1.
	message _ smtp sentMails first.
	self assert: message to = 'mock@example.com'.
	
	self deny: (policy receivedConfirmationFor: 'garbage' community: 'garbage' signature: 'more garbage').
	self deny: (policy receivedConfirmationFor: user id community: 'garbage' signature: 'garbage').
	goodSig _ policy signWith: user id with: community id.
	self deny: (policy receivedConfirmationFor: 'garbage' community: community id signature: goodSig).
	self assert: (policy receivedConfirmationFor: user id community: community id signature: goodSig).
	
	community _ DGVCommunity get: community id.
	self assert: (community users includes: user).

	"operation is idempotent so can be repeated"
	self assert: (policy receivedConfirmationFor: user id community: community id signature: goodSig)]! !

!DGVNewUserPolicyTest methodsFor: 'as yet unclassified' stamp: 'CdG 4/26/2005 09:41'!
testAddUserOwnerConfirms
	| policy community answer user goodSig smtp message owner |
	Bindings clamp:	[#SMTPClient binding: (smtp _ DGVMockSMTPClient new).
		
	policy _ DGVPolicyOwnerConfirm new.
	policy save.
	owner _ DGVUser testUser.
	owner
		fullName: 'Test Owner';
		email: 'owner@example.com';
		save.
	community _ DGVCommunity new.
	community 
		owner: owner;
		newUserPolicy: policy;
		save.
	user _ DGVUser testUser.
	user 
		fullName: 'Test User';
		email: 'user@example.com';
		save.
	answer _ policy acceptNewUser: user id in: community id.
	self assert: answer isNil.
	self assert: smtp sentMailCount = 1.
	message _ smtp sentMails first.
	self assert: message to = 'owner@example.com'.
	
	self deny: (policy receivedConfirmationFor: 'garbage' community: 'garbage' signature: 'more garbage').
	self deny: (policy receivedConfirmationFor: user id community: 'garbage' signature: 'garbage').
	goodSig _ policy signWith: user id with: community id.
	self deny: (policy receivedConfirmationFor: 'garbage' community: community id signature: goodSig).
	self assert: (policy receivedConfirmationFor: user id community: community id signature: goodSig).
	self assert: smtp sentMailCount = 2.
	message _ smtp sentMails second.
	self assert: message to = 'user@example.com'.
	
	community _ DGVCommunity get: community id.
	self assert: (community users includes: user).

	"operation is idempotent so can be repeated"
	self assert: (policy receivedConfirmationFor: user id community: community id signature: goodSig)]! !

DGVTestCase subclass: #DGVNodeStatisticsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVNodeStatisticsTest methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:56'!
testGet
	| stats |
	stats := DGVNodeStatistics forDefaultNode.
	self assert: stats notNil.
	self assert: stats totalBlobDownloads = 0.

	stats incrementTotalBlobDownloads; save.
	stats := DGVNodeStatistics forDefaultNode.
	self assert: stats totalBlobDownloads = 1.
! !

DGVTestCase subclass: #DGVNotifyingCollectionTest
	instanceVariableNames: 'presences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVNotifyingCollectionTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/7/2005 10:48'!
testAdd
	| coll flag1 flag2 |
	coll _ DGVNotifyingCollection new.
	flag1 _ flag2 _ false.

	coll when: #activeItemsChanged send: #value to: [flag1 _ true].
	coll when: #itemAdded send: #value to: [flag2 _ true].
	
	self assert: (coll add: 42) = 42.
	self assert: flag1.
	self assert: flag2.

	self assert: coll activeItems size = 1.
	self assert: coll activeItems first = 42! !

!DGVNotifyingCollectionTest methodsFor: 'as yet unclassified' stamp: 'JJ 12/2/2005 10:52'!
testAsSortedCollection
	| coll flag1 |


	flag1 := false.
	coll _ DGVNotifyingCollection new.
	coll when: #activeItemsChanged send: #value to: [flag1 := true].
	coll 
		add: 2;
		add: 4;
		add: 10;
		add: 1.

	coll sortWith: [:a :b | a < b].


	
	self assert: flag1.
	self assert: coll size = 4.
! !

!DGVNotifyingCollectionTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/27/2005 14:03'!
testProtocol
	| list coll |
	list _ DGVNotifyingCollection new.
	list add: 42.
	self assert: list asOrderedCollection class == OrderedCollection.
	self assert: list asOrderedCollection first = 42.

	coll _ list asOrderedCollection.
	coll add: 43.

	self deny: list size = 2.! !

!DGVNotifyingCollectionTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/7/2005 10:48'!
testRemove
	| coll flag1 flag2 |
	coll _ DGVNotifyingCollection new.
	flag1 _ flag2 _ false.
	
	coll add: 42.

	coll when: #activeItemsChanged send: #value to: [flag1 _ true].
	coll when: #itemRemoved send: #value to: [flag2 _ true].

	coll remove: 42.
	self assert: flag1.
	self assert: flag2.
	
	self assert: coll activeItems size = 0.! !

DGVTestCase subclass: #DGVOAICollectionTest
	instanceVariableNames: 'nodes repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVOAICollectionTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/11/2005 20:46'!
livetestCreateOAICollection
	| coll |
	coll := DGVOAICollection 
				with: (OAIClient withBaseUrl: 'http://dare.ubvu.vu.nl/oai/request').
	self assert: coll collectionName = 'DSpace at VU'! !

!DGVOAICollectionTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/29/2005 10:48'!
livetestHarvestBatch
	| coll baseUrl id hash |
	baseUrl := 'http://dare.ubvu.vu.nl/oai/request'.
	coll := DGVOAICollection
				with: (OAIClient withBaseUrl: baseUrl).
	coll save.
	"self assert: DGVOAIRecord localStorage items isEmpty."
	coll harvest.
	self deny: DGVOAIRecord localStorage items isEmpty.
	hash := SecureHashAlgorithm new hashMessage: baseUrl , 'oai:dare.ubvu.vu.nl:1871/1315'.
	id := UUID
				newFrom: (hash asByteArray copyFrom: 1 to: 16).
	self assert: (DGVOAIRecord localStorage get: id) notNil! !

!DGVOAICollectionTest methodsFor: 'as yet unclassified' stamp: 'EG 6/20/2005 16:29'!
testCalculateIdFor
	| string1 string2 uuid1 uuid2 |
	string1 _ 'http://dare.ubvu.vu.nl/oai/request', 'oai:DiVA.org:ntnu-1'.
	string2 _ 'http://dare.ubvu.vu.nl/oai/request', 'oai:DiVA.org:ntnu-2'.
	uuid1 _ UUID fromSHAHash: (SecureHashAlgorithm new hashMessage: string1).
	uuid2 _ UUID fromSHAHash: (SecureHashAlgorithm new hashMessage: string2).
	self deny: uuid1 = uuid2.
	self assert: uuid1 printString = '57cbc532-5014-47a4-80c5-573fb7a09087'! !

!DGVOAICollectionTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:53'!
testHarvestBatch
	| coll baseUrl id hash oaiClient |
	baseUrl := 'http://dare.ubvu.vu.nl/oai/request'.
	oaiClient := OAIPluggableTextClient withBaseUrl: baseUrl.
	oaiClient text: OAITest new listRecordsInitialResponse.
	coll := DGVOAICollection with: oaiClient.
	self assert: DGVOAIRecord localStorage items isEmpty.
	coll harvest.
	self assert: DGVOAIRecord localStorage items size = 2.
	self assert: DGVOAIRecord localStorage items first parent = coll.

	hash := SecureHashAlgorithm new 
				hashMessage: baseUrl , 'oai:DiVA.org:ntnu-1'.
	id := UUID fromSHAHash: hash.
	self assert: (DGVOAIRecord localStorage get: id) notNil! !

!DGVOAICollectionTest methodsFor: 'as yet unclassified' stamp: 'EG 6/21/2005 14:14'!
testHarvestIncrement
	| coll baseUrl id hash oaiClient |
	baseUrl := 'http://dare.ubvu.vu.nl/oai/request'.
	oaiClient := OAIPluggableTextClient withBaseUrl: baseUrl.
	oaiClient text: OAITest new listRecordsInitialResponse.
	coll := DGVOAICollection with: oaiClient.
	self assert: DGVOAIRecord localStorage items isEmpty.
	coll harvest.
	self assert: DGVOAIRecord localStorage items size = 2.
	hash := SecureHashAlgorithm new 
				hashMessage: baseUrl , 'oai:DiVA.org:ntnu-1'.
	id := UUID fromSHAHash: hash.
	self assert: (DGVOAIRecord localStorage get: id) notNil! !

DGVTestCase subclass: #DGVOAIRecordTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVOAIRecordTest methodsFor: 'as yet unclassified' stamp: 'EG,JJ 10/25/2005 14:34'!
testCreateOAIRecord
	| oairecord record |
	oairecord _ OAITest new getMockRecord.
	record _ DGVOAIRecord from: oairecord.
	record save.

	record _ DGVOAIRecord localStorage get: record id.
	self assert: record notNil.

	self assert: record metadata title = 'Static Histomorphometry and Bone Strength of the Human Spine and Iliac Crest : relation to Age and Sex'.

	self assert: record metadata identifier = 'http://urn.kb.se/resolve?urn=urn:nbn:dk:au:diva-1'! !

DGVTestCase subclass: #DGVObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG,TJ 2/15/2005 14:03'!
testBasicSecurity
	| o u |
	o _ DGVMockObject new.
	u _ DGVUser testUser.
	o signBy: u.
	self assert: (o signer = u).
	self assert: (o signatureIsValid).
	o delete.
	self deny: (o signatureIsValid)! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'TJ 3/7/2005 13:19'!
testDelete
	| o |
	o _ DGVMockObject new save.
	self assert: (DGVMockObject get: o id) notNil.
	o delete.
	self assert: (DGVMockObject get: o id) deleted.

! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG,EjG 6/8/2005 16:07'!
testEquality
	| obj1 obj2 |
	obj1 _ DGVMockObject new.
	obj2 _ DGVLink for: obj1.
	self assert: obj1 = obj2.
	self assert: obj2 = obj1.

	self deny: obj1 = 42! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/29/2005 14:41'!
testGetCollectAtWithException
	| id link object coll |
	"Test that when a link to a collection is not resolvable, an empty new collection
	 with the correct UUID is created on-the-fly"
	
	object _ DGVMockObject new.
	id _ object id asString, '.collection'.
	link _ DGVLink class: DGVCollection id: id.
	object collection: link.
	
	coll _ object getCollectionAt: #collection.
	
	self assert: coll notNil.
	self assert: coll class = DGVCollection.
	self assert: coll id = id
	! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 19:59'!
testIsNewer
	| o1 o2 |
	o1 := DGVObject new.
	o2 := o1 copy.

	o1 version: 4.
	o2 version: 5.

	self assert: (o2 isNewerThan: o1).
	self deny: (o1 isNewerThan: o2).
	self deny: (o2 isNewerThan: o2).

	o1 version: 5.1.

	self assert: (o1 isNewerThan: o2).
	self deny: (o2 isNewerThan: o1).

	self deny: (o1 isNewerThan: nil).
	self deny: (o1 isNewerThan: 'aap')
! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/28/2005 14:55'!
testMarshallAsArrayElement
	| obj1  result array obj2 |
	obj1 _ DGVMockObject new.
	obj2 _ DGVMockObject new.
	obj1 link: obj2.
	array _ Array with: obj1.

	result _ Object fromByteArray: array marshallAsByteArray.

	self assert: result = array.
	self deny: result first isDGVLink.
	self assert: result first = obj1.
	self assert: result first link isDGVLink.! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/28/2005 14:56'!
testMarshallWithArray
	| obj1  result array obj2 |
	obj1 _ DGVMockObject new.
	obj2 _ DGVMockObject new.
	array _ Array with: obj2.
	obj1 collection: array.

	result _ Object fromByteArray: obj1 marshallAsByteArray.

	self assert: result = obj1.
	self assert: result collection first isDGVLink.
! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/28/2005 14:53'!
testMarshallWithLink
	| obj1 obj2 result |
	obj1 _ DGVMockObject new.
	obj2 _ DGVMockObject new.
	obj1 link: obj2.

	result _ Object fromByteArray: obj1 marshallAsByteArray.

	self assert: result = obj1.
	self assert: result link isDGVLink! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 19:52'!
testNew
	| o |
	o _ DGVMockObject new.
	self assert: o id isUUID.
	o save.
	self assert: o version floor = 1.
	o dirty: true; save.
	self assert: o version floor = 2! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/13/2005 11:59'!
testSaveAndUpdateSavedTime
	| obj longago |
	obj _ DGVMockObject new.
	longago _ TimeStamp fromString: '1-1-1999'.
	obj lastSavedTime: longago.
	self assert: obj lastSavedTime = longago.
	obj save.
	self assert: obj lastSavedTime > longago
! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 15:19'!
testSaveCollectionWithLinks
	| parent child |
	parent _ DGVMockObject new.
	child _ DGVCollection parent: parent named: 'whatever'.
	parent link: child.
	child add: DGVMockObject new.
	
	parent _ DGVObject fromByteArray: parent marshallAsByteArray.
	
	self assert: (parent link isDGVLink).
	self assert: (parent link myclass =  DGVCollection).
	self assert: (parent link id = child id)! !

!DGVObjectTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/1/2005 15:22'!
testSaveWithLinks
	| parent child |
	parent _ DGVMockObject new save.
	child _ DGVMockObject new save.
	parent link: child.
	
	parent _ DGVMockObject fromByteArray: parent marshallAsByteArray.
	
	self assert: (parent link isDGVLink).
	self assert: (parent link myclass =  DGVMockObject).
	self assert: (parent link id = child id)! !

DGVTestCase subclass: #DGVOSSupportTest
	instanceVariableNames: 'dataDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVOSSupportTest methodsFor: 'as yet unclassified' stamp: 'CdG 11/1/2005 11:36'!
testCreateWindows98
	| osSupportClass osSupport os |
	osSupportClass _ DGVOSSupport activeOSSupportClass.
	osSupport _ osSupportClass new.
	os _ SmalltalkImage current osVersion.
	os = '95' ifTrue: [
		self assert: DGVWin98Support isActiveOSSupportClass.
		self deny: DGVWin32Support isActiveOSSupportClass.
		self assert: (osSupportClass = DGVWin98Support).
		self assert: osSupport homeDirectory pathName = 'C:\Local Settings'.
		self assert: osSupport tempDirectory pathName = 'c:\windows\TEMP'
	] ifFalse: [ os = 'NT' ifTrue: [
		self deny: DGVWin98Support isActiveOSSupportClass.
		self assert: DGVWin32Support isActiveOSSupportClass.
		self assert: osSupportClass = DGVWin32Support.
		self assert: (osSupport homeDirectory pathName beginsWith: 'C:\Documents and Settings\').
		self assert: (osSupport dataDirectory pathName endsWith: '\Application Data\Kolibri')
	] ifFalse: [ 
		self assert: os = 'linux-gnu'.
		self assert: DGVUnixSupport isActiveOSSupportClass.
		self assert: (osSupport homeDirectory pathName beginsWith: '/home/').
		self assert: (osSupport dataDirectory pathName endsWith: '/.kolibri')
	]]! !

DGVTestCase subclass: #DGVPolicyMailServiceRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVPolicyMailServiceRequestTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/24/2005 13:56'!
testAsFromByteArray
	| req byteArray req2 |
	req _ (DGVPolicyMailServiceRequest new 
		communityId: UUID new; 
		userId: UUID new;
		replyEndPoint: 16r83).
	byteArray _ req marshallAsByteArray.
	req2 _ (DGVPolicyMailServiceRequest fromByteArray: byteArray).
	self assert: req2 class = DGVPolicyMailServiceRequest.
	self assert: req2 replyEndPoint = 16r83.! !

DGVTestCase subclass: #DGVPresenceTest
	instanceVariableNames: 'set'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVPresenceTest methodsFor: 'as yet unclassified' stamp: 'CdG 12/15/2005 23:56'!
setUp
	super setUp.
	set _ nil! !

!DGVPresenceTest methodsFor: 'as yet unclassified' stamp: 'CdG 12/15/2005 20:17'!
tearDown
	set ifNotNil: [set stop]! !

!DGVPresenceTest methodsFor: 'as yet unclassified' stamp: 'CdG 12/15/2005 20:17'!
testAddPresenceEntry
	| obj |
	set _ AWGPresenceSet new.
	obj := DGVMockObject new save.
	set addPresenceOf: (DGVLink for: obj).
	self assert: (set entries someElement object version floor = 1).

	obj markDirty; save.
	self assert: obj version floor = 2.
	set addPresenceOf: (DGVLink for: obj).

	self assert: (set entries someElement object version floor = 2)! !

DGVTestCase subclass: #DGVProtoObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVProtoObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:20'!
testCreationTime
	| obj1 pre post |
	pre := DGVNode default networkTimeStamp.
	obj1 _ DGVMockObject new.
	post := DGVNode default networkTimeStamp.

	self assert: obj1 creationTime notNil.
	self assert: obj1 creationTime >= pre.
	self assert: obj1 creationTime <= post.

	obj1 instVarNamed: #creationTime put: nil.
	self assert: obj1 creationTime = AWGTimeStamp epoch.! !

!DGVProtoObjectTest methodsFor: 'as yet unclassified' stamp: 'CdG 8/11/2005 12:21'!
testGetPathToRootCommunity
	| com1 com2 obj path |
	com1 := DGVCommunity new.
	com2 := com1 
		createSubcommunityNamed: 'TestNewCom'
		description: 'A new testcom'
		owner: DGVUser testUser
		policyname: DGVNewUserPolicyAlwaysAccept policyName.
	
	obj := DGVMockObject new.
	com1 addFile: obj.
	path := obj getPathToRootCommunity.
	self assert: path size = 1.
	self assert: path first = com1.

	obj := DGVMockObject new.
	com2 addFile: obj.
	path := obj getPathToRootCommunity.
	self assert: path size = 2.
	self assert: path first = com1.
	self assert: path second = com2
	! !

!DGVProtoObjectTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 6/13/2005 11:52'!
testRefreshIfNecessary
	| obj1 obj2 |
	obj1 _ DGVMockObject new.
	obj2 _ DGVProtoObject refreshIfNecessary: obj1.
	self deny: obj1 needsRefresh.
	self assert: obj2 == obj1.

	self deny: obj1 needsRefresh.
	obj1 lastSavedTime: (DGVNode default networkTimeStamp - DGVProtoObject refreshTime - 1 second).
	self assert: obj1 needsRefresh.! !

DGVTestCase subclass: #DGVRelationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:59'!
testAdd
	| relation object |
	relation _ DGVRelation new.
	
	object _ DGVObject new.
	
	relation add: object.
	
	self assert: (relation objects includes: object).
	self assert: (object relations includes: relation).
	self assert: (relation parent = object)! !

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 15:07'!
testAddAll
	| relation object1 object2 array |
	relation _ DGVRelation new.
	
	object1 _ DGVObject new.
	object2 _ DGVObject new.
	array _ {object1. object2}.
	
	relation addAll: array.
	
	self assert: (relation objects includes: object1).
	self assert: (relation objects includes: object2).

	self assert: (object1 relations includes: relation).
	self assert: (object2 relations includes: relation)
		! !

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 15:09'!
testAddDuplicates
	| relation object |
	relation _ DGVRelation new.
	
	object _ DGVObject new.
	
	relation add: object.
	relation add: object.
	
	self assert: (relation objects includes: object).
	self assert: (object relations includes: relation).
	
	self assert: relation objects size = 1.
	self assert: object relations size = 1.! !

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 15:06'!
testInitialize
	| relation object |
	relation _ DGVRelation new.
	
	self assert: (relation instVarNamed: #objects) notNil. "initialize-time initialization!!"
	self assert: (relation instVarNamed: #id) notNil. "super initialize done?"
	
	object _ DGVObject new.

	self assert: (object instVarNamed: #relations) notNil. "initialize-time initialization!!"
	
! !

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/11/2005 20:28'!
testIsRelatedTo
	| relation object1 object2 array comm |
	relation _ DGVRelation new.
	
	object1 _ DGVObject new.
	object2 _ DGVObject new.
	array _ {object1. object2}.
	
	relation addAll: array.
	
	self assert: (object1 isRelatedTo: object2).
	self assert: (object2 isRelatedTo: object1).

	comm _ DGVCommunity new: 'foo'.

	self deny: (comm isRelatedTo: object1).
	self deny: (comm isRelatedTo: object2).
	self deny: (object1 isRelatedTo: comm).
	self deny: (object2 isRelatedTo: comm)! !

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/27/2005 11:58'!
testParentBarfs
	| relation |
	relation _ DGVRelation new.
	self should: [relation parent: DGVObject new] raise: Error
! !

!DGVRelationTest methodsFor: 'as yet unclassified' stamp: 'CdG 5/2/2005 15:10'!
testRemove
	| relation object |
	relation _ DGVRelation new.
	object _ DGVObject new.
	relation add: object.
	
	relation remove: object.
	
	self assert: relation objects isEmpty.
	self assert: object relations isEmpty.! !

DGVTestCase subclass: #DGVServiceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVServiceTest methodsFor: 'as yet unclassified' stamp: 'CdG 5/3/2005 17:01'!
testCreateCollectionService
	| actions |
	actions _ DGVNode default actionSequenceForEvent: #entryAdded.
	self assert: actions size = 0.	
	DGVCollectionService new start.
	actions _ DGVNode default actionSequenceForEvent: #entryAdded.
	self assert: actions size = 1! !

!DGVServiceTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 14:22'!
testEntryAdded
	"Tests the addition of a CollectionElement received from the network to an existing collection"
	| service entry collection object |
	service _ DGVCollectionService new start.
	self assert: (service isKindOf: DGVCollectionService).
	collection _ (DGVCollection newWithId: 'test') save.
	object _ DGVMockObject new save; yourself.
	entry _ AWGPresenceEntry new.
	entry object: (DGVCollectionElement 
		collId: collection id 
		elementId: object id 
		elementClass: object class).
	self assert: collection items isEmpty.
	self deny: collection dirty.
	service entryAdded: entry.
	collection _ DGVCollection get: collection id.
	self assert: collection items isEmpty not.
	self assert: (collection includes: object).
	self deny: collection dirty "automagically saved"
! !

!DGVServiceTest methodsFor: 'as yet unclassified' stamp: 'CdG 5/3/2005 17:02'!
testEntryAddedToNonLocalCollection
	"Test adding an element to a collection that is not stored locally (not in
	memory, not on disk, not in the cache). This should result in the creation 
	of a new collection with the element."
	| service nonLocalCollectionId object entry |
	service _ DGVCollectionService new start.
	nonLocalCollectionId _ UUID new.
	object _ DGVMockObject new.
	entry _ AWGPresenceEntry new.
	entry object: (DGVCollectionElement 
		collId: nonLocalCollectionId
		elementId: object id 
		elementClass: object class).
	service entryAdded: entry.
	"service searches locally for the DGVCollection"
	self assert: (DGVCollection localStorage get: nonLocalCollectionId) notNil! !

!DGVServiceTest methodsFor: 'as yet unclassified' stamp: 'TJ 5/4/2005 16:08'!
testEntryAddedWithRemovalFlag
	"Tests the addition of a CollectionElement received from the network to an excisting collection"
	| service entry collection object |
	service _ DGVCollectionService new start.
	self assert: (service isKindOf: DGVCollectionService).
	collection _ (DGVCollection newWithId: 'test') save.
	object _ DGVMockObject new.
	collection add: object.
	collection save.
	entry _ AWGPresenceEntry new.
	entry object: (DGVCollectionElement 
		removeFromCollId: collection id 
		elementId: object id 
		elementClass: object class).
	self assert: collection size = 1.
	self deny: collection dirty.
	service entryAdded: entry.
	collection _ DGVCollection get: collection id.
	self assert: collection isEmpty.
	self deny: collection dirty "automagically saved"
! !

DGVTestCase subclass: #DGVStorageTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'CdG 11/1/2005 18:27'!
testDeadlockBug
	"a couple of times it has happened that Kolibri hit on a locked file. Analysis indicates that
	this normally cannot happen, maybe file handles are not free'd immediately. In any case,
	we have added lock-detection code that we test here."

	| object storage filename stream |
	object := DGVMockObject new.
	storage := DGVStorage new openOrCreateOn: 'test'.
	storage put: object.
	filename := storage storageDirectory fullNameFor: object id asString.
	stream := FileStream forceNewFileNamed: filename.
	[
		[self should: [storage put: object] raise: DGVFileLockedException]
			on: Error do: [:ex | self signalFailure: ex printString]
	] ensure: [stream close]! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 10:00'!
testDo
	| count |
	DGVMockObject new save.
	DGVMockObject new save.

	count := 0.
	DGVMockObject localStorage do: [:each | count := count + 1].
	self assert: count = 2.
! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/24/2005 11:55'!
testExists
	| o s |
	o _ DGVMockObject new.
	s _ DGVStorage named: 'test'.
	s put: o.
	self assert: (s exists: o id).
	self deny: (s exists: UUID new) 
! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'EG 10/31/2005 15:00'!
testFilterOldObjects
	| object storage |
	object _ DGVMockObject new.
	storage _ DGVStorage named: 'test'.
	storage put: object.
	self assert: (storage activeItems includes: object).

	object creationTime: AWGTimeStamp epoch.
	object save.
	
	self deny: (storage activeItems includes: object)
! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/7/2005 10:16'!
testGarbage
	| count storage |
	storage := DGVStorage new openOrCreateOn: DGVMockObject storageName.	
	
	storage put: DGVMockObject new.
	storage at: UUID new put: #(0 2 3 4) asByteArray. "garbage"

	count := 0.
	storage do: [:each | count := count + 1].
	self assert: count = 1
! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/24/2005 11:56'!
testKeys
	| keys |
	keys _ (DGVStorage named: 'test') 
		put: DGVMockObject new;
		put: DGVMockObject new;
		keys.
	self assert: keys size >= 2.
	self assert: (keys reject: [:each | each isKindOf: UUID]) isEmpty! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/24/2005 11:55'!
testPutGet
	| o s o2 |
	o _ DGVMockObject new.
	s _ DGVStorage named: 'test'.
	s put: o.
	o2 _ s get: o id.
	self assert: o2 notNil.
	self assert: o2 id = o id
! !

!DGVStorageTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/11/2005 15:12'!
testQualifiedReference
	| obj1 reference parts obj2 |
	obj1 := DGVMockObject new.
	obj1 save.
	reference := obj1 qualifiedReference.
	parts := reference subStrings: '/'.
	self assert: parts first = obj1 storageName.
	self assert: parts last = obj1 id printString.
	obj2 := DGVStorage getFromQualifiedReference: reference.
	self assert: obj2 notNil.
	self assert: obj1 = obj2! !

DGVTestCase subclass: #DGVStringRepresentationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVStringRepresentationTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/11/2005 20:16'!
testCommunity
	| comm |
	comm _ DGVCommunity new: 'test community'.
	self assert: comm stringRepresentation = 'Community "test community"'! !

!DGVStringRepresentationTest methodsFor: 'as yet unclassified' stamp: 'JJ 12/8/2005 20:11'!
testFile
	| file |
	file _ DGVFile create: 'foo.txt' mimeType: 'text/plain' user: DGVUser testUser.
	file metadata title: 'this is a test'.
	self assert: file stringRepresentation = 'File "this is a test" (foo.txt)' translated! !

!DGVStringRepresentationTest methodsFor: 'as yet unclassified' stamp: 'EG,JJ 10/25/2005 14:35'!
testRecord
	| record |
	record _ DGVOAIRecord new.
	record metadata title: 'this is a test'.
	self assert: record stringRepresentation = 'Record "this is a test"'! !

!DGVStringRepresentationTest methodsFor: 'as yet unclassified' stamp: 'CdG 7/11/2005 20:12'!
testUser
	| user |
	user _ DGVUser testUser.
	user fullName: 'Test U. Ser'.
	self assert: user stringRepresentation = 'Gebruiker "myname" (Test U. Ser)'! !

!DGVTestCase class methodsFor: 'as yet unclassified' stamp: 'CdG 6/14/2005 10:21'!
setNormalMode
	| current |
	current _ DGVNode getInstance.
	(current isKindOf: DGVNode) ifFalse:[
		DGVNode setInstance: TheOldInstance.
		TheOldInstance _ nil]
! !

!DGVTestCase class methodsFor: 'as yet unclassified' stamp: 'CdG 12/15/2005 20:21'!
setTestMode
	| current |
	current _ DGVNode getInstance.
	(current isNil or: [current isKindOf: DGVNode]) ifTrue:[
		TheOldInstance _ current.
		DGVNode setInstance: (DGVMockNode new).
		]
! !

!DGVTestCase methodsFor: 'as yet unclassified' stamp: 'CdG 12/14/2005 15:17'!
configurationClass
	"use indirection because otherwise we'll have load order issues"
	^Smalltalk at: #DivaApplicationConfiguration! !

!DGVTestCase methodsFor: 'as yet unclassified' stamp: 'CdG 11/15/2005 13:04'!
makeMockSubclassOf: class instanceVariables: ivars methods: methods classMethods: classMethods
	| answer |
	answer := ClassBuilder new
		newSubclassOf: class
		type: #normal
		instanceVariables: ivars
		from: nil.
	methods do: [:each |
		answer compile: each].
	classMethods do: [:each |
		answer class compile: each].
	^answer! !

!DGVTestCase methodsFor: 'as yet unclassified' stamp: 'JJ 11/22/2005 16:37'!
setUp
	super setUp.
	configuration := DGVApplicationConfiguration instance.
	DGVApplicationConfiguration 
		setConfigurationFor: self configurationClass.
	self class setTestMode.
	DGVOSSupport makeReadyForTesting.
	DGVStorage makeReadyForTesting.
	oldHaltOnErrorsFlag := TDSDebuggerSupport haltOnErrors: false! !

!DGVTestCase methodsFor: 'as yet unclassified' stamp: 'CdG 11/15/2005 15:39'!
tearDown
	TDSDebuggerSupport haltOnErrors: oldHaltOnErrorsFlag.
	DGVOSSupport testingDone.
	DGVStorage testingDone.
	self class setNormalMode.
	DGVApplicationConfiguration setConfigurationFor: configuration class.
	super tearDown! !

!DGVTestCase methodsFor: 'as yet unclassified' stamp: 'CdG 11/15/2005 18:44'!
withMockSubclassOf: class instanceVariables: ivars methods: methods classMethods: classMethods do: aBlock
	| mock |
	mock := self makeMockSubclassOf: class instanceVariables: ivars methods: methods classMethods: classMethods.
	[aBlock value: mock]
		ensure: [class removeSubclass: mock]! !

DGVTestCase subclass: #DGVTextSearchInCommunityTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVTextSearchInCommunityTest methodsFor: 'as yet unclassified' stamp: 'JJ 11/18/2005 11:37'!
testFileSearch
	| com1 com2 com3 user1 file |
	com1 := DGVCommunity new: 'test foo'.
	com2 := DGVCommunity new: 'bar'.
	com3 := DGVCommunity new: 'baz'.
	com1 addChild: com2.
	com2 addChild: com3.
	user1 := DGVUser testUser.
	com1 acceptNewUser: user1.
	file := DGVFile new originalFilename: 'something.txt'.
	com3 addFile: file.

	self assert: (com1 textSearchWithChildren: 'something.txt') size = 1! !

!DGVTextSearchInCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:52'!
testRecursion
	| com1 com2 com3 user1 |
	com1 := DGVCommunity new: 'test foo'.
	com2 := DGVCommunity new: 'bar'.
	com3 := DGVCommunity new: 'baz'.
	com1 addChild: com2.
	com2 addChild: com3.
	user1 := DGVUser testUser.
	com3 acceptNewUser: user1.

	self assert: (com1 textSearchWithChildren: 'myname') size = 1! !

!DGVTextSearchInCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:11'!
testSimpleScoping
	| com1 com2 com3 |
	com1 := DGVCommunity new: 'test foo'.
	com2 := DGVCommunity new: 'bar'.
	com3 := DGVCommunity new: 'baz'.
	com1 addChild: com2.
	com1 save.
	com2 save.
	com3 save.
	self assert: (com1 textSearchWithChildren: 'foo') notEmpty.
	self assert: (com1 textSearchWithChildren: 'bar') notEmpty.
	self assert: (com1 textSearchWithChildren: 'baz') isEmpty! !

!DGVTextSearchInCommunityTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 7/6/2005 11:50'!
testTypes
	| com1 com2 file1 user1 |
	com1 := DGVCommunity new: 'test foo'.
	com2 := DGVCommunity new: 'bar'.
	com1 addChild: com2.
	file1 _ DGVFile new originalFilename: 'test'.
	com1 addFile: file1.
	user1 _ DGVUser testUser.
	com1 acceptNewUser: user1.

	self assert: (com1 textSearchWithChildren: 'test') size = 2.
	self assert: (com1 textSearchWithChildren: 'bar') size = 1.
	self assert: (com1 textSearchWithChildren: 'myname') size = 1.! !

DGVTestCase subclass: #DGVTextSearchTest
	instanceVariableNames: 'dataDirectory returnCount returnResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVTextSearchTest methodsFor: 'testobjects' stamp: 'EG,CdG 6/24/2005 11:45'!
makeTestCommunity1
	| answer |
	(answer := DGVCommunity new: 'test Foo word')
		description: 'bar en boos';
		save.
	^answer! !

!DGVTextSearchTest methodsFor: 'testobjects' stamp: 'EG,CdG 6/24/2005 11:45'!
makeTestCommunity2
	| answer |
	(answer := DGVCommunity new: 'test a word')
		description: 'bar foo bozer';
		save.
	^answer! !

!DGVTextSearchTest methodsFor: 'testobjects' stamp: 'EG,JJ 10/25/2005 14:35'!
makeTestFile1
	| answer |
	(answer := DGVFile new) originalFilename: 'Original.doc'.
	(answer metadata)
		description: 'this is the description';
		title: nil.	"ensure that nil fields don't kill a search"
	answer save.
	^answer! !

!DGVTextSearchTest methodsFor: 'testobjects' stamp: 'EG,JJ 10/25/2005 14:28'!
makeTestOAIRecord1
	| dc answer |
	answer := DGVOAIRecord new.
	dc := (DGVDublinCore new)
				publisher: 'jantje beton';
				description: 'plaatje van een collecterend kind'.
	answer
		metadata: dc;
		save.
	^answer! !

!DGVTextSearchTest methodsFor: 'testobjects' stamp: 'EG,CdG 6/24/2005 11:45'!
makeTestUser1
	| answer |
	(answer := DGVUser testUserWithName: 'jan' password: 'verysecret')
		fullName: 'Jantje Beton';
		email: 'jb@beton.org';
		save.
	^answer! !

!DGVTextSearchTest methodsFor: 'self-shunt' stamp: 'EG,CdG 6/24/2005 14:16'!
returnCount
	^returnCount! !

!DGVTextSearchTest methodsFor: 'self-shunt' stamp: 'EG,CdG 6/24/2005 14:23'!
returnResult
	^returnResult! !

!DGVTextSearchTest methodsFor: 'self-shunt' stamp: 'EG,CdG 6/24/2005 14:22'!
returnResult: aByteArray resultCount: aSmallInteger onPeer: aDGVMockNode 
	returnResult := aByteArray.
	returnCount := aSmallInteger! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 7/6/2005 11:41'!
testAbstract
	self assert: DGVObject isDGVAbstractClass.
	self deny: DGVFile isDGVAbstractClass.

	self assert: (DGVObject textSearch: 'foo') isEmpty.
! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/27/2005 11:17'!
testDescription
	| communitySearch |
	communitySearch _ (DGVTextSearch new) queryString: 'mies'.
	self assert: communitySearch description = 'mies'! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 10:31'!
testDGVBlob
	self assert: (DGVBlob textSearch: 'whatever') isEmpty! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'CdG 6/30/2005 17:45'!
testDGVCommunity
	| c1 c2 |
	self assert: DGVCommunity localStorage items isEmpty.
	self assert: (DGVCommunity textSearch: 'foo') isEmpty.
	c1 := self makeTestCommunity1.
	self assert: (DGVCommunity textSearch: 'Foo') size = 1.
	self assert: (DGVCommunity textSearch: 'Foo') first id = c1 id.
	self assert: (DGVCommunity textSearch: 'foo') size = 1.
	c2 := self makeTestCommunity2.
	self assert: (DGVCommunity textSearch: 'foo') size = 2.
	self assert: ((DGVCommunity textSearch: 'Foo') includes: c1).
	self assert: ((DGVCommunity textSearch: 'Foo') includes: c2).
	self assert: (DGVCommunity textSearch: 'bar bozer') size = 1.
	self assert: (DGVCommunity textSearch: 'bar bozer') first id = c2 id.
	self assert: (DGVCommunity textSearch: 'test word') size = 2! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 11:35'!
testDGVFile
	"ensure that nil fields don't kill a search"

	self assert: DGVFile localStorage items isEmpty.
	self assert: (DGVFile textSearch: 'jantje') isEmpty.
	self makeTestFile1.
	self assert: (DGVFile textSearch: 'Original.doc') size = 1.
	self assert: (DGVFile textSearch: 'description') size = 1! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 11:35'!
testDGVOAIRecord
	self assert: DGVOAIRecord localStorage items isEmpty.
	self assert: (DGVOAIRecord textSearch: 'plaatje') isEmpty.
	self makeTestOAIRecord1.
	self assert: (DGVOAIRecord textSearch: 'collecterend kind') size = 1! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 10:24'!
testDGVRelation
	self assert: (DGVRelation textSearch: 'whatever') isEmpty! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 10:24'!
testDGVService
	self assert: (DGVService textSearch: 'whatever') isEmpty! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/24/2005 11:35'!
testDGVUser
	| u1 |
	self assert: DGVUser localStorage items isEmpty.
	self assert: (DGVUser textSearch: 'jantje') isEmpty.
	u1 := self makeTestUser1.
	self assert: (DGVUser textSearch: 'jantje') size = 1.
	self assert: (DGVUser textSearch: 'jantje') first id = u1 id.
	self deny: (DGVUser textSearch: 'verysecret') size = 1.
	self assert: (DGVUser textSearch: 'jan') size = 1.
	self deny: (DGVUser textSearch: 'jb') size = 1.
	self assert: (DGVUser textSearch: 'jb@beton.org') size = 1! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,JJ 10/25/2005 14:35'!
testFilterResults
	| ts f1 u1 |
	self assert: (DGVObject textSearchAllClasses: 'foo') isEmpty.
	self makeTestCommunity1.
	self makeTestCommunity2.
      self makeTestOAIRecord1.
	f1 _ self makeTestFile1.
	u1 _ self makeTestUser1.
	f1 metadata publisher: u1.


	ts _ DGVTextSearch new.
	ts queryString: 'foo'.
	ts doLocalSearch.
	self assert: ts communities size = 2.
	self assert: ts files size = 0.
	self assert: ts users size = 0.
	self assert: ts records size = 0.

	ts _ DGVTextSearch new.
	ts queryString: 'jantje'.
	ts doLocalSearch.
	self assert: ts communities size = 0.
	self assert: ts files size = 1.
	self assert: ts users size = 1.
	self assert: ts records size = 1! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,CdG 6/27/2005 13:44'!
testNotifyingCollections
	| ts flag block flag2 |
	self assert: (DGVObject textSearchAllClasses: 'foo') isEmpty.
	self makeTestCommunity1.
	self makeTestCommunity2.
	self makeTestFile1.
      self makeTestOAIRecord1.
	self makeTestUser1.

	flag _ false.
	ts _ DGVTextSearch new.
	ts queryString: 'foo'.
	block _ [flag _ true].
	ts searchResults
		when: #activeItemsChanged
		send: #value
		to: block.
	ts doLocalSearch.

	self assert: flag.

	self assert: ts users isEmpty.
	flag2 _ false.
	block _ [flag2 _ true].

	ts users when: #activeItemsChanged send: #value to: block.

	ts signalMatch: self makeTestUser1.
	
	self assert: flag2.
! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,JJ 10/25/2005 15:13'!
testQuery
	| mockSearch result f1 u1 |
	self assert: (DGVObject textSearchAllClasses: 'foo') isEmpty.
	self makeTestCommunity1.
	f1 := self makeTestFile1.
	u1 := self makeTestUser1.
	f1 metadata publisher: u1.
	mockSearch := self.
	(DGVQueryTextSearch new)
		queryString: 'jantje';
		executeIn: mockSearch.
	self assert: mockSearch returnCount = 2.
	self 
		assert: (result := Object fromByteArray: mockSearch returnResult) notNil.
	self assert: result size = 2.
	self assert: (result includes: f1).
	self assert: (result includes: u1)! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,JJ 10/25/2005 14:36'!
testSearchThroughAllClasses
	| c1 c2 f1 r1 u1 results |
	self assert: (DGVObject textSearchAllClasses: 'foo') isEmpty.
	c1 _ self makeTestCommunity1.
	c2 _ self makeTestCommunity2.
	f1 _	self makeTestFile1.
      r1 _ self makeTestOAIRecord1.
	u1 _ self makeTestUser1.
	f1 metadata publisher: u1.

	self assert: (results _ DGVObject textSearchAllClasses: 'foo') size = 2.
	self assert: (results includes: c1).
	self assert: (results includes: c2).

	self assert: (results _ DGVObject textSearchAllClasses: 'jantje') size = 3.
	self assert: (results includes: f1).
	self assert: (results includes: r1).
	self assert: (results includes: u1).

	self assert: (DGVObject textSearchAllClasses: 'pangalacticgargleblaster') isEmpty
! !

!DGVTextSearchTest methodsFor: 'tests' stamp: 'EG,JJ 10/25/2005 14:36'!
testTextSearch
	| ts results block f1 u1 |
	ts _ DGVTextSearch new.
	ts queryString: 'jantje'.
	ts doLocalSearch.
	self assert: ts searchResults isEmpty.

	f1 _ self makeTestFile1.
	u1 _ self makeTestUser1.
	f1 metadata publisher: u1.

	results _ OrderedCollection new.
	block _ [:v | results add: v].
	ts newItemFoundBlock: block.

	ts doLocalSearch.
	self assert: ts searchResults size = 2.
	self assert: results size = 2.! !

DGVTestCase subclass: #DGVUserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DGV-Tests'!

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG 2/14/2005 16:12'!
testBeDefaultLocalUser
	FileDirectory deleteFilePath: DGVUser defaultLocalUserFileName.
	(DGVUser testUserWithName: 'frans' password: 'snarf') 
		save;
		beDefaultLocalUser.
	self assert: (DGVOSSupport default dataDirectory fileExists: 'defaultlocaluser')! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,TJ 3/24/2005 14:27'!
testChangeSavedUser
	| user |
	(user _ DGVUser testUserWithName: 'frans' password: 'snarf') 
		save.
	user _ DGVUser localStorage get: user id.
	self assert: user isValid not.
	self assert: user fullName isNil.
	user validatePassword: 'snarf'.
	user fullName: 'F. Hoving'.
	user save.
	user _ DGVUser localStorage get: user id.
	self assert: user fullName = 'F. Hoving'! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 10/13/2005 14:34'!
testCheckValidEmail
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.

	"good mail"
	self shouldnt: [user email: 'aap.noot.mies@aap.noot.mies'] raise: DGVInvalidEmailAddressException.
	self shouldnt: [user email: 'a@n.nl'] raise: DGVInvalidEmailAddressException.
	self shouldnt: [user email: 'bert-en-ernie@sesame-street.org'] raise: DGVInvalidEmailAddressException.
	self shouldnt: [user email: 'aap@noot.mies'] raise: DGVInvalidEmailAddressException.

	"bad mail"
	self should: [user email: nil] raise: DGVInvalidEmailAddressException.
	self should: [user email: ''] raise: DGVInvalidEmailAddressException.
	self should: [user email: 'aap'] raise: DGVInvalidEmailAddressException.
	
	
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'TJ 1/25/2005 15:15'!
testCipherSaber
	| cs text passkey ciphered |
	cs _ CipherSaber new.
	text _ 'Some text'.
	passkey _ 'My Secret Passphrase'.
	ciphered _ cs encipher: text withKey: passkey.
	self assert: 19 = ciphered size.
	self assert: text = (cs decipher: ciphered withKey: passkey). ! !

!DGVUserTest methodsFor: 'nil' stamp: 'EG,CdG 9/9/2005 11:03'!
testCleanEmergencyAnswer
	| u |
	u := DGVUser basicNew.

	self assert: (u cleanEmergencyAnswer: 'maria') = 'maria'.
	self assert: (u cleanEmergencyAnswer: 'Maria') = 'maria'.
	self assert: (u cleanEmergencyAnswer: 'm aria') = 'maria'.
	self assert: (u cleanEmergencyAnswer: 'mar-.ia') = 'maria'.
	self assert: (u cleanEmergencyAnswer: 'maria...') = 'maria'.
	self assert: (u cleanEmergencyAnswer: '..M=a()&*RiA...') = 'maria'.
	self assert: (u cleanEmergencyAnswer: '..M=a()&12*RiA...') = 'ma12ria'.
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG 9/30/2005 11:03'!
testCreation
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	self assert: 'frans' = user userName.
	self assert: user isValid.
	self assert: (user validatePassword: 'somethingelse') isNil.
	self assert: user isValid not.
	self assert: (user validatePassword: 'something') = user.
	self assert: user friends notNil.
	self assert: user documentsFolder notNil.
	self assert: user documentsFolder parent = user! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/9/2005 10:35'!
testCreationWithSecretQuestion
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	user secretQuestion: 1 answer: 'maria'.
	user privateKey: nil.

	self deny: (user isValid).

	user recoverKeyWithSecretQuestionAnswer: 'maria' newPassword: 'racefiets'.
	self assert: (user isValid).
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG 11/15/2005 18:45'!
testDeletion
	| user answer  |
	self withMockSubclassOf: DGVUser 
		instanceVariables: '' 
		methods: #()
		classMethods: {
			'userByNameEmailQueryClass ^self'.
			'name: foo email: bar ^self testUserWithName: ''IdoNotExist'' password: ''42'' '}
		do: [:mock |

			user := mock testUserWithName: 'frans' password: 'something'.
			user delete; save.
	
			"check we short-circuit on deleted users"
			answer := mock login: 'frans' password: 'something'.
			self assert: answer isNil.
		
			"check we go out on the network if it's not there locally"
			answer := mock login: 'IdoNotExist' password: '42'.
			self assert: answer class = mock]! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG 6/7/2005 11:30'!
testDSA
	| dsa keys pri pub sig result pri2 |
	dsa _ DigitalSignatureAlgorithm new.
	dsa initRandomFromString: 'foo bar'.
	keys _ DigitalSignatureAlgorithm testKeySet.
	pri _ keys first.
	pub _ keys second.
	sig _ DigitalSignatureAlgorithm sign: 'testpassword' privateKey: pri dsa: dsa.
	result _ DigitalSignatureAlgorithm verify: sig isSignatureOf: 'testpassword' publicKey: pub.
	self assert: result.

	pri2_ Array with: pub first with: pub second with: pub third with: pri fourth.
	self assert: pri2 = pri ! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG 8/12/2005 14:06'!
testEditableString
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	self assert: user editableString = 'frans'! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 8/18/2005 11:36'!
testHtmlRepresentation
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	user email: 'foo@bar.com'.
	user fullName: 'Frans Hoving'.
	self assert: user htmlRepresentation = '<a href="mailto:foo@bar.com">Frans Hoving</a>'! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG 12/15/2005 17:20'!
testIsManagerOfCommunity
	| user community |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	community := DGVCommunity new: 'my community'.
	self deny: (user isManagerOfCommunity: community).
	community addManager: user.
	self assert: (user isManagerOfCommunity: community).

	self deny: (user isManagerOfCommunity: nil).
	self deny: (user isManagerOfCommunity: (DGVLink class: DGVObject id: 1))! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG 10/26/2005 15:29'!
testLoginCount
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	user save.
	self assert: user loginCount = 0.

	DGVUser login: 'frans' password: 'something'.
	user := DGVUser byName: 'frans'.
	self assert: user loginCount = 1.
	
	DGVUser login: 'frans' password: 'something'.
	user := DGVUser byName: 'frans'.
	self assert: user loginCount = 2.

	[DGVUser login: 'frans' password: 'somethingWRONG!!!!!!']
		on: Error do: [].
	user := DGVUser byName: 'frans'.
	self assert: user loginCount = 2.
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/9/2005 10:55'!
testNewPasswordViaSecretQuestion
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	user secretQuestion: 1 answer: 'maria'.
	user recoverKeyWithSecretQuestionAnswer: 'maria' newPassword: 'racefiets'.
	self assert: user isValid.

	user logout.
	user validatePassword: 'something'.
	self deny: user isValid.

	user logout.
	user validatePassword: 'racefiets'.
	self assert: user isValid.
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/9/2005 10:55'!
testNewPasswordViaSecretQuestionWithInvalidPassword
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	user secretQuestion: 1 answer: 'maria'.
	user recoverKeyWithSecretQuestionAnswer: 'm@ria' newPassword: 'racefiets'.

	self deny: user isValid.
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'EG,CdG 9/9/2005 10:56'!
testNewPasswordViaSecretQuestionWithValidMungedPassword
	| user |
	user := DGVUser testUserWithName: 'frans' password: 'something'.
	user secretQuestion: 1 answer: 'maR-ia'.

	user recoverKeyWithSecretQuestionAnswer: 'mAria' newPassword: 'something'.
	self assert: user isValid.

	user recoverKeyWithSecretQuestionAnswer: 'mA ri-a' newPassword: 'something'.
	self assert: user isValid.
! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'TJ 2/16/2005 20:27'!
testReadFromFile
	| user |
	(DGVUser testUserWithName: 'frans' password: 'snarf') 
		save;
		beDefaultLocalUser.
	user _ DGVUser getDefaultLocalUser.
	self assert: user isValid not! !

!DGVUserTest methodsFor: 'as yet unclassified' stamp: 'CdG,TJ 2/3/2005 12:11'!
testReadFromFileNotAvailable
	FileDirectory deleteFilePath: DGVUser defaultLocalUserFileName.
	self assert: DGVUser getDefaultLocalUser isNil! !

!Array methodsFor: '*dgv-model' stamp: 'CdG 6/28/2005 14:43'!
srpSavePreMap: aContext
	"if we're the root object, put ourselves in. This signals to DGVProtoObject that if it is one of our elements, it needs to 
	serialize itself, not itself as a link. This helps in streamlining returning search results (which are arrays of dgvobjects)"
	aContext userData at: #root ifAbsentPut: self.
	^self! !

!Integer methodsFor: '*dgv-base extensions' stamp: 'EG,CdG 4/12/2005 11:56'!
printKiMiGiStringOn: aStream
	| value suffix |
	self > (1024*1024*1024) 
		ifTrue: [
			value _ self / (1024*1024*1024).
			suffix _ 'G']
		ifFalse: [self > (1024*1024) 
			ifTrue: [
				value _ self / (1024*1024).
				suffix _ 'M']
			ifFalse: [self > 1024
				ifTrue: [
					value _ self / 1024.
					suffix _ 'k']
				ifFalse: [
					^self printOn: aStream]]].
	value _ ScaledDecimal newFromNumber: value scale: 1.
	value _ value printString.
	value _ value copyFrom: 1 to: (value size - 2).
	aStream 
		nextPutAll: value;
		nextPutAll: suffix! !

!TranslatedReceiverFinder methodsFor: '*dgv-development stuff' stamp: 'CdG 10/3/2005 21:34'!
dgvSenders
	"self new dgvSenders"
	| m o |
	m _ SystemNavigation default allCallsOn: #translated.
	m _ m select: [:e |
		e classSymbol beginsWith: 'DGV'].
	m _ m collect: [:e |
		e classIsMeta ifTrue: [
			(Smalltalk at: e classSymbol) class decompile: e methodSymbol.
		] ifFalse: [
			(Smalltalk at: e classSymbol) decompile: e methodSymbol.
		]
	].

	o _ SortedCollection new.
	m do: [:e | self searchMethodNode: e addTo: o].
	^ o.
! !
DGVConfiguration initialize!
DGVCollection initialize!
DGVQueryGetBlobBlock initialize!
